@@ -819,9 +819,15 @@ static int stabs_pts_read_type_def(struct ParseTypedefData* ptd, const char* typ
 	    PTS_ABORTIF(ptd, stabs_pts_read_array(ptd, &new_dt) == -1);
 	    break;
 	case 'r':
-	    PTS_ABORTIF(ptd, stabs_pts_read_range(ptd, typename, &new_dt) == -1);
-	    assert(!*stabs_find_ref(filenr1, subnr1));
-	    *stabs_find_ref(filenr1, subnr1) = new_dt;
+            {
+                struct symt**       prev_dt;
+                PTS_ABORTIF(ptd, stabs_pts_read_range(ptd, typename, &new_dt) == -1);
+
+                prev_dt = stabs_find_ref(filenr1, subnr1);
+                /* allow redefining with same base type */
+                if (*prev_dt && *prev_dt != new_dt) WARN("Multiple range def in %ls\n", ptd->module->module.ModuleName);
+                else *prev_dt = new_dt;
+            }
 	    break;
 	case 'f':
 	    PTS_ABORTIF(ptd, stabs_pts_read_type_def(ptd, NULL, &ref_dt) == -1);
@@ -1166,9 +1172,9 @@ static void pending_flush(struct pending_list* pending, struct module* module,
             break;
         case PENDING_LINE:
             if (module->type == DMT_MACHO)
-                pending->objs[i].u.line.offset -= func->address - pending->objs[i].u.line.load_offset;
+                pending->objs[i].u.line.offset -= func->ranges[0].low - pending->objs[i].u.line.load_offset;
             symt_add_func_line(module, func, pending->objs[i].u.line.source_idx,
-                               pending->objs[i].u.line.line_num, func->address + pending->objs[i].u.line.offset);
+                               pending->objs[i].u.line.line_num, func->ranges[0].low + pending->objs[i].u.line.offset);
             break;
         default:
             ERR("Unknown pending object tag %u\n", (unsigned)pending->objs[i].tag);
@@ -1199,15 +1205,15 @@ static void stabs_finalize_function(struct module* module, struct symt_function*
      * Not 100% bullet proof, but better than nothing
      */
     il.SizeOfStruct = sizeof(il);
-    if (SymGetLineFromAddr64(module->process->handle, func->address, &disp, &il) &&
+    if (SymGetLineFromAddr64(module->process->handle, func->ranges[0].low, &disp, &il) &&
         SymGetLineNext64(module->process->handle, &il))
     {
         loc.kind = loc_absolute;
-        loc.offset = il.Address - func->address;
+        loc.offset = il.Address - func->ranges[0].low;
         symt_add_function_point(module, func, SymTagFuncDebugStart, 
                                 &loc, NULL);
     }
-    if (size) func->size = size;
+    if (size) func->ranges[0].high = func->ranges[0].low + size;
 }
 
 static inline void stabbuf_append(char **buf, unsigned *buf_size, const char *str)
@@ -1374,15 +1380,18 @@ BOOL stabs_parse(struct module* module, ULONG_PTR load_offset,
         case N_LBRAC:
             if (curr_func)
             {
-                block = symt_open_func_block(module, curr_func, block,
-                                             n_value, 0);
+                block = symt_open_func_block(module, curr_func, block, 1);
+                block->ranges[0].low = curr_func->ranges[0].low + n_value;
+                block->ranges[0].high = 0; /* will be set by N_RBRAC */
                 pending_flush(&pending_block, module, curr_func, block);
             }
             break;
         case N_RBRAC:
             if (curr_func)
-                block = symt_close_func_block(module, curr_func, block,
-                                              n_value);
+            {
+                block->ranges[0].high = curr_func->ranges[0].low + n_value;
+                block = symt_close_func_block(module, curr_func, block);
+            }
             break;
         case N_PSYM:
             /* These are function parameters. */
@@ -1490,9 +1499,9 @@ BOOL stabs_parse(struct module* module, ULONG_PTR load_offset,
             {
                 ULONG_PTR offset = n_value;
                 if (module->type == DMT_MACHO)
-                    offset -= curr_func->address - load_offset;
+                    offset -= curr_func->ranges[0].low - load_offset;
                 symt_add_func_line(module, curr_func, source_idx, 
-                                   stab_ptr->n_desc, curr_func->address + offset);
+                                   stab_ptr->n_desc, curr_func->ranges[0].low + offset);
             }
             else pending_add_line(&pending_func, source_idx, stab_ptr->n_desc,
                                   n_value, load_offset);
@@ -1528,7 +1537,7 @@ BOOL stabs_parse(struct module* module, ULONG_PTR load_offset,
                      */
                     stabs_finalize_function(module, curr_func, 
                                             n_value ?
-                                                (load_offset + n_value - curr_func->address) : 0);
+                                                (load_offset + n_value - curr_func->ranges[0].low) : 0);
                 }
                 func_type = symt_new_function_signature(module, 
                                                         stabs_parse_type(ptr), -1);
@@ -1570,7 +1579,7 @@ BOOL stabs_parse(struct module* module, ULONG_PTR load_offset,
                 {
                     stabs_reset_includes();
                     source_idx = source_new(module, srcpath, ptr);
-                    compiland = symt_new_compiland(module, 0 /* FIXME */, source_idx);
+                    compiland = symt_new_compiland(module, source_idx);
                 }
                 else
                 {
@@ -1661,7 +1670,7 @@ BOOL stabs_parse(struct module* module, ULONG_PTR load_offset,
               stab_ptr->n_type, (ULONG_PTR)n_value, debugstr_a(strs + stab_ptr->n_strx));
     }
     module->module.SymType = SymDia;
-    module->module.CVSig = 'S' | ('T' << 8) | ('A' << 16) | ('B' << 24);
+    module->debug_format_bitmask |= DHEXT_FORMAT_STABS;
     /* FIXME: we could have a finer grain here */
     module->module.LineNumbers = TRUE;
     module->module.GlobalSymbols = TRUE;
