@@ -95,7 +95,7 @@ static ULONG WINAPI d3d_vertex_buffer7_Release(IDirect3DVertexBuffer7 *iface)
         if (buffer->version == 7)
             IDirectDraw7_Release(&buffer->ddraw->IDirectDraw7_iface);
 
-        heap_free(buffer);
+        free(buffer);
     }
 
     return ref;
@@ -111,11 +111,11 @@ static HRESULT d3d_vertex_buffer_create_wined3d_buffer(struct d3d_vertex_buffer
     struct wined3d_buffer_desc desc;
 
     desc.byte_width = buffer->size;
-    desc.usage = WINED3DUSAGE_STATICDECL;
+    desc.usage = WINED3DUSAGE_STATICDECL | WINED3DUSAGE_VIDMEM_ACCOUNTING;
     if (dynamic)
         desc.usage |= WINED3DUSAGE_DYNAMIC;
     desc.bind_flags = WINED3D_BIND_VERTEX_BUFFER;
-    if (buffer->Caps & D3DVBCAPS_SYSTEMMEMORY)
+    if (buffer->sysmem)
         desc.access = WINED3D_RESOURCE_ACCESS_CPU | WINED3D_RESOURCE_ACCESS_MAP_R | WINED3D_RESOURCE_ACCESS_MAP_W;
     else
         desc.access = WINED3D_RESOURCE_ACCESS_GPU | WINED3D_RESOURCE_ACCESS_MAP_R | WINED3D_RESOURCE_ACCESS_MAP_W;
@@ -448,7 +448,7 @@ HRESULT d3d_vertex_buffer_create(struct d3d_vertex_buffer **vertex_buf,
     TRACE("    FVF %#lx\n", desc->dwFVF);
     TRACE("    dwNumVertices %lu\n", desc->dwNumVertices);
 
-    if (!(buffer = heap_alloc_zero(sizeof(*buffer))))
+    if (!(buffer = calloc(1, sizeof(*buffer))))
         return DDERR_OUTOFMEMORY;
 
     buffer->IDirect3DVertexBuffer7_iface.lpVtbl = &d3d_vertex_buffer7_vtbl;
@@ -461,6 +461,19 @@ HRESULT d3d_vertex_buffer_create(struct d3d_vertex_buffer **vertex_buf,
     buffer->fvf = desc->dwFVF;
     buffer->size = get_flexible_vertex_size(desc->dwFVF) * desc->dwNumVertices;
 
+    /* ddraw4 vertex buffers ignore DISCARD and NOOVERWRITE, even on
+     * pretransformed geometry, which means that a GPU-based buffer cannot
+     * perform well.
+     *
+     * While at least one contemporaneous card (Geforce 4) does seem to show a
+     * difference in its performance characteristics based on whether
+     * D3DVBCAPS_SYSTEMMEMORY is set, it also doesn't *improve* performance to
+     * use a non-SYSTEMMEMORY buffer with ddraw4. For wined3d it should always
+     * be better to use sysmem.
+     *
+     * This improves performance in Prince of Persia 3D. */
+    buffer->sysmem = ((buffer->Caps & D3DVBCAPS_SYSTEMMEMORY) || buffer->version < 7);
+
     wined3d_mutex_lock();
 
     if (FAILED(hr = d3d_vertex_buffer_create_wined3d_buffer(buffer, FALSE, &buffer->wined3d_buffer)))
@@ -485,7 +498,7 @@ end:
     if (hr == D3D_OK)
         *vertex_buf = buffer;
     else
-        heap_free(buffer);
+        free(buffer);
 
     return hr;
 }
