@@ -20,8 +20,6 @@
 #include <string.h>
 
 #define COBJMACROS
-#define NONAMELESSUNION
-
 #include "windef.h"
 #include "winbase.h"
 #include "winerror.h"
@@ -38,7 +36,7 @@ WINE_DEFAULT_DEBUG_CHANNEL(dplay);
 
 /* Forward declarations for this module helper methods */
 HRESULT DPL_CreateCompoundAddress ( LPCDPCOMPOUNDADDRESSELEMENT lpElements, DWORD dwElementCount,
-                                    LPVOID lpAddress, LPDWORD lpdwAddressSize, BOOL bAnsiInterface )DECLSPEC_HIDDEN;
+                                    LPVOID lpAddress, LPDWORD lpdwAddressSize, BOOL bAnsiInterface );
 
 static HRESULT DPL_CreateAddress( REFGUID guidSP, REFGUID guidDataType, LPCVOID lpData, DWORD dwDataSize,
                            LPVOID lpAddress, LPDWORD lpdwAddressSize, BOOL bAnsiInterface );
@@ -117,7 +115,7 @@ static void dplobby_destroy(IDirectPlayLobbyImpl *obj)
     DPQ_DELETEQ( obj->msgs, msgs, LPDPLMSG, cbDeleteElemFromHeap );
     obj->lock.DebugInfo->Spare[0] = 0;
     DeleteCriticalSection( &obj->lock );
-    HeapFree( GetProcessHeap(), 0, obj );
+    free( obj );
 }
 
 static HRESULT WINAPI IDirectPlayLobbyAImpl_QueryInterface( IDirectPlayLobbyA *iface, REFIID riid,
@@ -211,7 +209,7 @@ static ULONG WINAPI IDirectPlayLobbyAImpl_AddRef( IDirectPlayLobbyA *iface )
     IDirectPlayLobbyImpl *This = impl_from_IDirectPlayLobbyA( iface );
     ULONG ref = InterlockedIncrement( &This->refA );
 
-    TRACE( "(%p) refA=%d\n", This, ref );
+    TRACE( "(%p) refA=%ld\n", This, ref );
 
     if ( ref == 1 )
         InterlockedIncrement( &This->numIfaces );
@@ -224,7 +222,7 @@ static ULONG WINAPI IDirectPlayLobbyImpl_AddRef( IDirectPlayLobby *iface )
     IDirectPlayLobbyImpl *This = impl_from_IDirectPlayLobby( iface );
     ULONG ref = InterlockedIncrement( &This->ref );
 
-    TRACE( "(%p) ref=%d\n", This, ref );
+    TRACE( "(%p) ref=%ld\n", This, ref );
 
     if ( ref == 1 )
         InterlockedIncrement( &This->numIfaces );
@@ -237,7 +235,7 @@ static ULONG WINAPI IDirectPlayLobby2AImpl_AddRef(IDirectPlayLobby2A *iface)
     IDirectPlayLobbyImpl *This = impl_from_IDirectPlayLobby2A( iface );
     ULONG ref = InterlockedIncrement( &This->ref2A );
 
-    TRACE( "(%p) ref2A=%d\n", This, ref );
+    TRACE( "(%p) ref2A=%ld\n", This, ref );
 
     if ( ref == 1 )
         InterlockedIncrement( &This->numIfaces );
@@ -250,7 +248,7 @@ static ULONG WINAPI IDirectPlayLobby2Impl_AddRef(IDirectPlayLobby2 *iface)
     IDirectPlayLobbyImpl *This = impl_from_IDirectPlayLobby2( iface );
     ULONG ref = InterlockedIncrement( &This->ref2 );
 
-    TRACE( "(%p) ref2=%d\n", This, ref );
+    TRACE( "(%p) ref2=%ld\n", This, ref );
 
     if ( ref == 1 )
         InterlockedIncrement( &This->numIfaces );
@@ -263,7 +261,7 @@ static ULONG WINAPI IDirectPlayLobby3AImpl_AddRef(IDirectPlayLobby3A *iface)
     IDirectPlayLobbyImpl *This = impl_from_IDirectPlayLobby3A( iface );
     ULONG ref = InterlockedIncrement( &This->ref3A );
 
-    TRACE( "(%p) ref3A=%d\n", This, ref );
+    TRACE( "(%p) ref3A=%ld\n", This, ref );
 
     if ( ref == 1 )
         InterlockedIncrement( &This->numIfaces );
@@ -276,7 +274,7 @@ static ULONG WINAPI IDirectPlayLobby3Impl_AddRef(IDirectPlayLobby3 *iface)
     IDirectPlayLobbyImpl *This = impl_from_IDirectPlayLobby3( iface );
     ULONG ref = InterlockedIncrement( &This->ref3 );
 
-    TRACE( "(%p) ref3=%d\n", This, ref );
+    TRACE( "(%p) ref3=%ld\n", This, ref );
 
     if ( ref == 1 )
         InterlockedIncrement( &This->numIfaces );
@@ -289,7 +287,7 @@ static ULONG WINAPI IDirectPlayLobbyAImpl_Release( IDirectPlayLobbyA *iface )
     IDirectPlayLobbyImpl *This = impl_from_IDirectPlayLobbyA( iface );
     ULONG ref = InterlockedDecrement( &This->refA );
 
-    TRACE( "(%p) refA=%d\n", This, ref );
+    TRACE( "(%p) refA=%ld\n", This, ref );
 
     if ( !ref && !InterlockedDecrement( &This->numIfaces ) )
         dplobby_destroy( This );
@@ -302,7 +300,7 @@ static ULONG WINAPI IDirectPlayLobbyImpl_Release( IDirectPlayLobby *iface )
     IDirectPlayLobbyImpl *This = impl_from_IDirectPlayLobby( iface );
     ULONG ref = InterlockedDecrement( &This->ref );
 
-    TRACE( "(%p) ref=%d\n", This, ref );
+    TRACE( "(%p) ref=%ld\n", This, ref );
 
     if ( !ref && !InterlockedDecrement( &This->numIfaces ) )
         dplobby_destroy( This );
@@ -315,7 +313,7 @@ static ULONG WINAPI IDirectPlayLobby2AImpl_Release(IDirectPlayLobby2A *iface)
     IDirectPlayLobbyImpl *This = impl_from_IDirectPlayLobby2A( iface );
     ULONG ref = InterlockedDecrement( &This->ref2A );
 
-    TRACE( "(%p) ref2A=%d\n", This, ref );
+    TRACE( "(%p) ref2A=%ld\n", This, ref );
 
     if ( !ref && !InterlockedDecrement( &This->numIfaces ) )
         dplobby_destroy( This );
@@ -328,7 +326,7 @@ static ULONG WINAPI IDirectPlayLobby2Impl_Release(IDirectPlayLobby2 *iface)
     IDirectPlayLobbyImpl *This = impl_from_IDirectPlayLobby2( iface );
     ULONG ref = InterlockedDecrement( &This->ref2 );
 
-    TRACE( "(%p) ref2=%d\n", This, ref );
+    TRACE( "(%p) ref2=%ld\n", This, ref );
 
     if ( !ref && !InterlockedDecrement( &This->numIfaces ) )
         dplobby_destroy( This );
@@ -341,7 +339,7 @@ static ULONG WINAPI IDirectPlayLobby3AImpl_Release(IDirectPlayLobby3A *iface)
     IDirectPlayLobbyImpl *This = impl_from_IDirectPlayLobby3A( iface );
     ULONG ref = InterlockedDecrement( &This->ref3A );
 
-    TRACE( "(%p) ref3A=%d\n", This, ref );
+    TRACE( "(%p) ref3A=%ld\n", This, ref );
 
     if ( !ref && !InterlockedDecrement( &This->numIfaces ) )
         dplobby_destroy( This );
@@ -354,7 +352,7 @@ static ULONG WINAPI IDirectPlayLobby3Impl_Release(IDirectPlayLobby3 *iface)
     IDirectPlayLobbyImpl *This = impl_from_IDirectPlayLobby3( iface );
     ULONG ref = InterlockedDecrement( &This->ref3 );
 
-    TRACE( "(%p) ref3=%d\n", This, ref );
+    TRACE( "(%p) ref3=%ld\n", This, ref );
 
     if ( !ref && !InterlockedDecrement( &This->numIfaces ) )
         dplobby_destroy( This );
@@ -379,7 +377,7 @@ static HRESULT DPL_ConnectEx( IDirectPlayLobbyImpl *This, DWORD dwFlags, REFIID
   DWORD           dwConnSize = 0;
   LPDPLCONNECTION lpConn;
 
-  FIXME("(%p)->(0x%08x,%p,%p): semi stub\n", This, dwFlags, lplpDP, pUnk );
+  FIXME("(%p)->(0x%08lx,%p,%p): semi stub\n", This, dwFlags, lplpDP, pUnk );
 
   if( pUnk )
   {
@@ -407,7 +405,7 @@ static HRESULT DPL_ConnectEx( IDirectPlayLobbyImpl *This, DWORD dwFlags, REFIID
     return hr;
   }
 
-  lpConn = HeapAlloc( GetProcessHeap(), HEAP_ZERO_MEMORY, dwConnSize );
+  lpConn = calloc( 1, dwConnSize );
 
   if( lpConn == NULL )
   {
@@ -419,7 +417,7 @@ static HRESULT DPL_ConnectEx( IDirectPlayLobbyImpl *This, DWORD dwFlags, REFIID
                                                0, lpConn, &dwConnSize );
   if( FAILED( hr ) )
   {
-    HeapFree( GetProcessHeap(), 0, lpConn );
+    free( lpConn );
     return hr;
   }
 
@@ -444,7 +442,7 @@ static HRESULT DPL_ConnectEx( IDirectPlayLobbyImpl *This, DWORD dwFlags, REFIID
   hr = IDirectPlayX_Open( (*(LPDIRECTPLAY2*)lplpDP), lpConn->lpSessionDesc,
                           dwOpenFlags );
 
-  HeapFree( GetProcessHeap(), 0, lpConn );
+  free( lpConn );
 
   return hr;
 }
@@ -560,7 +558,7 @@ static HRESULT DPL_CreateAddress(
   const DWORD dwNumAddElements = 2; /* Service Provide & address data type */
   DPCOMPOUNDADDRESSELEMENT addressElements[ 2 /* dwNumAddElements */ ];
 
-  TRACE( "(%p)->(%p,%p,0x%08x,%p,%p,%d)\n", guidSP, guidDataType, lpData, dwDataSize,
+  TRACE( "(%p)->(%p,%p,0x%08lx,%p,%p,%d)\n", guidSP, guidDataType, lpData, dwDataSize,
                                              lpAddress, lpdwAddressSize, bAnsiInterface );
 
   addressElements[ 0 ].guidDataType = DPAID_ServiceProvider;
@@ -623,7 +621,7 @@ static HRESULT WINAPI IDirectPlayLobby3AImpl_EnumAddress( IDirectPlayLobby3A *if
 {
   IDirectPlayLobbyImpl *This = impl_from_IDirectPlayLobby3A( iface );
 
-  TRACE("(%p)->(%p,%p,0x%08x,%p)\n", This, lpEnumAddressCallback, lpAddress,
+  TRACE("(%p)->(%p,%p,0x%08lx,%p)\n", This, lpEnumAddressCallback, lpAddress,
                                       dwAddressSize, lpContext );
 
   return DPL_EnumAddress( lpEnumAddressCallback, lpAddress, dwAddressSize, lpContext );
@@ -635,7 +633,7 @@ static HRESULT WINAPI IDirectPlayLobby3Impl_EnumAddress( IDirectPlayLobby3 *ifac
 {
   IDirectPlayLobbyImpl *This = impl_from_IDirectPlayLobby3( iface );
 
-  TRACE("(%p)->(%p,%p,0x%08x,%p)\n", This, lpEnumAddressCallback, lpAddress,
+  TRACE("(%p)->(%p,%p,0x%08lx,%p)\n", This, lpEnumAddressCallback, lpAddress,
                                       dwAddressSize, lpContext );
 
   return DPL_EnumAddress( lpEnumAddressCallback, lpAddress, dwAddressSize, lpContext );
@@ -720,7 +718,7 @@ static HRESULT WINAPI IDirectPlayLobby3AImpl_EnumAddressTypes( IDirectPlayLobby3
   char   subKeyName[51];
   FILETIME filetime;
 
-  TRACE(" (%p)->(%p,%p,%p,0x%08x)\n", This, lpEnumAddressTypeCallback, guidSP, lpContext, dwFlags );
+  TRACE(" (%p)->(%p,%p,%p,0x%08lx)\n", This, lpEnumAddressTypeCallback, guidSP, lpContext, dwFlags );
 
   if( dwFlags != 0 )
   {
@@ -782,7 +780,7 @@ static HRESULT WINAPI IDirectPlayLobby3AImpl_EnumAddressTypes( IDirectPlayLobby3
       }
 
       /* FIXME: Check return types to ensure we're interpreting data right */
-      MultiByteToWideChar( CP_ACP, 0, returnBuffer, -1, buff, sizeof(buff)/sizeof(WCHAR) );
+      MultiByteToWideChar( CP_ACP, 0, returnBuffer, -1, buff, ARRAY_SIZE( buff ));
       CLSIDFromString( buff, &serviceProviderGUID );
       /* FIXME: Have I got a memory leak on the serviceProviderGUID? */
 
@@ -809,7 +807,7 @@ static HRESULT WINAPI IDirectPlayLobby3AImpl_EnumAddressTypes( IDirectPlayLobby3
         TRACE( "Found Address Type GUID %s\n", atSubKey );
 
         /* FIXME: Check return types to ensure we're interpreting data right */
-        MultiByteToWideChar( CP_ACP, 0, atSubKey, -1, buff, sizeof(buff)/sizeof(WCHAR) );
+        MultiByteToWideChar( CP_ACP, 0, atSubKey, -1, buff, ARRAY_SIZE( buff ));
         CLSIDFromString( buff, &serviceProviderGUID );
         /* FIXME: Have I got a memory leak on the serviceProviderGUID? */
 
@@ -832,8 +830,9 @@ static HRESULT WINAPI IDirectPlayLobby3AImpl_EnumAddressTypes( IDirectPlayLobby3
 static HRESULT WINAPI IDirectPlayLobby3Impl_EnumAddressTypes( IDirectPlayLobby3 *iface,
         LPDPLENUMADDRESSTYPESCALLBACK enumaddrtypecb, REFGUID sp, void *context, DWORD flags )
 {
-  FIXME(":stub\n");
-  return DPERR_OUTOFMEMORY;
+    IDirectPlayLobbyImpl *This = impl_from_IDirectPlayLobby3( iface );
+    return IDirectPlayLobby_EnumAddressTypes( &This->IDirectPlayLobby3A_iface, enumaddrtypecb, sp,
+          context, flags );
 }
 
 /********************************************************************
@@ -847,7 +846,7 @@ static HRESULT WINAPI IDirectPlayLobby3Impl_EnumLocalApplications( IDirectPlayLo
 {
   IDirectPlayLobbyImpl *This = impl_from_IDirectPlayLobby3( iface );
 
-  FIXME("(%p)->(%p,%p,0x%08x):stub\n", This, lpEnumLocalAppCallback, lpContext, dwFlags );
+  FIXME("(%p)->(%p,%p,0x%08lx):stub\n", This, lpEnumLocalAppCallback, lpContext, dwFlags );
 
   return DPERR_OUTOFMEMORY;
 }
@@ -896,7 +895,7 @@ static HRESULT WINAPI IDirectPlayLobby3AImpl_EnumLocalApplications( IDirectPlayL
   char subKeyName[51];
   FILETIME filetime;
 
-  TRACE("(%p)->(%p,%p,0x%08x)\n", This, lpEnumLocalAppCallback, lpContext, dwFlags );
+  TRACE("(%p)->(%p,%p,0x%08lx)\n", This, lpEnumLocalAppCallback, lpContext, dwFlags );
 
   if( dwFlags != 0 )
   {
@@ -949,13 +948,13 @@ static HRESULT WINAPI IDirectPlayLobby3AImpl_EnumLocalApplications( IDirectPlayL
     }
 
     /* FIXME: Check return types to ensure we're interpreting data right */
-    MultiByteToWideChar( CP_ACP, 0, returnBuffer, -1, buff, sizeof(buff)/sizeof(WCHAR) );
+    MultiByteToWideChar( CP_ACP, 0, returnBuffer, -1, buff, ARRAY_SIZE( buff ));
     CLSIDFromString( buff, &serviceProviderGUID );
     /* FIXME: Have I got a memory leak on the serviceProviderGUID? */
 
-    dplAppInfo.dwSize               = sizeof( dplAppInfo );
-    dplAppInfo.guidApplication      = serviceProviderGUID;
-    dplAppInfo.u.lpszAppNameA = subKeyName;
+    dplAppInfo.dwSize          = sizeof( dplAppInfo );
+    dplAppInfo.guidApplication = serviceProviderGUID;
+    dplAppInfo.lpszAppNameA    = subKeyName;
 
     EnterCriticalSection( &This->lock );
 
@@ -1021,7 +1020,7 @@ static HRESULT WINAPI IDirectPlayLobby3AImpl_GetConnectionSettings( IDirectPlayL
   IDirectPlayLobbyImpl *This = impl_from_IDirectPlayLobby3A( iface );
   HRESULT hr;
 
-  TRACE("(%p)->(0x%08x,%p,%p)\n", This, dwAppID, lpData, lpdwDataSize );
+  TRACE("(%p)->(0x%08lx,%p,%p)\n", This, dwAppID, lpData, lpdwDataSize );
 
   EnterCriticalSection( &This->lock );
 
@@ -1041,7 +1040,7 @@ static HRESULT WINAPI IDirectPlayLobby3Impl_GetConnectionSettings( IDirectPlayLo
   IDirectPlayLobbyImpl *This = impl_from_IDirectPlayLobby3( iface );
   HRESULT hr;
 
-  TRACE("(%p)->(0x%08x,%p,%p)\n", This, dwAppID, lpData, lpdwDataSize );
+  TRACE("(%p)->(0x%08lx,%p,%p)\n", This, dwAppID, lpData, lpdwDataSize );
 
   EnterCriticalSection( &This->lock );
 
@@ -1098,7 +1097,7 @@ static HRESULT WINAPI IDirectPlayLobby3AImpl_ReceiveLobbyMessage( IDirectPlayLob
         DWORD *lpdwDataSize )
 {
   IDirectPlayLobbyImpl *This = impl_from_IDirectPlayLobby3A( iface );
-  FIXME(":stub %p %08x %08x %p %p %p\n", This, dwFlags, dwAppID, lpdwMessageFlags, lpData,
+  FIXME(":stub %p %08lx %08lx %p %p %p\n", This, dwFlags, dwAppID, lpdwMessageFlags, lpData,
          lpdwDataSize );
   return DPERR_OUTOFMEMORY;
 }
@@ -1108,7 +1107,7 @@ static HRESULT WINAPI IDirectPlayLobby3Impl_ReceiveLobbyMessage( IDirectPlayLobb
         DWORD *lpdwDataSize )
 {
   IDirectPlayLobbyImpl *This = impl_from_IDirectPlayLobby3( iface );
-  FIXME(":stub %p %08x %08x %p %p %p\n", This, dwFlags, dwAppID, lpdwMessageFlags, lpData,
+  FIXME(":stub %p %08lx %08lx %p %p %p\n", This, dwFlags, dwAppID, lpdwMessageFlags, lpData,
          lpdwDataSize );
   return DPERR_OUTOFMEMORY;
 }
@@ -1154,8 +1153,7 @@ static BOOL CALLBACK RunApplicationA_EnumLocalApplications
     }
     else
     {
-        if ((lpData->lpszCommandLine = HeapAlloc( GetProcessHeap(), 0, strlen(returnBuffer)+1 )))
-            strcpy( lpData->lpszCommandLine, returnBuffer );
+        lpData->lpszCommandLine = strdup( returnBuffer );
     }
 
     sizeOfReturnBuffer = 200;
@@ -1168,8 +1166,7 @@ static BOOL CALLBACK RunApplicationA_EnumLocalApplications
     }
     else
     {
-        if ((lpData->lpszCurrentDirectory = HeapAlloc( GetProcessHeap(), 0, strlen(returnBuffer)+1 )))
-            strcpy( lpData->lpszCurrentDirectory, returnBuffer );
+        lpData->lpszCurrentDirectory = strdup( returnBuffer );
     }
 
     sizeOfReturnBuffer = 200;
@@ -1182,8 +1179,7 @@ static BOOL CALLBACK RunApplicationA_EnumLocalApplications
     }
     else
     {
-        if ((lpData->lpszFileName = HeapAlloc( GetProcessHeap(), 0, strlen(returnBuffer)+1 )))
-            strcpy( lpData->lpszFileName, returnBuffer );
+        lpData->lpszFileName = strdup( returnBuffer );
     }
 
     sizeOfReturnBuffer = 200;
@@ -1196,8 +1192,7 @@ static BOOL CALLBACK RunApplicationA_EnumLocalApplications
     }
     else
     {
-        if ((lpData->lpszPath = HeapAlloc( GetProcessHeap(), 0, strlen(returnBuffer)+1 )))
-            strcpy( lpData->lpszPath, returnBuffer );
+        lpData->lpszPath = strdup( returnBuffer );
     }
 
     return FALSE; /* No need to keep going as we found what we wanted */
@@ -1303,7 +1298,7 @@ static HRESULT WINAPI IDirectPlayLobby3AImpl_RunApplication( IDirectPlayLobby3A
   DWORD dwSuspendCount;
   HANDLE hStart, hDeath, hSettingRead;
 
-  TRACE( "(%p)->(0x%08x,%p,%p,%p)\n",
+  TRACE( "(%p)->(0x%08lx,%p,%p,%p)\n",
          This, dwFlags, lpdwAppID, lpConn, hReceiveEvent );
 
   if( dwFlags != 0 )
@@ -1331,16 +1326,15 @@ static HRESULT WINAPI IDirectPlayLobby3AImpl_RunApplication( IDirectPlayLobby3A
   strcpy( temp, enumData.lpszPath );
   strcat( temp, "\\" );
   strcat( temp, enumData.lpszFileName );
-  HeapFree( GetProcessHeap(), 0, enumData.lpszPath );
-  HeapFree( GetProcessHeap(), 0, enumData.lpszFileName );
-  if ((appName = HeapAlloc( GetProcessHeap(), 0, strlen(temp)+1 ))) strcpy( appName, temp );
+  free( enumData.lpszPath );
+  free( enumData.lpszFileName );
+  appName = strdup( temp );
 
   /* Now the command line */
   strcat( temp, " " );
   strcat( temp, enumData.lpszCommandLine );
-  HeapFree( GetProcessHeap(), 0, enumData.lpszCommandLine );
-  if ((enumData.lpszCommandLine = HeapAlloc( GetProcessHeap(), 0, strlen(temp)+1 )))
-      strcpy( enumData.lpszCommandLine, temp );
+  free( enumData.lpszCommandLine );
+  enumData.lpszCommandLine = strdup( temp );
 
   ZeroMemory( &startupInfo, sizeof( startupInfo ) );
   startupInfo.cb = sizeof( startupInfo );
@@ -1363,22 +1357,22 @@ static HRESULT WINAPI IDirectPlayLobby3AImpl_RunApplication( IDirectPlayLobby3A
   {
     ERR( "Failed to create process for app %s\n", appName );
 
-    HeapFree( GetProcessHeap(), 0, appName );
-    HeapFree( GetProcessHeap(), 0, enumData.lpszCommandLine );
-    HeapFree( GetProcessHeap(), 0, enumData.lpszCurrentDirectory );
+    free( appName );
+    free( enumData.lpszCommandLine );
+    free( enumData.lpszCurrentDirectory );
 
     LeaveCriticalSection( &This->lock );
     return DPERR_CANTCREATEPROCESS;
   }
 
-  HeapFree( GetProcessHeap(), 0, appName );
-  HeapFree( GetProcessHeap(), 0, enumData.lpszCommandLine );
-  HeapFree( GetProcessHeap(), 0, enumData.lpszCurrentDirectory );
+  free( appName );
+  free( enumData.lpszCommandLine );
+  free( enumData.lpszCurrentDirectory );
 
   /* Reserve this global application id! */
   if( !DPLAYX_CreateLobbyApplication( newProcessInfo.dwProcessId ) )
   {
-    ERR( "Unable to create global application data for 0x%08x\n",
+    ERR( "Unable to create global application data for 0x%08lx\n",
            newProcessInfo.dwProcessId );
   }
 
@@ -1409,7 +1403,7 @@ static HRESULT WINAPI IDirectPlayLobby3AImpl_RunApplication( IDirectPlayLobby3A
   /* Unsuspend the process - should return the prev suspension count */
   if( ( dwSuspendCount = ResumeThread( newProcessInfo.hThread ) ) != 1 )
   {
-    ERR( "ResumeThread failed with 0x%08x\n", dwSuspendCount );
+    ERR( "ResumeThread failed with 0x%08lx\n", dwSuspendCount );
   }
 
   return DP_OK;
@@ -1419,7 +1413,7 @@ static HRESULT WINAPI IDirectPlayLobby3Impl_RunApplication( IDirectPlayLobby3 *i
         DWORD *lpdwAppID, DPLCONNECTION *lpConn, HANDLE hReceiveEvent )
 {
   IDirectPlayLobbyImpl *This = impl_from_IDirectPlayLobby3( iface );
-  FIXME( "(%p)->(0x%08x,%p,%p,%p):stub\n", This, dwFlags, lpdwAppID, lpConn, hReceiveEvent );
+  FIXME( "(%p)->(0x%08lx,%p,%p,%p):stub\n", This, dwFlags, lpdwAppID, lpConn, hReceiveEvent );
   return DPERR_OUTOFMEMORY;
 }
 
@@ -1487,7 +1481,7 @@ static HRESULT WINAPI IDirectPlayLobby3Impl_SetConnectionSettings( IDirectPlayLo
   IDirectPlayLobbyImpl *This = impl_from_IDirectPlayLobby3( iface );
   HRESULT hr;
 
-  TRACE("(%p)->(0x%08x,0x%08x,%p)\n", This, dwFlags, dwAppID, lpConn );
+  TRACE("(%p)->(0x%08lx,0x%08lx,%p)\n", This, dwFlags, dwAppID, lpConn );
 
   EnterCriticalSection( &This->lock );
 
@@ -1550,7 +1544,7 @@ static HRESULT WINAPI IDirectPlayLobby3AImpl_SetConnectionSettings( IDirectPlayL
   IDirectPlayLobbyImpl *This = impl_from_IDirectPlayLobby3A( iface );
   HRESULT hr;
 
-  TRACE("(%p)->(0x%08x,0x%08x,%p)\n", This, dwFlags, dwAppID, lpConn );
+  TRACE("(%p)->(0x%08lx,0x%08lx,%p)\n", This, dwFlags, dwAppID, lpConn );
 
   EnterCriticalSection( &This->lock );
 
@@ -1666,7 +1660,7 @@ HRESULT DPL_CreateCompoundAddress
   DWORD dwElements;
   LPCDPCOMPOUNDADDRESSELEMENT lpOrigElements = lpElements;
 
-  TRACE("(%p,0x%08x,%p,%p)\n", lpElements, dwElementCount, lpAddress, lpdwAddressSize );
+  TRACE("(%p,0x%08lx,%p,%p)\n", lpElements, dwElementCount, lpAddress, lpdwAddressSize );
 
   /* Parameter check */
   if( ( lpElements == NULL ) ||
@@ -1874,7 +1868,7 @@ static HRESULT WINAPI IDirectPlayLobby3Impl_WaitForConnectionSettings( IDirectPl
   HRESULT hr         = DP_OK;
   BOOL    bStartWait = !(dwFlags & DPLWAIT_CANCEL);
 
-  TRACE( "(%p)->(0x%08x)\n", iface, dwFlags );
+  TRACE( "(%p)->(0x%08lx)\n", iface, dwFlags );
 
   if( DPLAYX_WaitForConnectionSettings( bStartWait ) )
   {
@@ -1891,7 +1885,7 @@ static HRESULT WINAPI IDirectPlayLobby3AImpl_WaitForConnectionSettings( IDirectP
   HRESULT hr         = DP_OK;
   BOOL    bStartWait = !(dwFlags & DPLWAIT_CANCEL);
 
-  TRACE( "(%p)->(0x%08x)\n", iface, dwFlags );
+  TRACE( "(%p)->(0x%08lx)\n", iface, dwFlags );
 
   if( DPLAYX_WaitForConnectionSettings( bStartWait ) )
   {
@@ -2030,7 +2024,7 @@ HRESULT dplobby_create( REFIID riid, void **ppv )
     TRACE( "(%s, %p)\n", debugstr_guid( riid ), ppv );
 
     *ppv = NULL;
-    obj = HeapAlloc( GetProcessHeap(), 0, sizeof( *obj ) );
+    obj = malloc( sizeof( *obj ) );
     if ( !obj )
         return DPERR_OUTOFMEMORY;
 
@@ -2068,7 +2062,7 @@ HRESULT dplobby_create( REFIID riid, void **ppv )
 HRESULT WINAPI DirectPlayLobbyCreateA( GUID *lpGUIDDSP, IDirectPlayLobbyA **lplpDPL,
         IUnknown *lpUnk, void *lpData, DWORD dwDataSize )
 {
-  TRACE("lpGUIDDSP=%p lplpDPL=%p lpUnk=%p lpData=%p dwDataSize=%08x\n",
+  TRACE("lpGUIDDSP=%p lplpDPL=%p lpUnk=%p lpData=%p dwDataSize=%08lx\n",
         lpGUIDDSP,lplpDPL,lpUnk,lpData,dwDataSize);
 
   /* Parameter Check: lpGUIDSP, lpUnk & lpData must be NULL. dwDataSize must
@@ -2097,7 +2091,7 @@ HRESULT WINAPI DirectPlayLobbyCreateA( GUID *lpGUIDDSP, IDirectPlayLobbyA **lplp
 HRESULT WINAPI DirectPlayLobbyCreateW( GUID *lpGUIDDSP, IDirectPlayLobby **lplpDPL,
         IUnknown *lpUnk, void *lpData, DWORD dwDataSize )
 {
-  TRACE("lpGUIDDSP=%p lplpDPL=%p lpUnk=%p lpData=%p dwDataSize=%08x\n",
+  TRACE("lpGUIDDSP=%p lplpDPL=%p lpUnk=%p lpData=%p dwDataSize=%08lx\n",
         lpGUIDDSP,lplpDPL,lpUnk,lpData,dwDataSize);
 
   /* Parameter Check: lpGUIDSP, lpUnk & lpData must be NULL. dwDataSize must
