@@ -24,7 +24,6 @@
 
 #include "ntstatus.h"
 #define WIN32_NO_STATUS
-#define NONAMELESSUNION
 #include "windef.h"
 #include "winbase.h"
 #include "winnls.h"
@@ -84,7 +83,7 @@ HANDLE WINAPI DECLSPEC_HOTPATCH CreateRemoteThreadEx( HANDLE process, SECURITY_A
     else stack_commit = stack;
 
     if (!set_ntstatus( RtlCreateUserThread( process, sa ? sa->lpSecurityDescriptor : NULL, TRUE,
-                                            NULL, stack_reserve, stack_commit,
+                                            0, stack_reserve, stack_commit,
                                             (PRTL_THREAD_START_ROUTINE)start, param, &handle, &client_id )))
         return 0;
 
@@ -245,6 +244,15 @@ DWORD WINAPI DECLSPEC_HOTPATCH GetThreadId( HANDLE thread )
 }
 
 
+/***********************************************************************
+ *           GetThreadIdealProcessorEx   (kernelbase.@)
+ */
+BOOL WINAPI DECLSPEC_HOTPATCH GetThreadIdealProcessorEx( HANDLE thread, PROCESSOR_NUMBER *ideal )
+{
+    return set_ntstatus( NtQueryInformationThread( thread, ThreadIdealProcessorEx, ideal, sizeof(*ideal), NULL));
+}
+
+
 /***********************************************************************
  *	GetThreadLocale   (kernelbase.@)
  */
@@ -275,8 +283,7 @@ INT WINAPI DECLSPEC_HOTPATCH GetThreadPriority( HANDLE thread )
  */
 BOOL WINAPI DECLSPEC_HOTPATCH GetThreadPriorityBoost( HANDLE thread, BOOL *state )
 {
-    if (state) *state = FALSE;
-    return TRUE;
+    return set_ntstatus( NtQueryInformationThread( thread, ThreadPriorityBoost, state, sizeof(*state), NULL ));
 }
 
 
@@ -368,6 +375,18 @@ DWORD WINAPI DECLSPEC_HOTPATCH QueueUserAPC( PAPCFUNC func, HANDLE thread, ULONG
 }
 
 
+/***********************************************************************
+ *           QueryThreadCycleTime   (kernelbase.@)
+ */
+BOOL WINAPI DECLSPEC_HOTPATCH QueryThreadCycleTime( HANDLE thread, ULONG64 *cycle )
+{
+    static int once;
+    if (!once++) FIXME( "(%p,%p): stub!\n", thread, cycle );
+    SetLastError( ERROR_CALL_NOT_IMPLEMENTED );
+    return FALSE;
+}
+
+
 /**********************************************************************
  *           ResumeThread   (kernelbase.@)
  */
@@ -394,7 +413,7 @@ BOOL WINAPI DECLSPEC_HOTPATCH SetThreadContext( HANDLE thread, const CONTEXT *co
  */
 HRESULT WINAPI DECLSPEC_HOTPATCH SetThreadDescription( HANDLE thread, PCWSTR description )
 {
-    THREAD_DESCRIPTION_INFORMATION info;
+    THREAD_NAME_INFORMATION info;
     int length;
 
     TRACE( "(%p, %s)\n", thread, debugstr_w( description ));
@@ -404,10 +423,10 @@ HRESULT WINAPI DECLSPEC_HOTPATCH SetThreadDescription( HANDLE thread, PCWSTR des
     if (length > USHRT_MAX)
         return HRESULT_FROM_NT(STATUS_INVALID_PARAMETER);
 
-    info.Description.Length = info.Description.MaximumLength = length;
-    info.Description.Buffer = (WCHAR *)description;
+    info.ThreadName.Length = info.ThreadName.MaximumLength = length;
+    info.ThreadName.Buffer = (WCHAR *)description;
 
-    return HRESULT_FROM_NT(NtSetInformationThread( thread, ThreadDescription, &info, sizeof(info) ));
+    return HRESULT_FROM_NT(NtSetInformationThread( thread, ThreadNameInformation, &info, sizeof(info) ));
 }
 
 /***********************************************************************
@@ -415,7 +434,7 @@ HRESULT WINAPI DECLSPEC_HOTPATCH SetThreadDescription( HANDLE thread, PCWSTR des
  */
 HRESULT WINAPI DECLSPEC_HOTPATCH GetThreadDescription( HANDLE thread, WCHAR **description )
 {
-    THREAD_DESCRIPTION_INFORMATION *info;
+    THREAD_NAME_INFORMATION *info;
     NTSTATUS status;
     ULONG length;
 
@@ -424,23 +443,23 @@ HRESULT WINAPI DECLSPEC_HOTPATCH GetThreadDescription( HANDLE thread, WCHAR **de
     *description = NULL;
 
     length = 0;
-    status = NtQueryInformationThread( thread, ThreadDescription, NULL, 0, &length );
+    status = NtQueryInformationThread( thread, ThreadNameInformation, NULL, 0, &length );
     if (status != STATUS_BUFFER_TOO_SMALL)
         return HRESULT_FROM_NT(status);
 
     if (!(info = heap_alloc( length )))
         return HRESULT_FROM_NT(STATUS_NO_MEMORY);
 
-    status = NtQueryInformationThread( thread, ThreadDescription, info, length, &length );
+    status = NtQueryInformationThread( thread, ThreadNameInformation, info, length, &length );
     if (!status)
     {
-        if (!(*description = LocalAlloc( 0, info->Description.Length + sizeof(WCHAR))))
+        if (!(*description = LocalAlloc( 0, info->ThreadName.Length + sizeof(WCHAR))))
             status = STATUS_NO_MEMORY;
         else
         {
-            if (info->Description.Length)
-                memcpy(*description, info->Description.Buffer, info->Description.Length);
-            (*description)[info->Description.Length / sizeof(WCHAR)] = 0;
+            if (info->ThreadName.Length)
+                memcpy(*description, info->ThreadName.Buffer, info->ThreadName.Length);
+            (*description)[info->ThreadName.Length / sizeof(WCHAR)] = 0;
         }
     }
 
@@ -489,13 +508,13 @@ BOOL WINAPI DECLSPEC_HOTPATCH SetThreadGroupAffinity( HANDLE thread, const GROUP
  */
 DWORD WINAPI DECLSPEC_HOTPATCH SetThreadIdealProcessor( HANDLE thread, DWORD proc )
 {
-    FIXME( "(%p %u): stub\n", thread, proc );
-    if (proc > MAXIMUM_PROCESSORS)
-    {
-        SetLastError( ERROR_INVALID_PARAMETER );
-        return ~0u;
-    }
-    return 0;
+    NTSTATUS status;
+
+    status = NtSetInformationThread( thread, ThreadIdealProcessor, &proc, sizeof(proc) );
+    if (NT_SUCCESS(status)) return status;
+
+    SetLastError( RtlNtStatusToDosError( status ));
+    return ~0u;
 }
 
 
@@ -545,7 +564,7 @@ BOOL WINAPI DECLSPEC_HOTPATCH SetThreadPriority( HANDLE thread, INT priority )
  */
 BOOL WINAPI DECLSPEC_HOTPATCH SetThreadPriorityBoost( HANDLE thread, BOOL disable )
 {
-    return TRUE;
+    return set_ntstatus( NtSetInformationThread( thread, ThreadPriorityBoost, &disable, sizeof(disable) ));
 }
 
 
@@ -583,6 +602,25 @@ LANGID WINAPI DECLSPEC_HOTPATCH SetThreadUILanguage( LANGID langid )
 }
 
 
+/**********************************************************************
+ *            SetThreadInformation   (kernelbase.@)
+ */
+BOOL WINAPI DECLSPEC_HOTPATCH SetThreadInformation( HANDLE thread, THREAD_INFORMATION_CLASS info_class,
+        VOID *info, DWORD size )
+{
+    switch (info_class)
+    {
+        case ThreadMemoryPriority:
+            return set_ntstatus( NtSetInformationThread( thread, ThreadPagePriority, info, size ));
+        case ThreadPowerThrottling:
+            return set_ntstatus( NtSetInformationThread( thread, ThreadPowerThrottlingState, info, size ));
+        default:
+            FIXME("Unsupported class %u.\n", info_class);
+            return FALSE;
+    }
+}
+
+
 /**********************************************************************
  *           SuspendThread   (kernelbase.@)
  */
@@ -729,6 +767,12 @@ BOOL WINAPI DECLSPEC_HOTPATCH TlsSetValue( DWORD index, LPVOID value )
  ***********************************************************************/
 
 
+struct fiber_actctx
+{
+    ACTIVATION_CONTEXT_STACK stack_space;    /* activation context stack space */
+    ACTIVATION_CONTEXT_STACK *stack_ptr;     /* last value of ActivationContextStackPointer */
+};
+
 struct fiber_data
 {
     LPVOID                param;             /* 00/00 fiber param */
@@ -739,11 +783,12 @@ struct fiber_data
     CONTEXT               context;           /* 14/30 fiber context */
     DWORD                 flags;             /*       fiber flags */
     LPFIBER_START_ROUTINE start;             /*       start routine */
-    void                **fls_slots;         /*       fiber storage slots */
+    void                 *fls_slots;         /*       fiber storage slots */
+    struct fiber_actctx   actctx;            /*       activation context state */
 };
 
-extern void WINAPI switch_fiber( CONTEXT *old, CONTEXT *new );
 #ifdef __i386__
+extern void WINAPI switch_fiber( CONTEXT *old, CONTEXT *new );
 __ASM_STDCALL_FUNC( switch_fiber, 8,
                     "movl 4(%esp),%ecx\n\t"     /* old */
                     "movl %edi,0x9c(%ecx)\n\t"  /* old->Edi */
@@ -761,8 +806,37 @@ __ASM_STDCALL_FUNC( switch_fiber, 8,
                     "movl 0xb4(%ecx),%ebp\n\t"  /* new->Ebp */
                     "movl 0xc4(%ecx),%esp\n\t"  /* new->Esp */
                     "jmp *0xb8(%ecx)" )         /* new->Eip */
+#elif defined(__arm64ec__)
+static void __attribute__((naked)) WINAPI switch_fiber( CONTEXT *old, CONTEXT *new )
+{
+    asm( "mov x2, sp\n\t"
+         "stp x27, x2,  [x0, #0x90]\n\t"  /* old->Rbx,Rsp */
+         "str x29,      [x0, #0xa0]\n\t"  /* old->Rbp */
+         "stp x25, x26, [x0, #0xa8]\n\t"  /* old->Rsi,Rdi */
+         "stp x19, x20, [x0, #0xd8]\n\t"  /* old->R12,R13 */
+         "stp x21, x22, [x0, #0xe8]\n\t"  /* old->R14,R15 */
+         "str x30,      [x0, #0xf8]\n\t"  /* old->Rip */
+         "stp q8,  q9,  [x0, #0x220]\n\t" /* old->Xmm8,Xmm9 */
+         "stp q10, q11, [x0, #0x240]\n\t" /* old->Xmm10,Xmm11 */
+         "stp q12, q13, [x0, #0x260]\n\t" /* old->Xmm12,Xmm13 */
+         "stp q14, q15, [x0, #0x280]\n\t" /* old->Xmm14,Xmm15 */
+         /* FIXME: MxCsr */
+         "ldp x27, x2,  [x1, #0x90]\n\t"  /* old->Rbx,Rsp */
+         "ldr x29,      [x1, #0xa0]\n\t"  /* old->Rbp */
+         "ldp x25, x26, [x1, #0xa8]\n\t"  /* old->Rsi,Rdi */
+         "ldp x19, x20, [x1, #0xd8]\n\t"  /* old->R12,R13 */
+         "ldp x21, x22, [x1, #0xe8]\n\t"  /* old->R14,R15 */
+         "ldr x30,      [x1, #0xf8]\n\t"  /* old->Rip */
+         "ldp q8,  q9,  [x1, #0x220]\n\t" /* old->Xmm8,Xmm9 */
+         "ldp q10, q11, [x1, #0x240]\n\t" /* old->Xmm10,Xmm11 */
+         "ldp q12, q13, [x1, #0x260]\n\t" /* old->Xmm12,Xmm13 */
+         "ldp q14, q15, [x1, #0x280]\n\t" /* old->Xmm14,Xmm15 */
+         "mov sp, x2\n\t"
+         "ret" );
+}
 #elif defined(__x86_64__)
-__ASM_STDCALL_FUNC( switch_fiber, 8,
+extern void WINAPI switch_fiber( CONTEXT *old, CONTEXT *new );
+__ASM_GLOBAL_FUNC( switch_fiber,
                     "movq %rbx,0x90(%rcx)\n\t"       /* old->Rbx */
                     "leaq 0x8(%rsp),%rax\n\t"
                     "movq %rax,0x98(%rcx)\n\t"       /* old->Rsp */
@@ -806,7 +880,8 @@ __ASM_STDCALL_FUNC( switch_fiber, 8,
                     "movq 0x98(%rdx),%rsp\n\t"       /* new->Rsp */
                     "jmp *0xf8(%rdx)" )              /* new->Rip */
 #elif defined(__arm__)
-__ASM_STDCALL_FUNC( switch_fiber, 8,
+extern void WINAPI switch_fiber( CONTEXT *old, CONTEXT *new );
+__ASM_GLOBAL_FUNC( switch_fiber,
                    "str r4, [r0, #0x14]\n\t"   /* old->R4 */
                    "str r5, [r0, #0x18]\n\t"   /* old->R5 */
                    "str r6, [r0, #0x1c]\n\t"   /* old->R6 */
@@ -829,7 +904,8 @@ __ASM_STDCALL_FUNC( switch_fiber, 8,
                    "ldr r2, [r1, #0x40]\n\t"   /* new->Pc */
                    "bx r2" )
 #elif defined(__aarch64__)
-__ASM_STDCALL_FUNC( switch_fiber, 8,
+extern void WINAPI switch_fiber( CONTEXT *old, CONTEXT *new );
+__ASM_GLOBAL_FUNC( switch_fiber,
                    "stp x19, x20, [x0, #0xa0]\n\t"  /* old->X19,X20 */
                    "stp x21, x22, [x0, #0xb0]\n\t"  /* old->X21,X22 */
                    "stp x23, x24, [x0, #0xc0]\n\t"  /* old->X23,X24 */
@@ -850,7 +926,7 @@ __ASM_STDCALL_FUNC( switch_fiber, 8,
                    "mov sp, x2\n\t"
                    "ret" )
 #else
-void WINAPI switch_fiber( CONTEXT *old, CONTEXT *new )
+static void WINAPI switch_fiber( CONTEXT *old, CONTEXT *new )
 {
     FIXME( "not implemented\n" );
     DbgBreakPoint();
@@ -860,7 +936,7 @@ void WINAPI switch_fiber( CONTEXT *old, CONTEXT *new )
 /* call the fiber initial function once we have switched stack */
 static void CDECL start_fiber(void)
 {
-    struct fiber_data *fiber = NtCurrentTeb()->Tib.u.FiberData;
+    struct fiber_data *fiber = NtCurrentTeb()->Tib.FiberData;
     LPFIBER_START_ROUTINE start = fiber->start;
 
     __TRY
@@ -880,6 +956,9 @@ static void init_fiber_context( struct fiber_data *fiber )
 #ifdef __i386__
     fiber->context.Esp = (ULONG_PTR)fiber->stack_base - 4;
     fiber->context.Eip = (ULONG_PTR)start_fiber;
+#elif defined(__arm64ec__)
+    fiber->context.Rsp = (ULONG_PTR)fiber->stack_base;
+    fiber->context.Rip = (ULONG_PTR)start_fiber;
 #elif defined(__x86_64__)
     fiber->context.Rsp = (ULONG_PTR)fiber->stack_base - 0x28;
     fiber->context.Rip = (ULONG_PTR)start_fiber;
@@ -892,6 +971,38 @@ static void init_fiber_context( struct fiber_data *fiber )
 #endif
 }
 
+static void move_list( LIST_ENTRY *dest, LIST_ENTRY *src )
+{
+    LIST_ENTRY *head = src->Flink;
+    LIST_ENTRY *tail = src->Blink;
+
+    if (src != head)
+    {
+        dest->Flink = head;
+        dest->Blink = tail;
+        head->Blink = dest;
+        tail->Flink = dest;
+    }
+    else InitializeListHead( dest );
+}
+
+static void relocate_thread_actctx_stack( ACTIVATION_CONTEXT_STACK *dest )
+{
+    ACTIVATION_CONTEXT_STACK *src = NtCurrentTeb()->ActivationContextStackPointer;
+
+    C_ASSERT(sizeof(*dest) == sizeof(dest->ActiveFrame) + sizeof(dest->FrameListCache) +
+                              sizeof(dest->Flags) + sizeof(dest->NextCookieSequenceNumber) +
+                              sizeof(dest->StackId));
+
+    dest->ActiveFrame = src->ActiveFrame;
+    move_list( &dest->FrameListCache, &src->FrameListCache );
+    dest->Flags = src->Flags;
+    dest->NextCookieSequenceNumber = src->NextCookieSequenceNumber;
+    dest->StackId = src->StackId;
+
+    NtCurrentTeb()->ActivationContextStackPointer = dest;
+}
+
 
 /***********************************************************************
  *           CreateFiber   (kernelbase.@)
@@ -930,6 +1041,8 @@ LPVOID WINAPI DECLSPEC_HOTPATCH CreateFiberEx( SIZE_T stack_commit, SIZE_T stack
     fiber->except      = (void *)-1;
     fiber->start       = start;
     fiber->flags       = flags;
+    InitializeListHead( &fiber->actctx.stack_space.FrameListCache );
+    fiber->actctx.stack_ptr = &fiber->actctx.stack_space;
     init_fiber_context( fiber );
     return fiber;
 }
@@ -940,11 +1053,12 @@ LPVOID WINAPI DECLSPEC_HOTPATCH CreateFiberEx( SIZE_T stack_commit, SIZE_T stack
  */
 BOOL WINAPI DECLSPEC_HOTPATCH ConvertFiberToThread(void)
 {
-    struct fiber_data *fiber = NtCurrentTeb()->Tib.u.FiberData;
+    struct fiber_data *fiber = NtCurrentTeb()->Tib.FiberData;
 
     if (fiber)
     {
-        NtCurrentTeb()->Tib.u.FiberData = NULL;
+        relocate_thread_actctx_stack( &NtCurrentTeb()->ActivationContextStack );
+        NtCurrentTeb()->Tib.FiberData = NULL;
         HeapFree( GetProcessHeap(), 0, fiber );
     }
     return TRUE;
@@ -954,7 +1068,7 @@ BOOL WINAPI DECLSPEC_HOTPATCH ConvertFiberToThread(void)
 /***********************************************************************
  *           ConvertThreadToFiber   (kernelbase.@)
  */
-LPVOID WINAPI DECLSPEC_HOTPATCH ConvertThreadToFiber( LPVOID param )
+LPVOID WINAPI /* DECLSPEC_HOTPATCH */ ConvertThreadToFiber( LPVOID param )
 {
     return ConvertThreadToFiberEx( param, 0 );
 }
@@ -967,6 +1081,12 @@ LPVOID WINAPI DECLSPEC_HOTPATCH ConvertThreadToFiberEx( LPVOID param, DWORD flag
 {
     struct fiber_data *fiber;
 
+    if (NtCurrentTeb()->Tib.FiberData)
+    {
+        SetLastError( ERROR_ALREADY_FIBER );
+        return NULL;
+    }
+
     if (!(fiber = HeapAlloc( GetProcessHeap(), 0, sizeof(*fiber) )))
     {
         SetLastError( ERROR_NOT_ENOUGH_MEMORY );
@@ -980,7 +1100,8 @@ LPVOID WINAPI DECLSPEC_HOTPATCH ConvertThreadToFiberEx( LPVOID param, DWORD flag
     fiber->start            = NULL;
     fiber->flags            = flags;
     fiber->fls_slots        = NtCurrentTeb()->FlsSlots;
-    NtCurrentTeb()->Tib.u.FiberData = fiber;
+    relocate_thread_actctx_stack( &fiber->actctx.stack_space );
+    NtCurrentTeb()->Tib.FiberData = fiber;
     return fiber;
 }
 
@@ -993,13 +1114,15 @@ void WINAPI DECLSPEC_HOTPATCH DeleteFiber( LPVOID fiber_ptr )
     struct fiber_data *fiber = fiber_ptr;
 
     if (!fiber) return;
-    if (fiber == NtCurrentTeb()->Tib.u.FiberData)
+    if (fiber == NtCurrentTeb()->Tib.FiberData)
     {
+        relocate_thread_actctx_stack( &NtCurrentTeb()->ActivationContextStack );
         HeapFree( GetProcessHeap(), 0, fiber );
         RtlExitUserThread( 1 );
     }
     RtlFreeUserStack( fiber->stack_allocation );
-    HeapFree( GetProcessHeap(), 0, fiber->fls_slots );
+    RtlProcessFlsData( fiber->fls_slots, 3 );
+    RtlFreeActivationContextStack( &fiber->actctx.stack_space );
     HeapFree( GetProcessHeap(), 0, fiber );
 }
 
@@ -1009,7 +1132,7 @@ void WINAPI DECLSPEC_HOTPATCH DeleteFiber( LPVOID fiber_ptr )
  */
 BOOL WINAPI DECLSPEC_HOTPATCH IsThreadAFiber(void)
 {
-    return NtCurrentTeb()->Tib.u.FiberData != NULL;
+    return NtCurrentTeb()->Tib.FiberData != NULL;
 }
 
 
@@ -1019,20 +1142,22 @@ BOOL WINAPI DECLSPEC_HOTPATCH IsThreadAFiber(void)
 void WINAPI DECLSPEC_HOTPATCH SwitchToFiber( LPVOID fiber )
 {
     struct fiber_data *new_fiber = fiber;
-    struct fiber_data *current_fiber = NtCurrentTeb()->Tib.u.FiberData;
+    struct fiber_data *current_fiber = NtCurrentTeb()->Tib.FiberData;
 
     current_fiber->except      = NtCurrentTeb()->Tib.ExceptionList;
     current_fiber->stack_limit = NtCurrentTeb()->Tib.StackLimit;
     current_fiber->fls_slots   = NtCurrentTeb()->FlsSlots;
+    current_fiber->actctx.stack_ptr = NtCurrentTeb()->ActivationContextStackPointer;
     /* stack_allocation and stack_base never change */
 
     /* FIXME: should save floating point context if requested in fiber->flags */
-    NtCurrentTeb()->Tib.u.FiberData   = new_fiber;
+    NtCurrentTeb()->Tib.FiberData     = new_fiber;
     NtCurrentTeb()->Tib.ExceptionList = new_fiber->except;
     NtCurrentTeb()->Tib.StackBase     = new_fiber->stack_base;
     NtCurrentTeb()->Tib.StackLimit    = new_fiber->stack_limit;
     NtCurrentTeb()->DeallocationStack = new_fiber->stack_allocation;
     NtCurrentTeb()->FlsSlots          = new_fiber->fls_slots;
+    NtCurrentTeb()->ActivationContextStackPointer = new_fiber->actctx.stack_ptr;
     switch_fiber( &current_fiber->context, &new_fiber->context );
 }
 
@@ -1043,38 +1168,8 @@ void WINAPI DECLSPEC_HOTPATCH SwitchToFiber( LPVOID fiber )
 DWORD WINAPI DECLSPEC_HOTPATCH FlsAlloc( PFLS_CALLBACK_FUNCTION callback )
 {
     DWORD index;
-    PEB * const peb = NtCurrentTeb()->Peb;
 
-    RtlAcquirePebLock();
-    if (!peb->FlsCallback &&
-        !(peb->FlsCallback = HeapAlloc( GetProcessHeap(), HEAP_ZERO_MEMORY,
-                                        8 * sizeof(peb->FlsBitmapBits) * sizeof(void*) )))
-    {
-        SetLastError( ERROR_NOT_ENOUGH_MEMORY );
-        index = FLS_OUT_OF_INDEXES;
-    }
-    else
-    {
-        index = RtlFindClearBitsAndSet( peb->FlsBitmap, 1, 1 );
-        if (index != ~0U)
-        {
-            if (!NtCurrentTeb()->FlsSlots &&
-                !(NtCurrentTeb()->FlsSlots = HeapAlloc( GetProcessHeap(), HEAP_ZERO_MEMORY,
-                                                        8 * sizeof(peb->FlsBitmapBits) * sizeof(void*) )))
-            {
-                RtlClearBits( peb->FlsBitmap, index, 1 );
-                index = FLS_OUT_OF_INDEXES;
-                SetLastError( ERROR_NOT_ENOUGH_MEMORY );
-            }
-            else
-            {
-                NtCurrentTeb()->FlsSlots[index] = 0; /* clear the value */
-                peb->FlsCallback[index] = callback;
-            }
-        }
-        else SetLastError( ERROR_NO_MORE_ITEMS );
-    }
-    RtlReleasePebLock();
+    if (!set_ntstatus( RtlFlsAlloc( callback, &index ))) return FLS_OUT_OF_INDEXES;
     return index;
 }
 
@@ -1084,20 +1179,7 @@ DWORD WINAPI DECLSPEC_HOTPATCH FlsAlloc( PFLS_CALLBACK_FUNCTION callback )
  */
 BOOL WINAPI DECLSPEC_HOTPATCH FlsFree( DWORD index )
 {
-    BOOL ret;
-
-    RtlAcquirePebLock();
-    ret = RtlAreBitsSet( NtCurrentTeb()->Peb->FlsBitmap, index, 1 );
-    if (ret) RtlClearBits( NtCurrentTeb()->Peb->FlsBitmap, index, 1 );
-    if (ret)
-    {
-        /* FIXME: call Fls callback */
-        /* FIXME: add equivalent of ThreadZeroTlsCell here */
-        if (NtCurrentTeb()->FlsSlots) NtCurrentTeb()->FlsSlots[index] = 0;
-    }
-    else SetLastError( ERROR_INVALID_PARAMETER );
-    RtlReleasePebLock();
-    return ret;
+    return set_ntstatus( RtlFlsFree( index ));
 }
 
 
@@ -1106,13 +1188,11 @@ BOOL WINAPI DECLSPEC_HOTPATCH FlsFree( DWORD index )
  */
 PVOID WINAPI DECLSPEC_HOTPATCH FlsGetValue( DWORD index )
 {
-    if (!index || index >= 8 * sizeof(NtCurrentTeb()->Peb->FlsBitmapBits) || !NtCurrentTeb()->FlsSlots)
-    {
-        SetLastError( ERROR_INVALID_PARAMETER );
-        return NULL;
-    }
+    void *data;
+
+    if (!set_ntstatus( RtlFlsGetValue( index, &data ))) return NULL;
     SetLastError( ERROR_SUCCESS );
-    return NtCurrentTeb()->FlsSlots[index];
+    return data;
 }
 
 
@@ -1121,20 +1201,7 @@ PVOID WINAPI DECLSPEC_HOTPATCH FlsGetValue( DWORD index )
  */
 BOOL WINAPI DECLSPEC_HOTPATCH FlsSetValue( DWORD index, PVOID data )
 {
-    if (!index || index >= 8 * sizeof(NtCurrentTeb()->Peb->FlsBitmapBits))
-    {
-        SetLastError( ERROR_INVALID_PARAMETER );
-        return FALSE;
-    }
-    if (!NtCurrentTeb()->FlsSlots &&
-        !(NtCurrentTeb()->FlsSlots = HeapAlloc( GetProcessHeap(), HEAP_ZERO_MEMORY,
-                                        8 * sizeof(NtCurrentTeb()->Peb->FlsBitmapBits) * sizeof(void*) )))
-    {
-        SetLastError( ERROR_NOT_ENOUGH_MEMORY );
-        return FALSE;
-    }
-    NtCurrentTeb()->FlsSlots[index] = data;
-    return TRUE;
+    return set_ntstatus( RtlFlsSetValue( index, data ));
 }
 
 
@@ -1176,14 +1243,23 @@ PTP_CLEANUP_GROUP WINAPI DECLSPEC_HOTPATCH CreateThreadpoolCleanupGroup(void)
 }
 
 
+static void WINAPI tp_io_callback( TP_CALLBACK_INSTANCE *instance, void *userdata, void *cvalue, IO_STATUS_BLOCK *iosb, TP_IO *io )
+{
+    PTP_WIN32_IO_CALLBACK callback = *(void **)io;
+    callback( instance, userdata, cvalue, RtlNtStatusToDosError( iosb->Status ), iosb->Information, io );
+}
+
+
 /***********************************************************************
  *           CreateThreadpoolIo   (kernelbase.@)
  */
-PTP_IO WINAPI /* DECLSPEC_HOTPATCH */ CreateThreadpoolIo( HANDLE handle, PTP_WIN32_IO_CALLBACK callback,
+PTP_IO WINAPI DECLSPEC_HOTPATCH CreateThreadpoolIo( HANDLE handle, PTP_WIN32_IO_CALLBACK callback,
                                                     PVOID userdata, TP_CALLBACK_ENVIRON *environment )
 {
-    FIXME( "(%p, %p, %p, %p): stub\n", handle, callback, userdata, environment );
-    return FALSE;
+    TP_IO *io;
+    if (!set_ntstatus( TpAllocIoCompletion( &io, handle, tp_io_callback, userdata, environment ))) return NULL;
+    *(void **)io = callback; /* ntdll leaves us space to store our callback at the beginning of TP_IO struct */
+    return io;
 }
 
 
@@ -1243,3 +1319,19 @@ BOOL WINAPI DECLSPEC_HOTPATCH QueueUserWorkItem( LPTHREAD_START_ROUTINE func, PV
 {
     return set_ntstatus( RtlQueueWorkItem( func, context, flags ));
 }
+
+/***********************************************************************
+ *           SetThreadpoolStackInformation   (kernelbase.@)
+ */
+BOOL WINAPI DECLSPEC_HOTPATCH SetThreadpoolStackInformation( PTP_POOL pool, PTP_POOL_STACK_INFORMATION stack_info )
+{
+    return set_ntstatus( TpSetPoolStackInformation( pool, stack_info ));
+}
+
+/***********************************************************************
+ *           QueryThreadpoolStackInformation   (kernelbase.@)
+ */
+BOOL WINAPI DECLSPEC_HOTPATCH QueryThreadpoolStackInformation( PTP_POOL pool, PTP_POOL_STACK_INFORMATION stack_info )
+{
+    return set_ntstatus( TpQueryPoolStackInformation( pool, stack_info ));
+}
