@@ -17,7 +17,6 @@
  */
 
 #include <stdarg.h>
-#include <assert.h>
 #include <math.h>
 
 #define COBJMACROS
@@ -32,6 +31,7 @@
 #include "wine/debug.h"
 
 #include "mshtml_private.h"
+#include "htmlevent.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(mshtml);
 
@@ -173,42 +173,52 @@ static const ISVGElementVtbl SVGElementVtbl = {
     SVGElement_get_focusable
 };
 
-static inline SVGElement *SVGElement_from_HTMLDOMNode(HTMLDOMNode *iface)
+static inline SVGElement *SVGElement_from_DispatchEx(DispatchEx *iface)
 {
-    return CONTAINING_RECORD(iface, SVGElement, element.node);
+    return CONTAINING_RECORD(iface, SVGElement, element.node.event_target.dispex);
 }
 
-static HRESULT SVGElement_QI(HTMLDOMNode *iface, REFIID riid, void **ppv)
+static void *SVGElement_query_interface(DispatchEx *dispex, REFIID riid)
 {
-    SVGElement *This = SVGElement_from_HTMLDOMNode(iface);
-
-    TRACE("(%p)->(%s %p)\n", This, debugstr_mshtml_guid(riid), ppv);
+    SVGElement *This = SVGElement_from_DispatchEx(dispex);
 
     if(IsEqualGUID(&IID_ISVGElement, riid))
-        *ppv = &This->ISVGElement_iface;
-    else
-        return HTMLElement_QI(&This->element.node, riid, ppv);
+        return &This->ISVGElement_iface;
 
-    IUnknown_AddRef((IUnknown*)*ppv);
-    return S_OK;
+    return HTMLElement_query_interface(&This->element.node.event_target.dispex, riid);
 }
 
 static const NodeImplVtbl SVGElementImplVtbl = {
-    &CLSID_SVGElement,
-    SVGElement_QI,
-    HTMLElement_destructor,
-    HTMLElement_cpc,
-    HTMLElement_clone,
-    NULL,
-    HTMLElement_get_attr_col,
+    .clsid                 = &CLSID_SVGElement,
+    .cpc_entries           = HTMLElement_cpc,
+    .clone                 = HTMLElement_clone,
+    .get_attr_col          = HTMLElement_get_attr_col,
 };
 
-static void init_svg_element(SVGElement *svg_element, HTMLDocumentNode *doc, nsIDOMSVGElement *nselem)
+static const event_target_vtbl_t SVGElement_event_target_vtbl = {
+    {
+        HTMLELEMENT_DISPEX_VTBL_ENTRIES,
+        .query_interface= SVGElement_query_interface,
+        .destructor     = HTMLElement_destructor,
+        .traverse       = HTMLElement_traverse,
+        .unlink         = HTMLElement_unlink
+    },
+    HTMLELEMENT_EVENT_TARGET_VTBL_ENTRIES,
+    .handle_event       = HTMLElement_handle_event
+};
+
+static dispex_static_data_t SVGElement_dispex = {
+    "HTMLUnknownElement",
+    &SVGElement_event_target_vtbl.dispex_vtbl,
+    DispHTMLUnknownElement_tid,
+    HTMLElement_iface_tids,
+    HTMLElement_init_dispex_info
+};
+
+static void init_svg_element(SVGElement *svg_element, HTMLDocumentNode *doc, nsIDOMSVGElement *nselem, dispex_static_data_t *dispex_data)
 {
-    if(!svg_element->element.node.vtbl)
-        svg_element->element.node.vtbl = &SVGElementImplVtbl;
     svg_element->ISVGElement_iface.lpVtbl = &SVGElementVtbl;
-    HTMLElement_Init(&svg_element->element, doc, (nsIDOMElement*)nselem, NULL);
+    HTMLElement_Init(&svg_element->element, doc, (nsIDOMElement*)nselem, dispex_data);
 }
 
 struct SVGSVGElement {
@@ -488,14 +498,14 @@ static HRESULT WINAPI SVGSVGElement_get_currentTranslate(ISVGSVGElement *iface,
 static HRESULT WINAPI SVGSVGElement_suspendRedraw(ISVGSVGElement *iface, ULONG max_wait, ULONG *p)
 {
     SVGSVGElement *This = impl_from_ISVGSVGElement(iface);
-    FIXME("(%p)->(%u %p)\n", This, max_wait, p);
+    FIXME("(%p)->(%lu %p)\n", This, max_wait, p);
     return E_NOTIMPL;
 }
 
 static HRESULT WINAPI SVGSVGElement_unsuspendRedraw(ISVGSVGElement *iface, ULONG id)
 {
     SVGSVGElement *This = impl_from_ISVGSVGElement(iface);
-    FIXME("(%p)->(%u)\n", This, id);
+    FIXME("(%p)->(%lu)\n", This, id);
     return E_NOTIMPL;
 }
 
@@ -714,48 +724,60 @@ static const ISVGSVGElementVtbl SVGSVGElementVtbl = {
     SVGSVGElement_getElementById
 };
 
-static inline SVGSVGElement *SVGSVGElement_from_HTMLDOMNode(HTMLDOMNode *iface)
+static inline SVGSVGElement *SVGSVGElement_from_DispatchEx(DispatchEx *iface)
 {
-    return CONTAINING_RECORD(iface, SVGSVGElement, svg_element.element.node);
+    return CONTAINING_RECORD(iface, SVGSVGElement, svg_element.element.node.event_target.dispex);
 }
 
-static HRESULT SVGSVGElement_QI(HTMLDOMNode *iface, REFIID riid, void **ppv)
+static void *SVGSVGElement_query_interface(DispatchEx *dispex, REFIID riid)
 {
-    SVGSVGElement *This = SVGSVGElement_from_HTMLDOMNode(iface);
-
-    TRACE("(%p)->(%s %p)\n", This, debugstr_mshtml_guid(riid), ppv);
+    SVGSVGElement *This = SVGSVGElement_from_DispatchEx(dispex);
 
     if(IsEqualGUID(&IID_ISVGSVGElement, riid))
-        *ppv = &This->ISVGSVGElement_iface;
-    else
-        return SVGElement_QI(&This->svg_element.element.node, riid, ppv);
+        return &This->ISVGSVGElement_iface;
 
-    IUnknown_AddRef((IUnknown*)*ppv);
-    return S_OK;
+    return SVGElement_query_interface(&This->svg_element.element.node.event_target.dispex, riid);
 }
 
 static const NodeImplVtbl SVGSVGElementImplVtbl = {
-    &CLSID_SVGSVGElement,
-    SVGSVGElement_QI,
-    HTMLElement_destructor,
-    HTMLElement_cpc,
-    HTMLElement_clone,
-    NULL,
-    HTMLElement_get_attr_col,
+    .clsid                 = &CLSID_SVGSVGElement,
+    .cpc_entries           = HTMLElement_cpc,
+    .clone                 = HTMLElement_clone,
+    .get_attr_col          = HTMLElement_get_attr_col,
+};
+
+static const event_target_vtbl_t SVGSVGElement_event_target_vtbl = {
+    {
+        HTMLELEMENT_DISPEX_VTBL_ENTRIES,
+        .query_interface= SVGSVGElement_query_interface,
+        .destructor     = HTMLElement_destructor,
+        .traverse       = HTMLElement_traverse,
+        .unlink         = HTMLElement_unlink
+    },
+    HTMLELEMENT_EVENT_TARGET_VTBL_ENTRIES,
+    .handle_event       = HTMLElement_handle_event
+};
+
+static dispex_static_data_t SVGSVGElement_dispex = {
+    "HTMLUnknownElement",
+    &SVGSVGElement_event_target_vtbl.dispex_vtbl,
+    DispHTMLUnknownElement_tid,
+    HTMLElement_iface_tids,
+    HTMLElement_init_dispex_info
 };
 
 static HRESULT create_viewport_element(HTMLDocumentNode *doc, nsIDOMSVGElement *nselem, HTMLElement **elem)
 {
     SVGSVGElement *ret;
 
-    ret = heap_alloc_zero(sizeof(SVGSVGElement));
+    ret = calloc(1, sizeof(SVGSVGElement));
     if(!ret)
         return E_OUTOFMEMORY;
 
     ret->ISVGSVGElement_iface.lpVtbl = &SVGSVGElementVtbl;
     ret->svg_element.element.node.vtbl = &SVGSVGElementImplVtbl;
 
-    init_svg_element(&ret->svg_element, doc, nselem);
+    init_svg_element(&ret->svg_element, doc, nselem, &SVGSVGElement_dispex);
 
     *elem = &ret->svg_element.element;
     return S_OK;
@@ -883,48 +905,60 @@ static const ISVGCircleElementVtbl SVGCircleElementVtbl = {
     SVGCircleElement_get_r
 };
 
-static inline SVGCircleElement *SVGCircleElement_from_HTMLDOMNode(HTMLDOMNode *iface)
+static inline SVGCircleElement *SVGCircleElement_from_DispatchEx(DispatchEx *iface)
 {
-    return CONTAINING_RECORD(iface, SVGCircleElement, svg_element.element.node);
+    return CONTAINING_RECORD(iface, SVGCircleElement, svg_element.element.node.event_target.dispex);
 }
 
-static HRESULT SVGCircleElement_QI(HTMLDOMNode *iface, REFIID riid, void **ppv)
+static void *SVGCircleElement_query_interface(DispatchEx *dispex, REFIID riid)
 {
-    SVGCircleElement *This = SVGCircleElement_from_HTMLDOMNode(iface);
-
-    TRACE("(%p)->(%s %p)\n", This, debugstr_mshtml_guid(riid), ppv);
+    SVGCircleElement *This = SVGCircleElement_from_DispatchEx(dispex);
 
     if(IsEqualGUID(&IID_ISVGCircleElement, riid))
-        *ppv = &This->ISVGCircleElement_iface;
-    else
-        return SVGElement_QI(&This->svg_element.element.node, riid, ppv);
+        return &This->ISVGCircleElement_iface;
 
-    IUnknown_AddRef((IUnknown*)*ppv);
-    return S_OK;
+    return SVGElement_query_interface(&This->svg_element.element.node.event_target.dispex, riid);
 }
 
 static const NodeImplVtbl SVGCircleElementImplVtbl = {
-    &CLSID_SVGCircleElement,
-    SVGCircleElement_QI,
-    HTMLElement_destructor,
-    HTMLElement_cpc,
-    HTMLElement_clone,
-    NULL,
-    HTMLElement_get_attr_col,
+    .clsid                 = &CLSID_SVGCircleElement,
+    .cpc_entries           = HTMLElement_cpc,
+    .clone                 = HTMLElement_clone,
+    .get_attr_col          = HTMLElement_get_attr_col,
+};
+
+static const event_target_vtbl_t SVGCircleElement_event_target_vtbl = {
+    {
+        HTMLELEMENT_DISPEX_VTBL_ENTRIES,
+        .query_interface= SVGCircleElement_query_interface,
+        .destructor     = HTMLElement_destructor,
+        .traverse       = HTMLElement_traverse,
+        .unlink         = HTMLElement_unlink
+    },
+    HTMLELEMENT_EVENT_TARGET_VTBL_ENTRIES,
+    .handle_event       = HTMLElement_handle_event
+};
+
+static dispex_static_data_t SVGCircleElement_dispex = {
+    "HTMLUnknownElement",
+    &SVGCircleElement_event_target_vtbl.dispex_vtbl,
+    DispHTMLUnknownElement_tid,
+    HTMLElement_iface_tids,
+    HTMLElement_init_dispex_info
 };
 
 static HRESULT create_circle_element(HTMLDocumentNode *doc, nsIDOMSVGElement *nselem, HTMLElement **elem)
 {
     SVGCircleElement *ret;
 
-    ret = heap_alloc_zero(sizeof(SVGCircleElement));
+    ret = calloc(1, sizeof(SVGCircleElement));
     if(!ret)
         return E_OUTOFMEMORY;
 
     ret->ISVGCircleElement_iface.lpVtbl = &SVGCircleElementVtbl;
     ret->svg_element.element.node.vtbl = &SVGCircleElementImplVtbl;
 
-    init_svg_element(&ret->svg_element, doc, nselem);
+    init_svg_element(&ret->svg_element, doc, nselem, &SVGCircleElement_dispex);
 
     *elem = &ret->svg_element.element;
     return S_OK;
@@ -1039,7 +1073,7 @@ static HRESULT WINAPI SVGTextContentElement_getSubStringLength(ISVGTextContentEl
                                                                LONG charnum, LONG nchars, float *p)
 {
     SVGTextContentElement *This = impl_from_ISVGTextContentElement(iface);
-    FIXME("(%p)->(%d %d %p)\n", This, charnum, nchars, p);
+    FIXME("(%p)->(%ld %ld %p)\n", This, charnum, nchars, p);
     return E_NOTIMPL;
 }
 
@@ -1047,7 +1081,7 @@ static HRESULT WINAPI SVGTextContentElement_getStartPositionOfChar(ISVGTextConte
                                                                    LONG charnum, ISVGPoint **p)
 {
     SVGTextContentElement *This = impl_from_ISVGTextContentElement(iface);
-    FIXME("(%p)->(%d %p)\n", This, charnum, p);
+    FIXME("(%p)->(%ld %p)\n", This, charnum, p);
     return E_NOTIMPL;
 }
 
@@ -1055,7 +1089,7 @@ static HRESULT WINAPI SVGTextContentElement_getEndPositionOfChar(ISVGTextContent
                                                                  LONG charnum, ISVGPoint **p)
 {
     SVGTextContentElement *This = impl_from_ISVGTextContentElement(iface);
-    FIXME("(%p)->(%d %p)\n", This, charnum, p);
+    FIXME("(%p)->(%ld %p)\n", This, charnum, p);
     return E_NOTIMPL;
 }
 
@@ -1063,7 +1097,7 @@ static HRESULT WINAPI SVGTextContentElement_getExtentOfChar(ISVGTextContentEleme
                                                             LONG charnum, ISVGRect **p)
 {
     SVGTextContentElement *This = impl_from_ISVGTextContentElement(iface);
-    FIXME("(%p)->(%d %p)\n", This, charnum, p);
+    FIXME("(%p)->(%ld %p)\n", This, charnum, p);
     return E_NOTIMPL;
 }
 
@@ -1071,7 +1105,7 @@ static HRESULT WINAPI SVGTextContentElement_getRotationOfChar(ISVGTextContentEle
                                                               LONG charnum, float *p)
 {
     SVGTextContentElement *This = impl_from_ISVGTextContentElement(iface);
-    FIXME("(%p)->(%d %p)\n", This, charnum, p);
+    FIXME("(%p)->(%ld %p)\n", This, charnum, p);
     return E_NOTIMPL;
 }
 
@@ -1087,7 +1121,7 @@ static HRESULT WINAPI SVGTextContentElement_selectSubString(ISVGTextContentEleme
                                                             LONG charnum, LONG nchars)
 {
     SVGTextContentElement *This = impl_from_ISVGTextContentElement(iface);
-    FIXME("(%p)->(%d %d)\n", This, charnum, nchars);
+    FIXME("(%p)->(%ld %ld)\n", This, charnum, nchars);
     return E_NOTIMPL;
 }
 
@@ -1125,76 +1159,85 @@ struct SVGTSpanElement {
     SVGTextContentElement text_content;
 };
 
-static inline SVGTSpanElement *SVGTSpanElement_from_HTMLDOMNode(HTMLDOMNode *iface)
+static inline SVGTSpanElement *SVGTSpanElement_from_DispatchEx(DispatchEx *iface)
 {
-    return CONTAINING_RECORD(iface, SVGTSpanElement, svg_element.element.node);
+    return CONTAINING_RECORD(iface, SVGTSpanElement, svg_element.element.node.event_target.dispex);
 }
 
-static HRESULT SVGTSpanElement_QI(HTMLDOMNode *iface, REFIID riid, void **ppv)
+static void *SVGTSpanElement_query_interface(DispatchEx *dispex, REFIID riid)
 {
-    SVGTSpanElement *This = SVGTSpanElement_from_HTMLDOMNode(iface);
-
-    TRACE("(%p)->(%s %p)\n", This, debugstr_mshtml_guid(riid), ppv);
+    SVGTSpanElement *This = SVGTSpanElement_from_DispatchEx(dispex);
 
     if(IsEqualGUID(&IID_ISVGTSpanElement, riid))
-        *ppv = &This->svg_element.ISVGElement_iface; /* no additional methods */
-    else if(IsEqualGUID(&IID_ISVGTextContentElement, riid))
-        *ppv = &This->text_content.ISVGTextContentElement_iface;
-    else
-        return SVGElement_QI(&This->svg_element.element.node, riid, ppv);
+        return &This->svg_element.ISVGElement_iface; /* no additional methods */
+    if(IsEqualGUID(&IID_ISVGTextContentElement, riid))
+        return &This->text_content.ISVGTextContentElement_iface;
 
-    IUnknown_AddRef((IUnknown*)*ppv);
-    return S_OK;
+    return SVGElement_query_interface(&This->svg_element.element.node.event_target.dispex, riid);
 }
 
 static const NodeImplVtbl SVGTSpanElementImplVtbl = {
-    &CLSID_SVGTSpanElement,
-    SVGTSpanElement_QI,
-    HTMLElement_destructor,
-    HTMLElement_cpc,
-    HTMLElement_clone,
-    NULL,
-    HTMLElement_get_attr_col,
+    .clsid                 = &CLSID_SVGTSpanElement,
+    .cpc_entries           = HTMLElement_cpc,
+    .clone                 = HTMLElement_clone,
+    .get_attr_col          = HTMLElement_get_attr_col,
+};
+
+static const event_target_vtbl_t SVGTSpanElement_event_target_vtbl = {
+    {
+        HTMLELEMENT_DISPEX_VTBL_ENTRIES,
+        .query_interface= SVGTSpanElement_query_interface,
+        .destructor     = HTMLElement_destructor,
+        .traverse       = HTMLElement_traverse,
+        .unlink         = HTMLElement_unlink
+    },
+    HTMLELEMENT_EVENT_TARGET_VTBL_ENTRIES,
+    .handle_event       = HTMLElement_handle_event
+};
+
+static dispex_static_data_t SVGTSpanElement_dispex = {
+    "HTMLUnknownElement",
+    &SVGTSpanElement_event_target_vtbl.dispex_vtbl,
+    DispHTMLUnknownElement_tid,
+    HTMLElement_iface_tids,
+    HTMLElement_init_dispex_info
 };
 
 static HRESULT create_tspan_element(HTMLDocumentNode *doc, nsIDOMSVGElement *nselem, HTMLElement **elem)
 {
     SVGTSpanElement *ret;
 
-    ret = heap_alloc_zero(sizeof(SVGTSpanElement));
+    ret = calloc(1, sizeof(SVGTSpanElement));
     if(!ret)
         return E_OUTOFMEMORY;
 
     ret->svg_element.element.node.vtbl = &SVGTSpanElementImplVtbl;
     init_text_content_element(&ret->text_content, &ret->svg_element);
-    init_svg_element(&ret->svg_element, doc, nselem);
+    init_svg_element(&ret->svg_element, doc, nselem, &SVGTSpanElement_dispex);
 
     *elem = &ret->svg_element.element;
     return S_OK;
 }
 
-static const WCHAR circleW[] = {'c','i','r','c','l','e',0};
-static const WCHAR svgW[] = {'s','v','g',0};
-static const WCHAR tspanW[] = {'t','s','p','a','n',0};
-
 HRESULT create_svg_element(HTMLDocumentNode *doc, nsIDOMSVGElement *dom_element, const WCHAR *tag_name, HTMLElement **elem)
 {
     SVGElement *svg_element;
 
     TRACE("%s\n", debugstr_w(tag_name));
 
-    if(!wcscmp(tag_name, svgW))
+    if(!wcscmp(tag_name, L"svg"))
         return create_viewport_element(doc, dom_element, elem);
-    if(!wcscmp(tag_name, circleW))
+    if(!wcscmp(tag_name, L"circle"))
         return create_circle_element(doc, dom_element, elem);
-    if(!wcscmp(tag_name, tspanW))
+    if(!wcscmp(tag_name, L"tspan"))
         return create_tspan_element(doc, dom_element, elem);
 
-    svg_element = heap_alloc_zero(sizeof(*svg_element));
+    svg_element = calloc(1, sizeof(*svg_element));
     if(!svg_element)
         return E_OUTOFMEMORY;
 
-    init_svg_element(svg_element, doc, dom_element);
+    svg_element->element.node.vtbl = &SVGElementImplVtbl;
+    init_svg_element(svg_element, doc, dom_element, &SVGElement_dispex);
     *elem = &svg_element->element;
     return S_OK;
 }
