@@ -18,13 +18,10 @@
  */
 
 #define COBJMACROS
-#define NONAMELESSUNION
-#define NONAMELESSSTRUCT
 
 #include <stdarg.h>
 #include "windef.h"
 #include "winbase.h"
-#define USE_WS_PREFIX
 #include "winsock2.h"
 #include "ws2ipdef.h"
 #include "iphlpapi.h"
@@ -37,7 +34,6 @@
 #include "olectl.h"
 
 #include "wine/debug.h"
-#include "wine/heap.h"
 #include "wine/list.h"
 #include "netprofm_private.h"
 
@@ -45,12 +41,13 @@ WINE_DEFAULT_DEBUG_CHANNEL(netprofm);
 
 struct network
 {
-    INetwork     INetwork_iface;
-    LONG         refs;
-    struct list  entry;
-    GUID         id;
-    VARIANT_BOOL connected_to_internet;
-    VARIANT_BOOL connected;
+    INetwork             INetwork_iface;
+    LONG                 refs;
+    struct list          entry;
+    GUID                 id;
+    INetworkListManager *mgr;
+    VARIANT_BOOL         connected_to_internet;
+    VARIANT_BOOL         connected;
 };
 
 struct connection
@@ -85,6 +82,7 @@ struct list_manager
     struct connection_point list_mgr_cp;
     struct connection_point cost_mgr_cp;
     struct connection_point conn_mgr_cp;
+    struct connection_point events_cp;
 };
 
 struct sink_entry
@@ -199,7 +197,7 @@ static HRESULT WINAPI connection_point_Advise(
         return CO_E_FAILEDTOOPENTHREADTOKEN;
     }
 
-    sink_entry = heap_alloc( sizeof(*sink_entry) );
+    sink_entry = malloc( sizeof(*sink_entry) );
     if (!sink_entry)
     {
         IUnknown_Release( unk );
@@ -216,7 +214,7 @@ static void sink_entry_release( struct sink_entry *entry )
 {
     list_remove( &entry->entry );
     IUnknown_Release( entry->unk );
-    heap_free( entry );
+    free( entry );
 }
 
 static HRESULT WINAPI connection_point_Unadvise(
@@ -226,7 +224,7 @@ static HRESULT WINAPI connection_point_Unadvise(
     struct connection_point *cp = impl_from_IConnectionPoint( iface );
     struct sink_entry *iter;
 
-    TRACE( "%p, %d\n", cp, cookie );
+    TRACE( "%p, %ld\n", cp, cookie );
 
     LIST_FOR_EACH_ENTRY( iter, &cp->sinks, struct sink_entry, entry )
     {
@@ -328,7 +326,8 @@ static ULONG WINAPI network_Release(
     if (!(refs = InterlockedDecrement( &network->refs )))
     {
         list_remove( &network->entry );
-        heap_free( network );
+        INetworkListManager_Release( network->mgr );
+        free( network );
     }
     return refs;
 }
@@ -432,12 +431,24 @@ static HRESULT WINAPI network_GetDomainType(
     return S_OK;
 }
 
+static inline struct list_manager *impl_from_INetworkListManager(
+    INetworkListManager *iface )
+{
+    return CONTAINING_RECORD( iface, struct list_manager, INetworkListManager_iface );
+}
+
+static HRESULT create_connections_enum(
+    struct list_manager *, IEnumNetworkConnections** );
+
 static HRESULT WINAPI network_GetNetworkConnections(
     INetwork *iface,
-    IEnumNetworkConnections **ppEnumNetworkConnection )
+    IEnumNetworkConnections **ppEnum )
 {
-    FIXME( "%p, %p\n", iface, ppEnumNetworkConnection );
-    return E_NOTIMPL;
+    struct network *network = impl_from_INetwork( iface );
+    struct list_manager *mgr = impl_from_INetworkListManager( network->mgr );
+
+    TRACE( "%p, %p\n", iface, ppEnum );
+    return create_connections_enum( mgr, ppEnum );
 }
 
 static HRESULT WINAPI network_GetTimeCreatedAndConnected(
@@ -480,9 +491,17 @@ static HRESULT WINAPI network_GetConnectivity(
     INetwork *iface,
     NLM_CONNECTIVITY *pConnectivity )
 {
+    struct network *network = impl_from_INetwork( iface );
+
     FIXME( "%p, %p\n", iface, pConnectivity );
 
-    *pConnectivity = NLM_CONNECTIVITY_IPV4_INTERNET;
+    *pConnectivity = NLM_CONNECTIVITY_DISCONNECTED;
+
+    if (network->connected_to_internet)
+        *pConnectivity |= NLM_CONNECTIVITY_IPV4_INTERNET;
+    else if (network->connected)
+        *pConnectivity |= NLM_CONNECTIVITY_IPV4_LOCALNETWORK;
+
     return S_OK;
 }
 
@@ -532,7 +551,7 @@ static struct network *create_network( const GUID *id )
 {
     struct network *ret;
 
-    if (!(ret = heap_alloc( sizeof(*ret) ))) return NULL;
+    if (!(ret = calloc( 1, sizeof(*ret) ))) return NULL;
 
     ret->INetwork_iface.lpVtbl = &network_vtbl;
     ret->refs                  = 1;
@@ -582,14 +601,14 @@ static BOOL map_address_6to4( const SOCKADDR_IN6 *addr6, SOCKADDR_IN *addr4 )
 {
     ULONG i;
 
-    if (addr6->sin6_family != WS_AF_INET6) return FALSE;
+    if (addr6->sin6_family != AF_INET6) return FALSE;
 
     for (i = 0; i < 5; i++)
         if (addr6->sin6_addr.u.Word[i]) return FALSE;
 
     if (addr6->sin6_addr.u.Word[5] != 0xffff) return FALSE;
 
-    addr4->sin_family = WS_AF_INET;
+    addr4->sin_family = AF_INET;
     addr4->sin_port   = addr6->sin6_port;
     addr4->sin_addr.S_un.S_addr = addr6->sin6_addr.u.Word[6] << 16 | addr6->sin6_addr.u.Word[7];
     memset( &addr4->sin_zero, 0, sizeof(addr4->sin_zero) );
@@ -610,8 +629,8 @@ static HRESULT WINAPI cost_manager_GetDataPlanStatus(
 
     if (!pDataPlanStatus) return E_POINTER;
 
-    if (dst && ((dst->sa_family == WS_AF_INET && (dst4 = (SOCKADDR_IN *)dst)) ||
-               ((dst->sa_family == WS_AF_INET6 && map_address_6to4( (const SOCKADDR_IN6 *)dst, &addr4 )
+    if (dst && ((dst->sa_family == AF_INET && (dst4 = (SOCKADDR_IN *)dst)) ||
+               ((dst->sa_family == AF_INET6 && map_address_6to4( (const SOCKADDR_IN6 *)dst, &addr4 )
                 && (dst4 = &addr4)))))
     {
         if ((ret = GetBestInterface( dst4->sin_addr.S_un.S_addr, &index )))
@@ -665,6 +684,7 @@ struct networks_enum
     LONG                 refs;
     struct list_manager *mgr;
     struct list         *cursor;
+    NLM_ENUM_NETWORK     flags;
 };
 
 static inline struct networks_enum *impl_from_IEnumNetworks(
@@ -716,7 +736,7 @@ static ULONG WINAPI networks_enum_Release(
     if (!(refs = InterlockedDecrement( &iter->refs )))
     {
         INetworkListManager_Release( &iter->mgr->INetworkListManager_iface );
-        heap_free( iter );
+        free( iter );
     }
     return refs;
 }
@@ -773,24 +793,40 @@ static HRESULT WINAPI networks_enum_get__NewEnum(
     return E_NOTIMPL;
 }
 
+static BOOL match_enum_network_flags( NLM_ENUM_NETWORK flags, struct network *network )
+{
+    if (flags == NLM_ENUM_NETWORK_ALL) return TRUE;
+    if (network->connected)
+    {
+        if (flags & NLM_ENUM_NETWORK_CONNECTED) return TRUE;
+    }
+    else if (flags & NLM_ENUM_NETWORK_DISCONNECTED) return TRUE;
+    return FALSE;
+}
+
 static HRESULT WINAPI networks_enum_Next(
     IEnumNetworks *iface, ULONG count, INetwork **ret, ULONG *fetched )
 {
     struct networks_enum *iter = impl_from_IEnumNetworks( iface );
     ULONG i = 0;
 
-    TRACE( "%p, %u %p %p\n", iter, count, ret, fetched );
+    TRACE( "%p, %lu %p %p\n", iter, count, ret, fetched );
 
+    if (!ret) return E_POINTER;
+    *ret = NULL;
     if (fetched) *fetched = 0;
     if (!count) return S_OK;
 
     while (iter->cursor && i < count)
     {
         struct network *network = LIST_ENTRY( iter->cursor, struct network, entry );
-        ret[i] = &network->INetwork_iface;
-        INetwork_AddRef( ret[i] );
+        if (match_enum_network_flags( iter->flags, network ))
+        {
+            ret[i] = &network->INetwork_iface;
+            INetwork_AddRef( ret[i] );
+            i++;
+        }
         iter->cursor = list_next( &iter->mgr->networks, iter->cursor );
-        i++;
     }
     if (fetched) *fetched = i;
 
@@ -802,15 +838,19 @@ static HRESULT WINAPI networks_enum_Skip(
 {
     struct networks_enum *iter = impl_from_IEnumNetworks( iface );
 
-    TRACE( "%p, %u\n", iter, count);
+    TRACE( "%p, %lu\n", iter, count);
 
     if (!count) return S_OK;
     if (!iter->cursor) return S_FALSE;
 
-    while (count--)
+    for (;;)
     {
+        struct network *network;
         iter->cursor = list_next( &iter->mgr->networks, iter->cursor );
         if (!iter->cursor) break;
+        network = LIST_ENTRY( iter->cursor, struct network, entry );
+        if (match_enum_network_flags( iter->flags, network )) count--;
+        if (!count) break;
     }
 
     return count ? S_FALSE : S_OK;
@@ -828,7 +868,7 @@ static HRESULT WINAPI networks_enum_Reset(
 }
 
 static HRESULT create_networks_enum(
-    struct list_manager *, IEnumNetworks** );
+    struct list_manager *, NLM_ENUM_NETWORK, IEnumNetworks ** );
 
 static HRESULT WINAPI networks_enum_Clone(
     IEnumNetworks *iface, IEnumNetworks **ret )
@@ -836,7 +876,7 @@ static HRESULT WINAPI networks_enum_Clone(
     struct networks_enum *iter = impl_from_IEnumNetworks( iface );
 
     TRACE( "%p, %p\n", iter, ret );
-    return create_networks_enum( iter->mgr, ret );
+    return create_networks_enum( iter->mgr, iter->flags, ret );
 }
 
 static const IEnumNetworksVtbl networks_enum_vtbl =
@@ -856,29 +896,24 @@ static const IEnumNetworksVtbl networks_enum_vtbl =
 };
 
 static HRESULT create_networks_enum(
-    struct list_manager *mgr, IEnumNetworks **ret )
+    struct list_manager *mgr, NLM_ENUM_NETWORK flags, IEnumNetworks **ret )
 {
     struct networks_enum *iter;
 
     *ret = NULL;
-    if (!(iter = heap_alloc( sizeof(*iter) ))) return E_OUTOFMEMORY;
+    if (!(iter = calloc( 1, sizeof(*iter) ))) return E_OUTOFMEMORY;
 
     iter->IEnumNetworks_iface.lpVtbl = &networks_enum_vtbl;
     iter->cursor = list_head( &mgr->networks );
     iter->mgr    = mgr;
     INetworkListManager_AddRef( &mgr->INetworkListManager_iface );
+    iter->flags  = flags;
     iter->refs   = 1;
 
     *ret = &iter->IEnumNetworks_iface;
     return S_OK;
 }
 
-static inline struct list_manager *impl_from_INetworkListManager(
-    INetworkListManager *iface )
-{
-    return CONTAINING_RECORD( iface, struct list_manager, INetworkListManager_iface );
-}
-
 struct connections_enum
 {
     IEnumNetworkConnections IEnumNetworkConnections_iface;
@@ -936,7 +971,7 @@ static ULONG WINAPI connections_enum_Release(
     if (!(refs = InterlockedDecrement( &iter->refs )))
     {
         INetworkListManager_Release( &iter->mgr->INetworkListManager_iface );
-        heap_free( iter );
+        free( iter );
     }
     return refs;
 }
@@ -999,8 +1034,10 @@ static HRESULT WINAPI connections_enum_Next(
     struct connections_enum *iter = impl_from_IEnumNetworkConnections( iface );
     ULONG i = 0;
 
-    TRACE( "%p, %u %p %p\n", iter, count, ret, fetched );
+    TRACE( "%p, %lu %p %p\n", iter, count, ret, fetched );
 
+    if (!ret) return E_POINTER;
+    *ret = NULL;
     if (fetched) *fetched = 0;
     if (!count) return S_OK;
 
@@ -1022,7 +1059,7 @@ static HRESULT WINAPI connections_enum_Skip(
 {
     struct connections_enum *iter = impl_from_IEnumNetworkConnections( iface );
 
-    TRACE( "%p, %u\n", iter, count);
+    TRACE( "%p, %lu\n", iter, count);
 
     if (!count) return S_OK;
     if (!iter->cursor) return S_FALSE;
@@ -1047,9 +1084,6 @@ static HRESULT WINAPI connections_enum_Reset(
     return S_OK;
 }
 
-static HRESULT create_connections_enum(
-    struct list_manager *, IEnumNetworkConnections** );
-
 static HRESULT WINAPI connections_enum_Clone(
     IEnumNetworkConnections *iface, IEnumNetworkConnections **ret )
 {
@@ -1081,7 +1115,7 @@ static HRESULT create_connections_enum(
     struct connections_enum *iter;
 
     *ret = NULL;
-    if (!(iter = heap_alloc( sizeof(*iter) ))) return E_OUTOFMEMORY;
+    if (!(iter = calloc( 1, sizeof(*iter) ))) return E_OUTOFMEMORY;
 
     iter->IEnumNetworkConnections_iface.lpVtbl = &connections_enum_vtbl;
     iter->mgr         = mgr;
@@ -1107,26 +1141,24 @@ static ULONG WINAPI list_manager_Release(
     LONG refs = InterlockedDecrement( &mgr->refs );
     if (!refs)
     {
-        struct list *ptr;
+        struct network *network, *next_network;
+        struct connection *connection, *next_connection;
 
         TRACE( "destroying %p\n", mgr );
 
+        connection_point_release( &mgr->events_cp );
         connection_point_release( &mgr->conn_mgr_cp );
         connection_point_release( &mgr->cost_mgr_cp );
         connection_point_release( &mgr->list_mgr_cp );
-        while ((ptr = list_head( &mgr->networks )))
+        LIST_FOR_EACH_ENTRY_SAFE( connection, next_connection, &mgr->connections, struct connection, entry )
         {
-            struct network *network = LIST_ENTRY( ptr, struct network, entry );
-            list_remove( &network->entry );
-            INetwork_Release( &network->INetwork_iface );
-        }
-        while ((ptr = list_head( &mgr->connections )))
-        {
-            struct connection *connection = LIST_ENTRY( ptr, struct connection, entry );
-            list_remove( &connection->entry );
             INetworkConnection_Release( &connection->INetworkConnection_iface );
         }
-        heap_free( mgr );
+        LIST_FOR_EACH_ENTRY_SAFE( network, next_network, &mgr->networks, struct network, entry )
+        {
+            INetwork_Release( &network->INetwork_iface );
+        }
+        free( mgr );
     }
     return refs;
 }
@@ -1217,9 +1249,8 @@ static HRESULT WINAPI list_manager_GetNetworks(
     struct list_manager *mgr = impl_from_INetworkListManager( iface );
 
     TRACE( "%p, %x, %p\n", iface, Flags, ppEnumNetwork );
-    if (Flags) FIXME( "flags %08x not supported\n", Flags );
 
-    return create_networks_enum( mgr, ppEnumNetwork );
+    return create_networks_enum( mgr, Flags, ppEnumNetwork );
 }
 
 static HRESULT WINAPI list_manager_GetNetwork(
@@ -1327,9 +1358,21 @@ static HRESULT WINAPI list_manager_GetConnectivity(
     INetworkListManager *iface,
     NLM_CONNECTIVITY *pConnectivity )
 {
+    struct list_manager *mgr = impl_from_INetworkListManager( iface );
+    struct network *network;
+
     FIXME( "%p, %p\n", iface, pConnectivity );
 
-    *pConnectivity = NLM_CONNECTIVITY_IPV4_INTERNET;
+    *pConnectivity = NLM_CONNECTIVITY_DISCONNECTED;
+
+    LIST_FOR_EACH_ENTRY( network, &mgr->networks, struct network, entry )
+    {
+        if (network->connected_to_internet)
+            *pConnectivity |= NLM_CONNECTIVITY_IPV4_INTERNET;
+        else if (network->connected)
+            *pConnectivity |= NLM_CONNECTIVITY_IPV4_LOCALNETWORK;
+    }
+
     return S_OK;
 }
 
@@ -1395,6 +1438,8 @@ static HRESULT WINAPI ConnectionPointContainer_FindConnectionPoint(IConnectionPo
         ret = &This->cost_mgr_cp;
     else if (IsEqualGUID( riid, &IID_INetworkConnectionEvents ))
         ret = &This->conn_mgr_cp;
+    else if (IsEqualGUID( riid, &IID_INetworkEvents))
+        ret = &This->events_cp;
     else
     {
         FIXME( "interface %s not implemented\n", debugstr_guid(riid) );
@@ -1467,9 +1512,9 @@ static ULONG WINAPI connection_Release(
 
     if (!(refs = InterlockedDecrement( &connection->refs )))
     {
-        INetwork_Release( connection->network );
         list_remove( &connection->entry );
-        heap_free( connection );
+        INetwork_Release( connection->network );
+        free( connection );
     }
     return refs;
 }
@@ -1560,9 +1605,17 @@ static HRESULT WINAPI connection_GetConnectivity(
     INetworkConnection *iface,
     NLM_CONNECTIVITY *pConnectivity )
 {
+    struct connection *connection = impl_from_INetworkConnection( iface );
+
     FIXME( "%p, %p\n", iface, pConnectivity );
 
-    *pConnectivity = NLM_CONNECTIVITY_IPV4_INTERNET;
+    *pConnectivity = NLM_CONNECTIVITY_DISCONNECTED;
+
+    if (connection->connected_to_internet)
+        *pConnectivity |= NLM_CONNECTIVITY_IPV4_INTERNET;
+    else if (connection->connected)
+        *pConnectivity |= NLM_CONNECTIVITY_IPV4_LOCALNETWORK;
+
     return S_OK;
 }
 
@@ -1693,7 +1746,7 @@ static struct connection *create_connection( const GUID *id )
 {
     struct connection *ret;
 
-    if (!(ret = heap_alloc( sizeof(*ret) ))) return NULL;
+    if (!(ret = calloc( 1, sizeof(*ret) ))) return NULL;
 
     ret->INetworkConnection_iface.lpVtbl     = &connection_vtbl;
     ret->INetworkConnectionCost_iface.lpVtbl = &connection_cost_vtbl;
@@ -1707,34 +1760,44 @@ static struct connection *create_connection( const GUID *id )
     return ret;
 }
 
+static IP_ADAPTER_ADDRESSES *get_network_adapters(void)
+{
+    ULONG err, size = 4096, flags = GAA_FLAG_SKIP_ANYCAST | GAA_FLAG_SKIP_MULTICAST |
+                                    GAA_FLAG_SKIP_DNS_SERVER | GAA_FLAG_INCLUDE_GATEWAYS;
+    IP_ADAPTER_ADDRESSES *tmp, *ret;
+
+    if (!(ret = malloc( size ))) return NULL;
+    err = GetAdaptersAddresses( AF_UNSPEC, flags, NULL, ret, &size );
+    while (err == ERROR_BUFFER_OVERFLOW)
+    {
+        if (!(tmp = realloc( ret, size ))) break;
+        ret = tmp;
+        err = GetAdaptersAddresses( AF_UNSPEC, flags, NULL, ret, &size );
+    }
+    if (err == ERROR_SUCCESS) return ret;
+    free( ret );
+    return NULL;
+}
+
 static void init_networks( struct list_manager *mgr )
 {
-    DWORD size = 0;
     IP_ADAPTER_ADDRESSES *buf, *aa;
     GUID id;
-    ULONG ret, flags = GAA_FLAG_SKIP_ANYCAST | GAA_FLAG_SKIP_MULTICAST |
-                       GAA_FLAG_SKIP_DNS_SERVER | GAA_FLAG_INCLUDE_ALL_GATEWAYS;
 
     list_init( &mgr->networks );
     list_init( &mgr->connections );
 
-    ret = GetAdaptersAddresses( WS_AF_UNSPEC, flags, NULL, NULL, &size );
-    if (ret != ERROR_BUFFER_OVERFLOW) return;
-
-    if (!(buf = heap_alloc( size ))) return;
-    if (GetAdaptersAddresses( WS_AF_UNSPEC, flags, NULL, buf, &size ))
-    {
-        heap_free( buf );
-        return;
-    }
+    if (!(buf = get_network_adapters())) return;
 
     memset( &id, 0, sizeof(id) );
     for (aa = buf; aa; aa = aa->Next)
     {
         struct network *network;
         struct connection *connection;
+        NET_LUID luid;
 
-        id.Data1 = aa->u.s.IfIndex;
+        ConvertInterfaceIndexToLuid(aa->IfIndex, &luid);
+        ConvertInterfaceLuidToGuid(&luid, &id);
 
         /* assume a one-to-one mapping between networks and connections */
         if (!(network = create_network( &id ))) goto done;
@@ -1755,6 +1818,8 @@ static void init_networks( struct list_manager *mgr )
             connection->connected_to_internet = VARIANT_TRUE;
         }
 
+        network->mgr = &mgr->INetworkListManager_iface;
+        INetworkListManager_AddRef( network->mgr );
         connection->network = &network->INetwork_iface;
         INetwork_AddRef( connection->network );
 
@@ -1763,7 +1828,7 @@ static void init_networks( struct list_manager *mgr )
     }
 
 done:
-    heap_free( buf );
+    free( buf );
 }
 
 HRESULT list_manager_create( void **obj )
@@ -1772,7 +1837,7 @@ HRESULT list_manager_create( void **obj )
 
     TRACE( "%p\n", obj );
 
-    if (!(mgr = heap_alloc( sizeof(*mgr) ))) return E_OUTOFMEMORY;
+    if (!(mgr = calloc( 1, sizeof(*mgr) ))) return E_OUTOFMEMORY;
     mgr->INetworkListManager_iface.lpVtbl = &list_manager_vtbl;
     mgr->INetworkCostManager_iface.lpVtbl = &cost_manager_vtbl;
     mgr->IConnectionPointContainer_iface.lpVtbl = &cpc_vtbl;
@@ -1785,6 +1850,8 @@ HRESULT list_manager_create( void **obj )
                            &mgr->IConnectionPointContainer_iface);
     connection_point_init( &mgr->conn_mgr_cp, &IID_INetworkConnectionEvents,
                            &mgr->IConnectionPointContainer_iface );
+    connection_point_init( &mgr->events_cp, &IID_INetworkEvents,
+                           &mgr->IConnectionPointContainer_iface );
 
     *obj = &mgr->INetworkListManager_iface;
     TRACE( "returning iface %p\n", *obj );
