@@ -21,9 +21,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
-#include "config.h"
-#include "wine/port.h"
-
 #include <string.h>
 #include <stdlib.h>
 #include <stdarg.h>
@@ -31,7 +28,6 @@
 #include "ntstatus.h"
 #define WIN32_NO_STATUS
 #include "windef.h"
-#include "wine/unicode.h"
 #include "wine/debug.h"
 #include "ntdll_misc.h"
 #include "ddk/wdm.h"
@@ -59,6 +55,7 @@ typedef enum
     WIN8,    /* Windows 8 */
     WIN81,   /* Windows 8.1 */
     WIN10,   /* Windows 10 */
+    WIN11,   /* Windows 11 */
     NB_WINDOWS_VERSIONS
 } WINDOWS_VERSION;
 
@@ -70,20 +67,17 @@ static const RTL_OSVERSIONINFOEXW VersionData[NB_WINDOWS_VERSIONS] =
     /* WIN20 FIXME: verify values */
     {
         sizeof(RTL_OSVERSIONINFOEXW), 2, 0, 0, VER_PLATFORM_WIN32s,
-        {'W','i','n','3','2','s',' ','1','.','3',0},
-        0, 0, 0, 0, 0
+        L"Win32s 1.3", 0, 0, 0, 0, 0
     },
     /* WIN30 FIXME: verify values */
     {
         sizeof(RTL_OSVERSIONINFOEXW), 3, 0, 0, VER_PLATFORM_WIN32s,
-        {'W','i','n','3','2','s',' ','1','.','3',0},
-        0, 0, 0, 0, 0
+        L"Win32s 1.3", 0, 0, 0, 0, 0
     },
     /* WIN31 */
     {
         sizeof(RTL_OSVERSIONINFOEXW), 3, 10, 0, VER_PLATFORM_WIN32s,
-        {'W','i','n','3','2','s',' ','1','.','3',0},
-        0, 0, 0, 0, 0
+        L"Win32s 1.3", 0, 0, 0, 0, 0
     },
     /* WIN95 */
     {
@@ -95,8 +89,7 @@ static const RTL_OSVERSIONINFOEXW VersionData[NB_WINDOWS_VERSIONS] =
          * Win95a/b can be discerned via regkey SubVersionNumber
          */
         sizeof(RTL_OSVERSIONINFOEXW), 4, 0, 0x40003B6, VER_PLATFORM_WIN32_WINDOWS,
-        {0},
-        0, 0, 0, 0, 0
+        L"", 0, 0, 0, 0, 0
     },
     /* WIN98 (second edition) */
     {
@@ -104,127 +97,157 @@ static const RTL_OSVERSIONINFOEXW VersionData[NB_WINDOWS_VERSIONS] =
          * Win98SE: 4, 10, 0x40A08AE, " A " 4.10.2222
          */
         sizeof(RTL_OSVERSIONINFOEXW), 4, 10, 0x40A08AE, VER_PLATFORM_WIN32_WINDOWS,
-        {' ','A',' ',0},
-        0, 0, 0, 0, 0
+        L" A ", 0, 0, 0, 0, 0
     },
     /* WINME */
     {
         sizeof(RTL_OSVERSIONINFOEXW), 4, 90, 0x45A0BB8, VER_PLATFORM_WIN32_WINDOWS,
-        {' ',0},
-        0, 0, 0, 0, 0
+        L" ", 0, 0, 0, 0, 0
     },
     /* NT351 */
     {
-        sizeof(RTL_OSVERSIONINFOEXW), 3, 51, 0x421, VER_PLATFORM_WIN32_NT,
-        {'S','e','r','v','i','c','e',' ','P','a','c','k',' ','5',0},
-        5, 0, 0, VER_NT_WORKSTATION, 0
+        sizeof(RTL_OSVERSIONINFOEXW), 3, 51, 1057, VER_PLATFORM_WIN32_NT,
+        L"Service Pack 5", 5, 0, 0, VER_NT_WORKSTATION, 0
     },
     /* NT40 */
     {
-        sizeof(RTL_OSVERSIONINFOEXW), 4, 0, 0x565, VER_PLATFORM_WIN32_NT,
-        {'S','e','r','v','i','c','e',' ','P','a','c','k',' ','6','a',0},
-        6, 0, 0, VER_NT_WORKSTATION, 0
+        sizeof(RTL_OSVERSIONINFOEXW), 4, 0, 1381, VER_PLATFORM_WIN32_NT,
+        L"Service Pack 6a", 6, 0, 0, VER_NT_WORKSTATION, 0
     },
     /* NT2K */
     {
-        sizeof(RTL_OSVERSIONINFOEXW), 5, 0, 0x893, VER_PLATFORM_WIN32_NT,
-        {'S','e','r','v','i','c','e',' ','P','a','c','k',' ','4',0},
-        4, 0, 0, VER_NT_WORKSTATION, 30 /* FIXME: Great, a reserved field with a value! */
+        sizeof(RTL_OSVERSIONINFOEXW), 5, 0, 2195, VER_PLATFORM_WIN32_NT,
+        L"Service Pack 4", 4, 0, 0, VER_NT_WORKSTATION,
+        30 /* FIXME: Great, a reserved field with a value! */
     },
     /* WINXP */
     {
-        sizeof(RTL_OSVERSIONINFOEXW), 5, 1, 0xA28, VER_PLATFORM_WIN32_NT,
-        {'S','e','r','v','i','c','e',' ','P','a','c','k',' ','3',0},
-        3, 0, VER_SUITE_SINGLEUSERTS, VER_NT_WORKSTATION, 30 /* FIXME: Great, a reserved field with a value! */
+        sizeof(RTL_OSVERSIONINFOEXW), 5, 1, 2600, VER_PLATFORM_WIN32_NT,
+        L"Service Pack 3", 3, 0, VER_SUITE_SINGLEUSERTS, VER_NT_WORKSTATION,
+        30 /* FIXME: Great, a reserved field with a value! */
     },
     /* WINXP64 */
     {
-        sizeof(RTL_OSVERSIONINFOEXW), 5, 2, 0xECE, VER_PLATFORM_WIN32_NT,
-        {'S','e','r','v','i','c','e',' ','P','a','c','k',' ','2',0},
-        2, 0, VER_SUITE_SINGLEUSERTS, VER_NT_WORKSTATION, 0
+        sizeof(RTL_OSVERSIONINFOEXW), 5, 2, 3790, VER_PLATFORM_WIN32_NT,
+        L"Service Pack 2", 2, 0, VER_SUITE_SINGLEUSERTS, VER_NT_WORKSTATION, 0
     },
     /* WIN2K3 */
     {
-        sizeof(RTL_OSVERSIONINFOEXW), 5, 2, 0xECE, VER_PLATFORM_WIN32_NT,
-        {'S','e','r','v','i','c','e',' ','P','a','c','k',' ','2',0},
-        2, 0, VER_SUITE_SINGLEUSERTS, VER_NT_SERVER, 0
+        sizeof(RTL_OSVERSIONINFOEXW), 5, 2, 3790, VER_PLATFORM_WIN32_NT,
+        L"Service Pack 2", 2, 0, VER_SUITE_SINGLEUSERTS, VER_NT_SERVER, 0
     },
     /* WINVISTA */
     {
-        sizeof(RTL_OSVERSIONINFOEXW), 6, 0, 0x1772, VER_PLATFORM_WIN32_NT,
-        {'S','e','r','v','i','c','e',' ','P','a','c','k',' ','2',0},
-        2, 0, VER_SUITE_SINGLEUSERTS, VER_NT_WORKSTATION, 0
+        sizeof(RTL_OSVERSIONINFOEXW), 6, 0, 6002, VER_PLATFORM_WIN32_NT,
+        L"Service Pack 2", 2, 0, VER_SUITE_SINGLEUSERTS, VER_NT_WORKSTATION, 0
     },
     /* WIN2K8 */
     {
-        sizeof(RTL_OSVERSIONINFOEXW), 6, 0, 0x1772, VER_PLATFORM_WIN32_NT,
-        {'S','e','r','v','i','c','e',' ','P','a','c','k',' ','2',0},
-        2, 0, VER_SUITE_SINGLEUSERTS, VER_NT_SERVER, 0
+        sizeof(RTL_OSVERSIONINFOEXW), 6, 0, 6002, VER_PLATFORM_WIN32_NT,
+        L"Service Pack 2", 2, 0, VER_SUITE_SINGLEUSERTS, VER_NT_SERVER, 0
     },
     /* WIN7 */
     {
-        sizeof(RTL_OSVERSIONINFOEXW), 6, 1, 0x1DB1, VER_PLATFORM_WIN32_NT,
-        {'S','e','r','v','i','c','e',' ','P','a','c','k',' ','1',0},
-        1, 0, VER_SUITE_SINGLEUSERTS, VER_NT_WORKSTATION, 0
+        sizeof(RTL_OSVERSIONINFOEXW), 6, 1, 7601, VER_PLATFORM_WIN32_NT,
+        L"Service Pack 1", 1, 0, VER_SUITE_SINGLEUSERTS, VER_NT_WORKSTATION, 0
     },
     /* WIN2K8R2 */
     {
-        sizeof(RTL_OSVERSIONINFOEXW), 6, 1, 0x1DB1, VER_PLATFORM_WIN32_NT,
-        {'S','e','r','v','i','c','e',' ','P','a','c','k',' ','1',0},
-        1, 0, VER_SUITE_SINGLEUSERTS, VER_NT_SERVER, 0
+        sizeof(RTL_OSVERSIONINFOEXW), 6, 1, 7601, VER_PLATFORM_WIN32_NT,
+        L"Service Pack 1", 1, 0, VER_SUITE_SINGLEUSERTS, VER_NT_SERVER, 0
     },
     /* WIN8 */
     {
-        sizeof(RTL_OSVERSIONINFOEXW), 6, 2, 0x23F0, VER_PLATFORM_WIN32_NT,
-        {0}, 0, 0, VER_SUITE_SINGLEUSERTS, VER_NT_WORKSTATION, 0
+        sizeof(RTL_OSVERSIONINFOEXW), 6, 2, 9200, VER_PLATFORM_WIN32_NT,
+        L"", 0, 0, VER_SUITE_SINGLEUSERTS, VER_NT_WORKSTATION, 0
     },
     /* WIN81 */
     {
-        sizeof(RTL_OSVERSIONINFOEXW), 6, 3, 0x2580, VER_PLATFORM_WIN32_NT,
-        {0}, 0, 0, VER_SUITE_SINGLEUSERTS, VER_NT_WORKSTATION, 0
+        sizeof(RTL_OSVERSIONINFOEXW), 6, 3, 9600, VER_PLATFORM_WIN32_NT,
+        L"", 0, 0, VER_SUITE_SINGLEUSERTS, VER_NT_WORKSTATION, 0
     },
     /* WIN10 */
     {
-        sizeof(RTL_OSVERSIONINFOEXW), 10, 0, 0x42EE, VER_PLATFORM_WIN32_NT,
-        {0}, 0, 0, VER_SUITE_SINGLEUSERTS, VER_NT_WORKSTATION, 0
+        sizeof(RTL_OSVERSIONINFOEXW), 10, 0, 19043, VER_PLATFORM_WIN32_NT,
+        L"", 0, 0, VER_SUITE_SINGLEUSERTS, VER_NT_WORKSTATION, 0
+    },
+    /* WIN11 */
+    {
+        sizeof(RTL_OSVERSIONINFOEXW), 10, 0, 22000, VER_PLATFORM_WIN32_NT,
+        L"", 0, 0, VER_SUITE_SINGLEUSERTS, VER_NT_WORKSTATION, 0
     },
-
 };
 
 static const struct { WCHAR name[12]; WINDOWS_VERSION ver; } version_names[] =
 {
-    { {'w','i','n','2','0',0}, WIN20 },
-    { {'w','i','n','3','0',0}, WIN30 },
-    { {'w','i','n','3','1',0}, WIN31 },
-    { {'w','i','n','9','5',0}, WIN95 },
-    { {'w','i','n','9','8',0}, WIN98 },
-    { {'w','i','n','m','e',0}, WINME },
-    { {'n','t','3','5','1',0}, NT351 },
-    { {'n','t','4','0',0}, NT40 },
-    { {'w','i','n','2','0','0','0',0}, NT2K },
-    { {'w','i','n','2','k',0}, NT2K },
-    { {'n','t','2','k',0}, NT2K },
-    { {'n','t','2','0','0','0',0}, NT2K },
-    { {'w','i','n','x','p',0}, WINXP },
-    { {'w','i','n','x','p','6','4',0}, WINXP64 },
-    { {'w','i','n','2','0','0','3',0}, WIN2K3 },
-    { {'w','i','n','2','k','3',0}, WIN2K3 },
-    { {'v','i','s','t','a',0}, WINVISTA },
-    { {'w','i','n','v','i','s','t','a',0}, WINVISTA },
-    { {'w','i','n','2','0','0','8',0}, WIN2K8 },
-    { {'w','i','n','2','k','8',0}, WIN2K8 },
-    { {'w','i','n','2','0','0','8','r','2',0}, WIN2K8R2 },
-    { {'w','i','n','2','k','8','r','2',0}, WIN2K8R2 },
-    { {'w','i','n','7',0}, WIN7 },
-    { {'w','i','n','8',0}, WIN8 },
-    { {'w','i','n','8','1',0}, WIN81 },
-    { {'w','i','n','1','0',0}, WIN10 },
+    { L"win20", WIN20 },
+    { L"win30", WIN30 },
+    { L"win31", WIN31 },
+    { L"win95", WIN95 },
+    { L"win98", WIN98 },
+    { L"winme", WINME },
+    { L"nt351", NT351 },
+    { L"nt40", NT40 },
+    { L"win2000", NT2K },
+    { L"win2k", NT2K },
+    { L"nt2k", NT2K },
+    { L"nt2000", NT2K },
+    { L"winxp", WINXP },
+    { L"winxp64", WINXP64 },
+    { L"win2003", WIN2K3 },
+    { L"win2k3", WIN2K3 },
+    { L"vista", WINVISTA },
+    { L"winvista", WINVISTA },
+    { L"win2008", WIN2K8 },
+    { L"win2k8", WIN2K8 },
+    { L"win2008r2", WIN2K8R2 },
+    { L"win2k8r2", WIN2K8R2 },
+    { L"win7", WIN7 },
+    { L"win8", WIN8 },
+    { L"win81", WIN81 },
+    { L"win10", WIN10 },
+    { L"win11", WIN11 },
 };
 
 
 /* initialized to null so that we crash if we try to retrieve the version too early at startup */
 static const RTL_OSVERSIONINFOEXW *current_version;
 
+static char wine_version[256];
+
+/*********************************************************************
+ *                  wine_get_version
+ */
+const char * CDECL wine_get_version(void)
+{
+    return wine_version;
+}
+
+
+/*********************************************************************
+ *                  wine_get_build_id
+ */
+const char * CDECL wine_get_build_id(void)
+{
+    const char *p = wine_version;
+    p += strlen(p) + 1;  /* skip version */
+    return p;
+}
+
+
+/*********************************************************************
+ *                  wine_get_host_version
+ */
+void CDECL wine_get_host_version( const char **sysname, const char **release )
+{
+    const char *p = wine_version;
+    p += strlen(p) + 1;  /* skip version */
+    p += strlen(p) + 1;  /* skip build id */
+    if (sysname) *sysname = p;
+    p += strlen(p) + 1;
+    if (release) *release = p;
+}
+
 
 /**********************************************************************
  *         get_nt_registry_version
@@ -233,29 +256,6 @@ static const RTL_OSVERSIONINFOEXW *current_version;
  */
 static BOOL get_nt_registry_version( RTL_OSVERSIONINFOEXW *version )
 {
-    static const WCHAR version_keyW[] = {'M','a','c','h','i','n','e','\\',
-                                         'S','o','f','t','w','a','r','e','\\',
-                                         'M','i','c','r','o','s','o','f','t','\\',
-                                         'W','i','n','d','o','w','s',' ','N','T','\\',
-                                         'C','u','r','r','e','n','t','V','e','r','s','i','o','n',0};
-    static const WCHAR service_pack_keyW[] = {'M','a','c','h','i','n','e','\\',
-                                              'S','y','s','t','e','m','\\',
-                                              'C','u','r','r','e','n','t','C','o','n','t','r','o','l','S','e','t','\\',
-                                              'C','o','n','t','r','o','l','\\',
-                                              'W','i','n','d','o','w','s',0};
-    static const WCHAR product_keyW[] = {'M','a','c','h','i','n','e','\\',
-                                         'S','y','s','t','e','m','\\',
-                                         'C','u','r','r','e','n','t','C','o','n','t','r','o','l','S','e','t','\\',
-                                         'C','o','n','t','r','o','l','\\',
-                                         'P','r','o','d','u','c','t','O','p','t','i','o','n','s',0};
-    static const WCHAR CurrentBuildNumberW[] = {'C','u','r','r','e','n','t','B','u','i','l','d','N','u','m','b','e','r',0};
-    static const WCHAR CSDVersionW[] = {'C','S','D','V','e','r','s','i','o','n',0};
-    static const WCHAR CurrentVersionW[] = {'C','u','r','r','e','n','t','V','e','r','s','i','o','n',0};
-    static const WCHAR ProductTypeW[] = {'P','r','o','d','u','c','t','T','y','p','e',0};
-    static const WCHAR WinNTW[]    = {'W','i','n','N','T',0};
-    static const WCHAR ServerNTW[] = {'S','e','r','v','e','r','N','T',0};
-    static const WCHAR LanmanNTW[] = {'L','a','n','m','a','n','N','T',0};
-
     OBJECT_ATTRIBUTES attr;
     UNICODE_STRING nameW, valueW;
     HANDLE hkey, hkey2;
@@ -270,24 +270,42 @@ static BOOL get_nt_registry_version( RTL_OSVERSIONINFOEXW *version )
     attr.Attributes = 0;
     attr.SecurityDescriptor = NULL;
     attr.SecurityQualityOfService = NULL;
-    RtlInitUnicodeString( &nameW, version_keyW );
+    RtlInitUnicodeString( &nameW, L"\\Registry\\Machine\\Software\\Microsoft\\Windows NT\\CurrentVersion" );
 
     if (NtOpenKey( &hkey, KEY_ALL_ACCESS, &attr )) return FALSE;
 
     memset( version, 0, sizeof(*version) );
 
-    RtlInitUnicodeString( &valueW, CurrentVersionW );
-    if (!NtQueryValueKey( hkey, &valueW, KeyValuePartialInformation, tmp, sizeof(tmp)-1, &count ))
+    RtlInitUnicodeString( &valueW, L"CurrentMajorVersionNumber" );
+    if (!NtQueryValueKey( hkey, &valueW, KeyValuePartialInformation, tmp, sizeof(tmp)-1, &count ) &&
+        info->Type == REG_DWORD)
     {
-        WCHAR *p, *str = (WCHAR *)info->Data;
-        str[info->DataLength / sizeof(WCHAR)] = 0;
-        p = strchrW( str, '.' );
-        if (p)
+        version->dwMajorVersion = *(DWORD *)info->Data;
+
+        RtlInitUnicodeString( &valueW, L"CurrentMinorVersionNumber" );
+        if (!NtQueryValueKey( hkey, &valueW, KeyValuePartialInformation, tmp, sizeof(tmp)-1, &count ) &&
+            info->Type == REG_DWORD)
         {
-            *p++ = 0;
-            version->dwMinorVersion = atoiW( p );
+            version->dwMinorVersion = *(DWORD *)info->Data;
+        }
+        else version->dwMajorVersion = 0;
+    }
+
+    if (!version->dwMajorVersion)
+    {
+        RtlInitUnicodeString( &valueW, L"CurrentVersion" );
+        if (!NtQueryValueKey( hkey, &valueW, KeyValuePartialInformation, tmp, sizeof(tmp)-1, &count ))
+        {
+            WCHAR *p, *str = (WCHAR *)info->Data;
+            str[info->DataLength / sizeof(WCHAR)] = 0;
+            p = wcschr( str, '.' );
+            if (p)
+            {
+                *p++ = 0;
+                version->dwMinorVersion = wcstoul( p, NULL, 10 );
+            }
+            version->dwMajorVersion = wcstoul( str, NULL, 10 );
         }
-        version->dwMajorVersion = atoiW( str );
     }
 
     if (version->dwMajorVersion)   /* we got the main version, now fetch the other fields */
@@ -297,17 +315,17 @@ static BOOL get_nt_registry_version( RTL_OSVERSIONINFOEXW *version )
 
         /* get build number */
 
-        RtlInitUnicodeString( &valueW, CurrentBuildNumberW );
+        RtlInitUnicodeString( &valueW, L"CurrentBuildNumber" );
         if (!NtQueryValueKey( hkey, &valueW, KeyValuePartialInformation, tmp, sizeof(tmp)-1, &count ))
         {
             WCHAR *str = (WCHAR *)info->Data;
             str[info->DataLength / sizeof(WCHAR)] = 0;
-            version->dwBuildNumber = atoiW( str );
+            version->dwBuildNumber = wcstoul( str, NULL, 10 );
         }
 
         /* get version description */
 
-        RtlInitUnicodeString( &valueW, CSDVersionW );
+        RtlInitUnicodeString( &valueW, L"CSDVersion" );
         if (!NtQueryValueKey( hkey, &valueW, KeyValuePartialInformation, tmp, sizeof(tmp)-1, &count ))
         {
             DWORD len = min( info->DataLength, sizeof(version->szCSDVersion) - sizeof(WCHAR) );
@@ -317,10 +335,10 @@ static BOOL get_nt_registry_version( RTL_OSVERSIONINFOEXW *version )
 
         /* get service pack version */
 
-        RtlInitUnicodeString( &nameW, service_pack_keyW );
+        RtlInitUnicodeString( &nameW, L"\\Registry\\Machine\\System\\CurrentControlSet\\Control\\Windows" );
         if (!NtOpenKey( &hkey2, KEY_ALL_ACCESS, &attr ))
         {
-            RtlInitUnicodeString( &valueW, CSDVersionW );
+            RtlInitUnicodeString( &valueW, L"CSDVersion" );
             if (!NtQueryValueKey( hkey2, &valueW, KeyValuePartialInformation, tmp, sizeof(tmp), &count ))
             {
                 if (info->DataLength >= sizeof(DWORD))
@@ -335,17 +353,17 @@ static BOOL get_nt_registry_version( RTL_OSVERSIONINFOEXW *version )
 
         /* get product type */
 
-        RtlInitUnicodeString( &nameW, product_keyW );
+        RtlInitUnicodeString( &nameW, L"\\Registry\\Machine\\System\\CurrentControlSet\\Control\\ProductOptions" );
         if (!NtOpenKey( &hkey2, KEY_ALL_ACCESS, &attr ))
         {
-            RtlInitUnicodeString( &valueW, ProductTypeW );
+            RtlInitUnicodeString( &valueW, L"ProductType" );
             if (!NtQueryValueKey( hkey2, &valueW, KeyValuePartialInformation, tmp, sizeof(tmp)-1, &count ))
             {
                 WCHAR *str = (WCHAR *)info->Data;
                 str[info->DataLength / sizeof(WCHAR)] = 0;
-                if (!strcmpiW( str, WinNTW )) version->wProductType = VER_NT_WORKSTATION;
-                else if (!strcmpiW( str, LanmanNTW )) version->wProductType = VER_NT_DOMAIN_CONTROLLER;
-                else if (!strcmpiW( str, ServerNTW )) version->wProductType = VER_NT_SERVER;
+                if (!wcsicmp( str, L"WinNT" )) version->wProductType = VER_NT_WORKSTATION;
+                else if (!wcsicmp( str, L"LanmanNT" )) version->wProductType = VER_NT_DOMAIN_CONTROLLER;
+                else if (!wcsicmp( str, L"ServerNT" )) version->wProductType = VER_NT_SERVER;
             }
             NtClose( hkey2 );
         }
@@ -365,14 +383,6 @@ static BOOL get_nt_registry_version( RTL_OSVERSIONINFOEXW *version )
  */
 static BOOL get_win9x_registry_version( RTL_OSVERSIONINFOEXW *version )
 {
-    static const WCHAR version_keyW[] = {'M','a','c','h','i','n','e','\\',
-                                         'S','o','f','t','w','a','r','e','\\',
-                                         'M','i','c','r','o','s','o','f','t','\\',
-                                         'W','i','n','d','o','w','s','\\',
-                                         'C','u','r','r','e','n','t','V','e','r','s','i','o','n',0};
-    static const WCHAR VersionNumberW[] = {'V','e','r','s','i','o','n','N','u','m','b','e','r',0};
-    static const WCHAR SubVersionNumberW[] = {'S','u','b','V','e','r','s','i','o','n','N','u','m','b','e','r',0};
-
     OBJECT_ATTRIBUTES attr;
     UNICODE_STRING nameW, valueW;
     HANDLE hkey;
@@ -387,30 +397,30 @@ static BOOL get_win9x_registry_version( RTL_OSVERSIONINFOEXW *version )
     attr.Attributes = 0;
     attr.SecurityDescriptor = NULL;
     attr.SecurityQualityOfService = NULL;
-    RtlInitUnicodeString( &nameW, version_keyW );
+    RtlInitUnicodeString( &nameW, L"\\Registry\\Machine\\Software\\Microsoft\\Windows\\CurrentVersion" );
 
     if (NtOpenKey( &hkey, KEY_ALL_ACCESS, &attr )) return FALSE;
 
     memset( version, 0, sizeof(*version) );
 
-    RtlInitUnicodeString( &valueW, VersionNumberW );
+    RtlInitUnicodeString( &valueW, L"VersionNumber" );
     if (!NtQueryValueKey( hkey, &valueW, KeyValuePartialInformation, tmp, sizeof(tmp)-1, &count ))
     {
         WCHAR *p, *str = (WCHAR *)info->Data;
         str[info->DataLength / sizeof(WCHAR)] = 0;
-        p = strchrW( str, '.' );
+        p = wcschr( str, '.' );
         if (p) *p++ = 0;
-        version->dwMajorVersion = atoiW( str );
+        version->dwMajorVersion = wcstoul( str, NULL, 10 );
         if (p)
         {
             str = p;
-            p = strchrW( str, '.' );
+            p = wcschr( str, '.' );
             if (p)
             {
                 *p++ = 0;
-                version->dwBuildNumber = atoiW( p );
+                version->dwBuildNumber = wcstoul( p, NULL, 10 );
             }
-            version->dwMinorVersion = atoiW( str );
+            version->dwMinorVersion = wcstoul( str, NULL, 10 );
         }
         /* build number contains version too on Win9x */
         version->dwBuildNumber |= MAKEWORD( version->dwMinorVersion, version->dwMajorVersion ) << 16;
@@ -421,7 +431,7 @@ static BOOL get_win9x_registry_version( RTL_OSVERSIONINFOEXW *version )
         ret = TRUE;
         version->dwPlatformId = VER_PLATFORM_WIN32_WINDOWS;
 
-        RtlInitUnicodeString( &valueW, SubVersionNumberW );
+        RtlInitUnicodeString( &valueW, L"SubVersionNumber" );
         if (!NtQueryValueKey( hkey, &valueW, KeyValuePartialInformation, tmp, sizeof(tmp)-1, &count ))
         {
             DWORD len = min( info->DataLength, sizeof(version->szCSDVersion) - sizeof(WCHAR) );
@@ -442,14 +452,12 @@ static BOOL get_win9x_registry_version( RTL_OSVERSIONINFOEXW *version )
  */
 static BOOL parse_win_version( HANDLE hkey )
 {
-    static const WCHAR VersionW[] = {'V','e','r','s','i','o','n',0};
-
     UNICODE_STRING valueW;
     WCHAR *name, tmp[64];
     KEY_VALUE_PARTIAL_INFORMATION *info = (KEY_VALUE_PARTIAL_INFORMATION *)tmp;
     DWORD i, count;
 
-    RtlInitUnicodeString( &valueW, VersionW );
+    RtlInitUnicodeString( &valueW, L"Version" );
     if (NtQueryValueKey( hkey, &valueW, KeyValuePartialInformation, tmp, sizeof(tmp) - sizeof(WCHAR), &count ))
         return FALSE;
 
@@ -458,7 +466,7 @@ static BOOL parse_win_version( HANDLE hkey )
 
     for (i = 0; i < ARRAY_SIZE(version_names); i++)
     {
-        if (strcmpW( version_names[i].name, name )) continue;
+        if (wcscmp( version_names[i].name, name )) continue;
         current_version = &VersionData[version_names[i].ver];
         TRACE( "got win version %s\n", debugstr_w(version_names[i].name) );
         return TRUE;
@@ -474,8 +482,6 @@ static BOOL parse_win_version( HANDLE hkey )
  */
 void version_init(void)
 {
-    static const WCHAR configW[] = {'S','o','f','t','w','a','r','e','\\','W','i','n','e',0};
-    static const WCHAR appdefaultsW[] = {'A','p','p','D','e','f','a','u','l','t','s','\\',0};
     OBJECT_ATTRIBUTES attr;
     UNICODE_STRING nameW;
     HANDLE root, hkey, config_key;
@@ -483,7 +489,9 @@ void version_init(void)
     const WCHAR *p, *appname = NtCurrentTeb()->Peb->ProcessParameters->ImagePathName.Buffer;
     WCHAR appversion[MAX_PATH+20];
 
-    current_version = &VersionData[WIN7];
+    NtQuerySystemInformation( SystemWineVersionInformation, wine_version, sizeof(wine_version), NULL );
+
+    current_version = &VersionData[WIN10];
 
     RtlOpenCurrentUser( KEY_ALL_ACCESS, &root );
     attr.Length = sizeof(attr);
@@ -492,7 +500,7 @@ void version_init(void)
     attr.Attributes = 0;
     attr.SecurityDescriptor = NULL;
     attr.SecurityQualityOfService = NULL;
-    RtlInitUnicodeString( &nameW, configW );
+    RtlInitUnicodeString( &nameW, L"Software\\Wine" );
 
     /* @@ Wine registry key: HKCU\Software\Wine */
     if (NtOpenKey( &config_key, KEY_ALL_ACCESS, &attr )) config_key = 0;
@@ -501,11 +509,11 @@ void version_init(void)
 
     /* open AppDefaults\\appname key */
 
-    if ((p = strrchrW( appname, '/' ))) appname = p + 1;
-    if ((p = strrchrW( appname, '\\' ))) appname = p + 1;
+    if ((p = wcsrchr( appname, '/' ))) appname = p + 1;
+    if ((p = wcsrchr( appname, '\\' ))) appname = p + 1;
 
-    strcpyW( appversion, appdefaultsW );
-    strcatW( appversion, appname );
+    wcscpy( appversion, L"AppDefaults\\" );
+    wcscat( appversion, appname );
     RtlInitUnicodeString( &nameW, appversion );
     attr.RootDirectory = config_key;
 
@@ -541,13 +549,7 @@ done:
     NtCurrentTeb()->Peb->OSBuildNumber  = current_version->dwBuildNumber;
     NtCurrentTeb()->Peb->OSPlatformId   = current_version->dwPlatformId;
 
-    user_shared_data->NtProductType      = current_version->wProductType;
-    user_shared_data->ProductTypeIsValid = TRUE;
-    user_shared_data->NtMajorVersion     = current_version->dwMajorVersion;
-    user_shared_data->NtMinorVersion     = current_version->dwMinorVersion;
-    user_shared_data->SuiteMask          = current_version->wSuiteMask;
-
-    TRACE( "got %d.%d platform %d build %x name %s service pack %d.%d product %d\n",
+    TRACE( "got %ld.%ld platform %ld build %lx name %s service pack %d.%d product %d\n",
            current_version->dwMajorVersion, current_version->dwMinorVersion,
            current_version->dwPlatformId, current_version->dwBuildNumber,
            debugstr_w(current_version->szCSDVersion),
@@ -566,7 +568,7 @@ done:
 BOOLEAN WINAPI RtlGetProductInfo(DWORD dwOSMajorVersion, DWORD dwOSMinorVersion, DWORD dwSpMajorVersion,
                                  DWORD dwSpMinorVersion, PDWORD pdwReturnedProductType)
 {
-    TRACE("(%d, %d, %d, %d, %p)\n", dwOSMajorVersion, dwOSMinorVersion,
+    TRACE("(%ld, %ld, %ld, %ld, %p)\n", dwOSMajorVersion, dwOSMinorVersion,
           dwSpMajorVersion, dwSpMinorVersion, pdwReturnedProductType);
 
     if (!pdwReturnedProductType)
@@ -595,7 +597,7 @@ NTSTATUS WINAPI RtlGetVersion( RTL_OSVERSIONINFOEXW *info )
     info->dwMinorVersion = current_version->dwMinorVersion;
     info->dwBuildNumber  = current_version->dwBuildNumber;
     info->dwPlatformId   = current_version->dwPlatformId;
-    strcpyW( info->szCSDVersion, current_version->szCSDVersion );
+    wcscpy( info->szCSDVersion, current_version->szCSDVersion );
     if(info->dwOSVersionInfoSize == sizeof(RTL_OSVERSIONINFOEXW))
     {
         info->wServicePackMajor = current_version->wServicePackMajor;
@@ -703,7 +705,7 @@ NTSTATUS WINAPI RtlVerifyVersionInfo( const RTL_OSVERSIONINFOEXW *info,
     RTL_OSVERSIONINFOEXW ver;
     NTSTATUS status;
 
-    TRACE("(%p,0x%x,0x%s)\n", info, dwTypeMask, wine_dbgstr_longlong(dwlConditionMask));
+    TRACE("(%p,0x%lx,0x%s)\n", info, dwTypeMask, wine_dbgstr_longlong(dwlConditionMask));
 
     ver.dwOSVersionInfoSize = sizeof(ver);
     if ((status = RtlGetVersion( &ver )) != STATUS_SUCCESS) return status;
@@ -781,3 +783,21 @@ NTSTATUS WINAPI RtlVerifyVersionInfo( const RTL_OSVERSIONINFOEXW *info,
 
     return STATUS_SUCCESS;
 }
+
+
+/******************************************************************************
+ *        VerSetConditionMask   (NTDLL.@)
+ */
+ULONGLONG WINAPI VerSetConditionMask( ULONGLONG condition_mask, DWORD type_mask, BYTE condition )
+{
+    condition &= 0x07;
+    if (type_mask & VER_PRODUCT_TYPE) condition_mask |= condition << 7*3;
+    else if (type_mask & VER_SUITENAME) condition_mask |= condition << 6*3;
+    else if (type_mask & VER_SERVICEPACKMAJOR) condition_mask |= condition << 5*3;
+    else if (type_mask & VER_SERVICEPACKMINOR) condition_mask |= condition << 4*3;
+    else if (type_mask & VER_PLATFORMID) condition_mask |= condition << 3*3;
+    else if (type_mask & VER_BUILDNUMBER) condition_mask |= condition << 2*3;
+    else if (type_mask & VER_MAJORVERSION) condition_mask |= condition << 1*3;
+    else if (type_mask & VER_MINORVERSION) condition_mask |= condition << 0*3;
+    return condition_mask;
+}
