@@ -20,8 +20,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
-#define NONAMELESSUNION
-
 #include "ntoskrnl_private.h"
 #include "winreg.h"
 #include "winuser.h"
@@ -89,12 +87,12 @@ static NTSTATUS get_device_id( DEVICE_OBJECT *device, BUS_QUERY_ID_TYPE type, WC
     irpsp->MinorFunction = IRP_MN_QUERY_ID;
     irpsp->Parameters.QueryId.IdType = type;
 
-    irp->IoStatus.u.Status = STATUS_NOT_SUPPORTED;
+    irp->IoStatus.Status = STATUS_NOT_SUPPORTED;
     if (IoCallDriver( device, irp ) == STATUS_PENDING)
         KeWaitForSingleObject( &event, Executive, KernelMode, FALSE, NULL );
 
     *id = (WCHAR *)irp_status.Information;
-    return irp_status.u.Status;
+    return irp_status.Status;
 }
 
 static NTSTATUS send_pnp_irp( DEVICE_OBJECT *device, UCHAR minor )
@@ -116,11 +114,11 @@ static NTSTATUS send_pnp_irp( DEVICE_OBJECT *device, UCHAR minor )
     irpsp->Parameters.StartDevice.AllocatedResources = NULL;
     irpsp->Parameters.StartDevice.AllocatedResourcesTranslated = NULL;
 
-    irp->IoStatus.u.Status = STATUS_NOT_SUPPORTED;
+    irp->IoStatus.Status = STATUS_NOT_SUPPORTED;
     if (IoCallDriver( device, irp ) == STATUS_PENDING)
         KeWaitForSingleObject( &event, Executive, KernelMode, FALSE, NULL );
 
-    return irp_status.u.Status;
+    return irp_status.Status;
 }
 
 static NTSTATUS get_device_instance_id( DEVICE_OBJECT *device, WCHAR *buffer )
@@ -176,11 +174,11 @@ static NTSTATUS get_device_caps( DEVICE_OBJECT *device, DEVICE_CAPABILITIES *cap
     irpsp->MinorFunction = IRP_MN_QUERY_CAPABILITIES;
     irpsp->Parameters.DeviceCapabilities.Capabilities = caps;
 
-    irp->IoStatus.u.Status = STATUS_NOT_SUPPORTED;
+    irp->IoStatus.Status = STATUS_NOT_SUPPORTED;
     if (IoCallDriver( device, irp ) == STATUS_PENDING)
         KeWaitForSingleObject( &event, Executive, KernelMode, FALSE, NULL );
 
-    return irp_status.u.Status;
+    return irp_status.Status;
 }
 
 static void load_function_driver( DEVICE_OBJECT *device, HDEVINFO set, SP_DEVINFO_DATA *sp_device )
@@ -435,14 +433,14 @@ static void handle_bus_relations( DEVICE_OBJECT *parent )
     irpsp->MinorFunction = IRP_MN_QUERY_DEVICE_RELATIONS;
     irpsp->Parameters.QueryDeviceRelations.Type = BusRelations;
 
-    irp->IoStatus.u.Status = STATUS_NOT_SUPPORTED;
+    irp->IoStatus.Status = STATUS_NOT_SUPPORTED;
     if (IoCallDriver( parent, irp ) == STATUS_PENDING)
         KeWaitForSingleObject( &event, Executive, KernelMode, FALSE, NULL );
 
     relations = (DEVICE_RELATIONS *)irp_status.Information;
-    if (irp_status.u.Status)
+    if (irp_status.Status)
     {
-        ERR("Failed to enumerate child devices, status %#lx.\n", irp_status.u.Status);
+        ERR("Failed to enumerate child devices, status %#lx.\n", irp_status.Status);
         SetupDiDestroyDeviceInfoList( set );
         return;
     }
@@ -720,18 +718,18 @@ NTSTATUS WINAPI IoSetDeviceInterfaceState( UNICODE_STRING *name, BOOLEAN enable
         'C','u','r','r','e','n','t','C','o','n','t','r','o','l','S','e','t','\\',
         'C','o','n','t','r','o','l','\\',
         'D','e','v','i','c','e','C','l','a','s','s','e','s','\\',0};
-    static const WCHAR controlW[] = {'C','o','n','t','r','o','l',0};
-    static const WCHAR linkedW[] = {'L','i','n','k','e','d',0};
     static const WCHAR slashW[] = {'\\',0};
     static const WCHAR hashW[] = {'#',0};
 
     size_t namelen = name->Length / sizeof(WCHAR);
     DEV_BROADCAST_DEVICEINTERFACE_W *broadcast;
-    WCHAR *path, *refstr, *p, *upper_end;
     struct device_interface *iface;
     HANDLE iface_key, control_key;
     OBJECT_ATTRIBUTES attr = {0};
     struct wine_rb_entry *entry;
+    UNICODE_STRING control = RTL_CONSTANT_STRING( L"Control" );
+    UNICODE_STRING linked = RTL_CONSTANT_STRING( L"Linked" );
+    WCHAR *path, *refstr, *p;
     UNICODE_STRING string;
     DWORD data = enable;
     NTSTATUS ret;
@@ -780,14 +778,14 @@ NTSTATUS WINAPI IoSetDeviceInterfaceState( UNICODE_STRING *name, BOOLEAN enable
         return ret;
 
     attr.RootDirectory = iface_key;
-    RtlInitUnicodeString( &string, controlW );
+    attr.ObjectName = &control;
     ret = NtCreateKey( &control_key, KEY_SET_VALUE, &attr, 0, NULL, REG_OPTION_VOLATILE, NULL );
     NtClose( iface_key );
     if (ret)
         return ret;
 
-    RtlInitUnicodeString( &string, linkedW );
-    ret = NtSetValueKey( control_key, &string, 0, REG_DWORD, &data, sizeof(data) );
+    attr.ObjectName = &linked;
+    ret = NtSetValueKey( control_key, &linked, 0, REG_DWORD, &data, sizeof(data) );
     if (ret)
     {
         NtClose( control_key );
@@ -800,7 +798,7 @@ NTSTATUS WINAPI IoSetDeviceInterfaceState( UNICODE_STRING *name, BOOLEAN enable
         ret = IoDeleteSymbolicLink( name );
     if (ret)
     {
-        NtDeleteValueKey( control_key, &string );
+        NtDeleteValueKey( control_key, &linked );
         NtClose( control_key );
         return ret;
     }
@@ -817,12 +815,6 @@ NTSTATUS WINAPI IoSetDeviceInterfaceState( UNICODE_STRING *name, BOOLEAN enable
         broadcast->dbcc_classguid  = iface->interface_class;
         lstrcpynW( broadcast->dbcc_name, name->Buffer, namelen + 1 );
         if (namelen > 1) broadcast->dbcc_name[1] = '\\';
-
-        upper_end = wcschr( broadcast->dbcc_name, '#' );
-        if (upper_end) upper_end = wcschr( upper_end + 1, '#' );
-        while (upper_end && upper_end-- != broadcast->dbcc_name)
-            *upper_end = towupper( *upper_end );
-
         send_devicechange( enable ? DBT_DEVICEARRIVAL : DBT_DEVICEREMOVECOMPLETE, broadcast, len );
         heap_free( broadcast );
     }
@@ -883,13 +875,12 @@ NTSTATUS WINAPI IoRegisterDeviceInterface(DEVICE_OBJECT *device, const GUID *cla
     SP_DEVICE_INTERFACE_DATA sp_iface = {sizeof(sp_iface)};
     SP_DEVINFO_DATA sp_device = {sizeof(sp_device)};
     WCHAR device_instance_id[MAX_DEVICE_ID_LEN];
-    SP_DEVICE_INTERFACE_DETAIL_DATA_W *data;
     NTSTATUS status = STATUS_SUCCESS;
     UNICODE_STRING device_path;
     struct device_interface *iface;
     struct wine_rb_entry *entry;
-    DWORD required;
     HDEVINFO set;
+    WCHAR *p;
 
     TRACE("device %p, class_guid %s, refstr %s, symbolic_link %p.\n",
             device, debugstr_guid(class_guid), debugstr_us(refstr), symbolic_link);
@@ -910,22 +901,31 @@ NTSTATUS WINAPI IoRegisterDeviceInterface(DEVICE_OBJECT *device, const GUID *cla
     if (!SetupDiCreateDeviceInterfaceW( set, &sp_device, class_guid, refstr ? refstr->Buffer : NULL, 0, &sp_iface ))
         return STATUS_UNSUCCESSFUL;
 
-    required = 0;
-    SetupDiGetDeviceInterfaceDetailW( set, &sp_iface, NULL, 0, &required, NULL );
-    if (required == 0) return STATUS_UNSUCCESSFUL;
+    /* setupapi mangles the case; construct the interface path manually. */
 
-    data = HeapAlloc( GetProcessHeap(), HEAP_ZERO_MEMORY, required );
-    data->cbSize = sizeof(SP_DEVICE_INTERFACE_DETAIL_DATA_W);
+    device_path.Length = (4 + wcslen( device_instance_id ) + 1 + 38) * sizeof(WCHAR);
+    if (refstr)
+        device_path.Length += sizeof(WCHAR) + refstr->Length;
+    device_path.MaximumLength = device_path.Length + sizeof(WCHAR);
 
-    if (!SetupDiGetDeviceInterfaceDetailW( set, &sp_iface, data, required, NULL, NULL ))
+    device_path.Buffer = RtlAllocateHeap( GetProcessHeap(), 0, device_path.MaximumLength );
+    swprintf( device_path.Buffer, device_path.MaximumLength / sizeof(WCHAR),
+            L"\\??\\%s#{%08lx-%04x-%04x-%02x%02x-%02x%02x%02x%02x%02x%02x}",
+            device_instance_id, class_guid->Data1, class_guid->Data2, class_guid->Data3,
+            class_guid->Data4[0], class_guid->Data4[1], class_guid->Data4[2], class_guid->Data4[3],
+            class_guid->Data4[4], class_guid->Data4[5], class_guid->Data4[6], class_guid->Data4[7] );
+    for (p = device_path.Buffer + 4; *p; p++)
     {
-        HeapFree( GetProcessHeap(), 0, data );
-        return STATUS_UNSUCCESSFUL;
+        if (*p == '\\')
+            *p = '#';
+    }
+    if (refstr)
+    {
+        *p++ = '\\';
+        wcscpy( p, refstr->Buffer );
     }
 
-    data->DevicePath[1] = '?';
-    TRACE("Returning path %s.\n", debugstr_w(data->DevicePath));
-    RtlCreateUnicodeString( &device_path, data->DevicePath);
+    TRACE("Returning path %s.\n", debugstr_us(&device_path));
 
     entry = wine_rb_get( &device_interfaces, &device_path );
     if (entry)
@@ -937,7 +937,7 @@ NTSTATUS WINAPI IoRegisterDeviceInterface(DEVICE_OBJECT *device, const GUID *cla
     else
     {
         iface = heap_alloc_zero( sizeof(struct device_interface) );
-        RtlCreateUnicodeString(&iface->symbolic_link, data->DevicePath);
+        RtlDuplicateUnicodeString( 1, &device_path, &iface->symbolic_link );
         if (wine_rb_put( &device_interfaces, &iface->symbolic_link, &iface->entry ))
             ERR("Failed to insert interface %s into tree.\n", debugstr_us(&iface->symbolic_link));
     }
@@ -945,9 +945,7 @@ NTSTATUS WINAPI IoRegisterDeviceInterface(DEVICE_OBJECT *device, const GUID *cla
     iface->device = device;
     iface->interface_class = *class_guid;
     if (symbolic_link)
-        RtlCreateUnicodeString( symbolic_link, data->DevicePath);
-
-    HeapFree( GetProcessHeap(), 0, data );
+        RtlDuplicateUnicodeString( 1, &device_path, symbolic_link );
 
     RtlFreeUnicodeString( &device_path );
 
@@ -1047,14 +1045,14 @@ static NTSTATUS WINAPI pnp_manager_device_pnp( DEVICE_OBJECT *device, IRP *irp )
     case IRP_MN_START_DEVICE:
     case IRP_MN_SURPRISE_REMOVAL:
         /* Nothing to do. */
-        irp->IoStatus.u.Status = STATUS_SUCCESS;
+        irp->IoStatus.Status = STATUS_SUCCESS;
         break;
     case IRP_MN_REMOVE_DEVICE:
         list_remove( &root_device->entry );
-        irp->IoStatus.u.Status = STATUS_SUCCESS;
+        irp->IoStatus.Status = STATUS_SUCCESS;
         break;
     case IRP_MN_QUERY_CAPABILITIES:
-        irp->IoStatus.u.Status = STATUS_SUCCESS;
+        irp->IoStatus.Status = STATUS_SUCCESS;
         break;
     case IRP_MN_QUERY_ID:
     {
@@ -1072,12 +1070,12 @@ static NTSTATUS WINAPI pnp_manager_device_pnp( DEVICE_OBJECT *device, IRP *irp )
                 memcpy( id, root_device->id, (p - root_device->id) * sizeof(WCHAR) );
                 id[p - root_device->id] = 0;
                 irp->IoStatus.Information = (ULONG_PTR)id;
-                irp->IoStatus.u.Status = STATUS_SUCCESS;
+                irp->IoStatus.Status = STATUS_SUCCESS;
             }
             else
             {
                 irp->IoStatus.Information = 0;
-                irp->IoStatus.u.Status = STATUS_NO_MEMORY;
+                irp->IoStatus.Status = STATUS_NO_MEMORY;
             }
             break;
         case BusQueryInstanceID:
@@ -1086,12 +1084,12 @@ static NTSTATUS WINAPI pnp_manager_device_pnp( DEVICE_OBJECT *device, IRP *irp )
             {
                 wcscpy( id, p + 1 );
                 irp->IoStatus.Information = (ULONG_PTR)id;
-                irp->IoStatus.u.Status = STATUS_SUCCESS;
+                irp->IoStatus.Status = STATUS_SUCCESS;
             }
             else
             {
                 irp->IoStatus.Information = 0;
-                irp->IoStatus.u.Status = STATUS_NO_MEMORY;
+                irp->IoStatus.Status = STATUS_NO_MEMORY;
             }
             break;
         default:
@@ -1103,7 +1101,7 @@ static NTSTATUS WINAPI pnp_manager_device_pnp( DEVICE_OBJECT *device, IRP *irp )
         FIXME("Unhandled PnP request %#x.\n", stack->MinorFunction);
     }
 
-    status = irp->IoStatus.u.Status;
+    status = irp->IoStatus.Status;
     IoCompleteRequest( irp, IO_NO_INCREMENT );
     return status;
 }
@@ -1141,15 +1139,13 @@ static DWORD CALLBACK device_enum_thread_proc(void *arg)
 
 void pnp_manager_start(void)
 {
-    static const WCHAR driver_nameW[] = {'\\','D','r','i','v','e','r','\\','P','n','p','M','a','n','a','g','e','r',0};
     WCHAR endpoint[] = L"\\pipe\\wine_plugplay";
     WCHAR protseq[] = L"ncacn_np";
-    UNICODE_STRING driver_nameU;
+    UNICODE_STRING driver_nameU = RTL_CONSTANT_STRING( L"\\Driver\\PnpManager" );
     RPC_WSTR binding_str;
     NTSTATUS status;
     RPC_STATUS err;
 
-    RtlInitUnicodeString( &driver_nameU, driver_nameW );
     if ((status = IoCreateDriver( &driver_nameU, pnp_manager_driver_entry )))
         ERR("Failed to create PnP manager driver, status %#lx.\n", status);
 
