@@ -38,8 +38,6 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(sapi);
 
-static HINSTANCE hinstance;
-
 struct class_factory
 {
     IClassFactory IClassFactory_iface;
@@ -107,9 +105,13 @@ static const struct IClassFactoryVtbl class_factory_vtbl =
 
 static struct class_factory data_key_cf       = { { &class_factory_vtbl }, data_key_create };
 static struct class_factory file_stream_cf    = { { &class_factory_vtbl }, file_stream_create };
+static struct class_factory mmaudio_out_cf    = { { &class_factory_vtbl }, mmaudio_out_create };
+static struct class_factory resource_mgr_cf   = { { &class_factory_vtbl }, resource_manager_create };
+static struct class_factory speech_stream_cf  = { { &class_factory_vtbl }, speech_stream_create };
 static struct class_factory speech_voice_cf   = { { &class_factory_vtbl }, speech_voice_create };
 static struct class_factory token_category_cf = { { &class_factory_vtbl }, token_category_create };
 static struct class_factory token_enum_cf     = { { &class_factory_vtbl }, token_enum_create };
+static struct class_factory token_cf          = { { &class_factory_vtbl }, token_create };
 
 /******************************************************************
  *             DllGetClassObject
@@ -124,10 +126,18 @@ HRESULT WINAPI DllGetClassObject( REFCLSID clsid, REFIID iid, void **obj )
         cf = &data_key_cf.IClassFactory_iface;
     else if (IsEqualCLSID( clsid, &CLSID_SpFileStream ))
         cf = &file_stream_cf.IClassFactory_iface;
+    else if (IsEqualCLSID( clsid, &CLSID_SpMMAudioOut ))
+        cf = &mmaudio_out_cf.IClassFactory_iface;
     else if (IsEqualCLSID( clsid, &CLSID_SpObjectTokenCategory ))
         cf = &token_category_cf.IClassFactory_iface;
     else if (IsEqualCLSID( clsid, &CLSID_SpObjectTokenEnum ))
         cf = &token_enum_cf.IClassFactory_iface;
+    else if (IsEqualCLSID( clsid, &CLSID_SpObjectToken ))
+        cf = &token_cf.IClassFactory_iface;
+    else if (IsEqualCLSID( clsid, &CLSID_SpResourceManager ))
+        cf = &resource_mgr_cf.IClassFactory_iface;
+    else if (IsEqualCLSID( clsid, &CLSID_SpStream ))
+        cf = &speech_stream_cf.IClassFactory_iface;
     else if (IsEqualCLSID( clsid, &CLSID_SpVoice ))
         cf = &speech_voice_cf.IClassFactory_iface;
 
@@ -135,40 +145,3 @@ HRESULT WINAPI DllGetClassObject( REFCLSID clsid, REFIID iid, void **obj )
 
     return IClassFactory_QueryInterface( cf, iid, obj );
 }
-
-BOOL WINAPI DllMain( HINSTANCE dll, DWORD reason, LPVOID reserved )
-{
-    switch (reason)
-    {
-    case DLL_PROCESS_ATTACH:
-        hinstance = dll;
-        DisableThreadLibraryCalls( dll );
-        break;
-    }
-    return TRUE;
-}
-
-/******************************************************************
- *              DllCanUnloadNow
- */
-HRESULT WINAPI DllCanUnloadNow( void )
-{
-    TRACE( "()\n" );
-    return S_FALSE;
-}
-
-/***********************************************************************
- *          DllRegisterServer
- */
-HRESULT WINAPI DllRegisterServer( void )
-{
-    return __wine_register_resources( hinstance );
-}
-
-/***********************************************************************
- *          DllUnregisterServer
- */
-HRESULT WINAPI DllUnregisterServer( void )
-{
-    return __wine_unregister_resources( hinstance );
-}
