@@ -18,9 +18,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
-#include "config.h"
-#include "wine/port.h"
-
 #include <string.h>
 #include <stdarg.h>
 #include <stdio.h>
@@ -37,7 +34,27 @@
 #include "shlobj.h"
 #include "shell32_main.h"
 #include "shresdef.h"
-#include "undocshell.h"
+
+/* RunFileDlg flags */
+#define RFF_NOBROWSE        0x01
+#define RFF_NODEFAULT       0x02
+#define RFF_CALCDIRECTORY   0x04
+#define RFF_NOLABEL         0x08
+#define RFF_NOSEPARATEMEM   0x20  /* NT only */
+
+/* RunFileFlg notification structure */
+typedef struct
+{
+    NMHDR hdr;
+    const char *lpFile;
+    const char *lpDirectory;
+    int nShow;
+} NM_RUNFILEDLG;
+
+/* RunFileDlg notification return values */
+#define RF_OK      0x00
+#define RF_CANCEL  0x01
+#define RF_RETRY   0x02
 
 typedef struct
     {
@@ -88,7 +105,6 @@ static void RunFileDlgW(
 	LPCWSTR lpstrDescription,
 	UINT uFlags)
 {
-    static const WCHAR resnameW[] = {'S','H','E','L','L','_','R','U','N','_','D','L','G',0};
     RUNFILEDLGPARAMS rfdp;
     HRSRC hRes;
     LPVOID template;
@@ -101,7 +117,7 @@ static void RunFileDlgW(
     rfdp.lpstrDescription = lpstrDescription;
     rfdp.uFlags           = uFlags;
 
-    if (!(hRes = FindResourceW(shell32_hInstance, resnameW, (LPWSTR)RT_DIALOG)) ||
+    if (!(hRes = FindResourceW(shell32_hInstance, L"SHELL_RUN_DLG", (LPWSTR)RT_DIALOG)) ||
         !(template = LoadResource(shell32_hInstance, hRes)))
     {
         ERR("Couldn't load SHELL_RUN_DLG resource\n");
@@ -119,9 +135,8 @@ static LPWSTR RunDlg_GetParentDir(LPCWSTR cmdline)
 {
     const WCHAR *src;
     WCHAR *dest, *result, *result_end=NULL;
-    static const WCHAR dotexeW[] = {'.','e','x','e',0};
 
-    result = HeapAlloc(GetProcessHeap(), 0, sizeof(WCHAR)*(strlenW(cmdline)+5));
+    result = malloc(sizeof(WCHAR) * (wcslen(cmdline) + 5));
 
     src = cmdline;
     dest = result;
@@ -139,12 +154,12 @@ static LPWSTR RunDlg_GetParentDir(LPCWSTR cmdline)
     else {
         while (*src)
         {
-            if (isspaceW(*src))
+            if (iswspace(*src))
             {
                 *dest = 0;
                 if (INVALID_FILE_ATTRIBUTES != GetFileAttributesW(result))
                     break;
-                strcatW(dest, dotexeW);
+                lstrcatW(dest, L".exe");
                 if (INVALID_FILE_ATTRIBUTES != GetFileAttributesW(result))
                     break;
             }
@@ -161,7 +176,7 @@ static LPWSTR RunDlg_GetParentDir(LPCWSTR cmdline)
     }
     else
     {
-        HeapFree(GetProcessHeap(), 0, result);
+        free(result);
         return NULL;
     }
 }
@@ -216,7 +231,7 @@ static INT_PTR CALLBACK RunDlgProc (HWND hwnd, UINT message, WPARAM wParam, LPAR
 
                         ZeroMemory (&sei, sizeof(sei)) ;
                         sei.cbSize = sizeof(sei) ;
-                        psz = HeapAlloc( GetProcessHeap(), 0, (ic + 1)*sizeof(WCHAR) );
+                        psz = malloc( (ic + 1) * sizeof(WCHAR) );
                         GetWindowTextW (htxt, psz, ic + 1) ;
 
                         /* according to http://www.codeproject.com/KB/shell/runfiledlg.aspx we should send a
@@ -233,8 +248,8 @@ static INT_PTR CALLBACK RunDlgProc (HWND hwnd, UINT message, WPARAM wParam, LPAR
 
                         if (!ShellExecuteExW( &sei ))
                         {
-                            HeapFree(GetProcessHeap(), 0, psz);
-                            HeapFree(GetProcessHeap(), 0, parent);
+                            free(psz);
+                            free(parent);
                             SendMessageA (htxt, CB_SETEDITSEL, 0, MAKELPARAM (0, -1)) ;
                             return TRUE ;
                         }
@@ -243,8 +258,8 @@ static INT_PTR CALLBACK RunDlgProc (HWND hwnd, UINT message, WPARAM wParam, LPAR
                         GetWindowTextA (htxt, (LPSTR)psz, ic + 1) ;
                         FillList (htxt, (LPSTR)psz, FALSE) ;
 
-                        HeapFree(GetProcessHeap(), 0, psz);
-                        HeapFree(GetProcessHeap(), 0, parent);
+                        free(psz);
+                        free(parent);
                         EndDialog (hwnd, 0);
                         }
                     }
@@ -256,10 +271,6 @@ static INT_PTR CALLBACK RunDlgProc (HWND hwnd, UINT message, WPARAM wParam, LPAR
 
                 case IDC_RUNDLG_BROWSE :
                     {
-                    static const WCHAR filterW[] = {'%','s','%','c','*','.','e','x','e','%','c','%','s','%','c','*','.','*','%','c',0};
-                    HMODULE hComdlg = NULL ;
-                    LPFNOFN ofnProc = NULL ;
-                    static const WCHAR comdlg32W[] = {'c','o','m','d','l','g','3','2',0};
                     WCHAR szFName[1024] = {0};
                     WCHAR filter_exe[256], filter_all[256], filter[MAX_PATH], szCaption[MAX_PATH];
                     OPENFILENAMEW ofn;
@@ -267,7 +278,7 @@ static INT_PTR CALLBACK RunDlgProc (HWND hwnd, UINT message, WPARAM wParam, LPAR
                     LoadStringW(shell32_hInstance, IDS_RUNDLG_BROWSE_FILTER_EXE, filter_exe, 256);
                     LoadStringW(shell32_hInstance, IDS_RUNDLG_BROWSE_FILTER_ALL, filter_all, 256);
                     LoadStringW(shell32_hInstance, IDS_RUNDLG_BROWSE_CAPTION, szCaption, MAX_PATH);
-                    snprintfW( filter, MAX_PATH, filterW, filter_exe, 0, 0, filter_all, 0, 0 );
+                    swprintf( filter, MAX_PATH, L"%s%c*.exe%c%s%c*.*%c", filter_exe, 0, 0, filter_all, 0, 0 );
 
                     ZeroMemory(&ofn, sizeof(ofn));
                     ofn.lStructSize = sizeof(OPENFILENAMEW);
@@ -279,15 +290,7 @@ static INT_PTR CALLBACK RunDlgProc (HWND hwnd, UINT message, WPARAM wParam, LPAR
                     ofn.Flags = OFN_ENABLESIZING | OFN_FILEMUSTEXIST | OFN_HIDEREADONLY | OFN_PATHMUSTEXIST;
                     ofn.lpstrInitialDir = prfdp->lpstrDirectory;
 
-                    if (NULL == (hComdlg = LoadLibraryExW (comdlg32W, NULL, 0)) ||
-                        NULL == (ofnProc = (LPFNOFN)GetProcAddress (hComdlg, "GetOpenFileNameW")))
-                    {
-                        ERR("Couldn't get GetOpenFileName function entry (lib=%p, proc=%p)\n", hComdlg, ofnProc);
-                        ShellMessageBoxW(shell32_hInstance, hwnd, MAKEINTRESOURCEW(IDS_RUNDLG_BROWSE_ERROR), NULL, MB_OK | MB_ICONERROR);
-                        return TRUE ;
-                    }
-
-                    if (ofnProc(&ofn))
+                    if (GetOpenFileNameW(&ofn))
                     {
                         SetFocus (GetDlgItem (hwnd, IDOK)) ;
                         SetWindowTextW (GetDlgItem (hwnd, IDC_RUNDLG_EDITPATH), szFName) ;
@@ -295,8 +298,6 @@ static INT_PTR CALLBACK RunDlgProc (HWND hwnd, UINT message, WPARAM wParam, LPAR
                         SetFocus (GetDlgItem (hwnd, IDOK)) ;
                     }
 
-                    FreeLibrary (hComdlg) ;
-
                     return TRUE ;
                     }
                 }
@@ -326,14 +327,14 @@ static void FillList (HWND hCb, char *pszLatest, BOOL fShowDefault)
 
     if (icList > 0)
         {
-        pszList = HeapAlloc( GetProcessHeap(), 0, icList) ;
+        pszList = malloc(icList) ;
         if (ERROR_SUCCESS != RegQueryValueExA (hkey, "MRUList", NULL, NULL, (LPBYTE)pszList, &icList))
             MessageBoxA (hCb, "Unable to grab MRUList !", "Nix", MB_OK) ;
         }
     else
         {
         icList = 1 ;
-        pszList = HeapAlloc( GetProcessHeap(), 0, icList) ;
+        pszList = malloc(icList) ;
         pszList[0] = 0 ;
         }
 
@@ -346,10 +347,7 @@ static void FillList (HWND hCb, char *pszLatest, BOOL fShowDefault)
 
         if (ERROR_SUCCESS != RegQueryValueExA (hkey, szIndex, NULL, NULL, NULL, &icCmd))
             MessageBoxA (hCb, "Unable to grab size of index", "Nix", MB_OK) ;
-        if( pszCmd )
-            pszCmd = HeapReAlloc(GetProcessHeap(), 0, pszCmd, icCmd) ;
-        else
-            pszCmd = HeapAlloc(GetProcessHeap(), 0, icCmd) ;
+        pszCmd = realloc(pszCmd, icCmd) ;
         if (ERROR_SUCCESS != RegQueryValueExA (hkey, szIndex, NULL, NULL, (LPBYTE)pszCmd, &icCmd))
             MessageBoxA (hCb, "Unable to grab index", "Nix", MB_OK) ;
 
@@ -415,10 +413,7 @@ static void FillList (HWND hCb, char *pszLatest, BOOL fShowDefault)
         SendMessageA (hCb, CB_SETEDITSEL, 0, MAKELPARAM (0, -1)) ;
 
         cMatch = ++cMax ;
-        if( pszList )
-            pszList = HeapReAlloc(GetProcessHeap(), 0, pszList, ++icList) ;
-        else
-            pszList = HeapAlloc(GetProcessHeap(), 0, ++icList) ;
+        pszList = realloc(pszList, ++icList) ;
         memmove (&pszList[1], pszList, icList - 1) ;
         pszList[0] = cMatch ;
         szIndex[0] = cMatch ;
@@ -427,9 +422,9 @@ static void FillList (HWND hCb, char *pszLatest, BOOL fShowDefault)
 
     RegSetValueExA (hkey, "MRUList", 0, REG_SZ, (LPBYTE)pszList, strlen (pszList) + 1) ;
 
-    HeapFree( GetProcessHeap(), 0, pszCmd) ;
-    HeapFree( GetProcessHeap(), 0, pszList) ;
-    }
+    free(pszCmd) ;
+    free(pszList) ;
+}
 
 /*************************************************************************
  * RunFileDlgA					[internal]
@@ -499,8 +494,8 @@ static BOOL ConfirmDialog(HWND hWndOwner, UINT PromptId, UINT TitleId)
   WCHAR Prompt[256];
   WCHAR Title[256];
 
-  LoadStringW(shell32_hInstance, PromptId, Prompt, sizeof(Prompt) / sizeof(WCHAR));
-  LoadStringW(shell32_hInstance, TitleId, Title, sizeof(Title) / sizeof(WCHAR));
+  LoadStringW(shell32_hInstance, PromptId, Prompt, ARRAY_SIZE(Prompt));
+  LoadStringW(shell32_hInstance, TitleId, Title, ARRAY_SIZE(Title));
   return MessageBoxW(hWndOwner, Prompt, Title, MB_YESNO|MB_ICONQUESTION) == IDYES;
 }
 
