@@ -300,8 +300,7 @@ static INT SIC_IconAppend (const WCHAR *sourcefile, INT src_index, HICON *hicons
     entry = SHAlloc(sizeof(*entry));
 
     GetFullPathNameW(sourcefile, MAX_PATH, path, NULL);
-    entry->sSourceFile = heap_alloc( (lstrlenW(path)+1)*sizeof(WCHAR) );
-    lstrcpyW( entry->sSourceFile, path );
+    entry->sSourceFile = wcsdup(path);
 
     entry->dwSourceIndex = src_index;
     entry->dwFlags = flags;
@@ -311,7 +310,7 @@ static INT SIC_IconAppend (const WCHAR *sourcefile, INT src_index, HICON *hicons
     index = DPA_InsertPtr(sic_hdpa, 0x7fff, entry);
     if ( INVALID_INDEX == index )
     {
-        heap_free(entry->sSourceFile);
+        free(entry->sSourceFile);
         SHFree(entry);
         ret = INVALID_INDEX;
     }
@@ -492,7 +491,7 @@ static BOOL WINAPI SIC_Initialize( INIT_ONCE *once, void *param, void **context
  */
 static INT CALLBACK sic_free( LPVOID ptr, LPVOID lparam )
 {
-	heap_free(((LPSIC_ENTRY)ptr)->sSourceFile);
+	free(((SIC_ENTRY *)ptr)->sSourceFile);
 	SHFree(ptr);
 	return TRUE;
 }
@@ -745,12 +744,12 @@ INT WINAPI Shell_GetCachedImageIndexA(LPCSTR szPath, INT nIndex, BOOL bSimulateD
 	WARN("(%s,%08x,%08x) semi-stub.\n",debugstr_a(szPath), nIndex, bSimulateDoc);
 
 	len = MultiByteToWideChar( CP_ACP, 0, szPath, -1, NULL, 0 );
-	szTemp = heap_alloc( len * sizeof(WCHAR) );
+	szTemp = malloc( len * sizeof(WCHAR) );
 	MultiByteToWideChar( CP_ACP, 0, szPath, -1, szTemp, len );
 
 	ret = SIC_GetIconIndex( szTemp, nIndex, 0 );
 
-	heap_free( szTemp );
+	free( szTemp );
 
 	return ret;
 }
@@ -789,7 +788,7 @@ UINT WINAPI ExtractIconExA(LPCSTR lpszFile, INT nIconIndex, HICON * phiconLarge,
 {
     UINT ret = 0;
     INT len = MultiByteToWideChar(CP_ACP, 0, lpszFile, -1, NULL, 0);
-    LPWSTR lpwstrFile = heap_alloc( len * sizeof(WCHAR));
+    WCHAR *lpwstrFile = malloc( len * sizeof(WCHAR));
 
     TRACE("%s %i %p %p %i\n", lpszFile, nIconIndex, phiconLarge, phiconSmall, nIcons);
 
@@ -797,7 +796,7 @@ UINT WINAPI ExtractIconExA(LPCSTR lpszFile, INT nIconIndex, HICON * phiconLarge,
     {
         MultiByteToWideChar(CP_ACP, 0, lpszFile, -1, lpwstrFile, len);
         ret = ExtractIconExW(lpwstrFile, nIconIndex, phiconLarge, phiconSmall, nIcons);
-        heap_free(lpwstrFile);
+        free(lpwstrFile);
     }
     return ret;
 }
@@ -817,7 +816,7 @@ HICON WINAPI ExtractAssociatedIconA(HINSTANCE hInst, LPSTR lpIconPath, LPWORD lp
      * lpIconPath itself is supposed to be large enough, so make sure lpIconPathW
      * is large enough too. Yes, I am puking too.
      */
-    LPWSTR lpIconPathW = heap_alloc(MAX_PATH * sizeof(WCHAR));
+    WCHAR *lpIconPathW = malloc(MAX_PATH * sizeof(WCHAR));
 
     TRACE("%p %s %p\n", hInst, debugstr_a(lpIconPath), lpiIcon);
 
@@ -826,7 +825,7 @@ HICON WINAPI ExtractAssociatedIconA(HINSTANCE hInst, LPSTR lpIconPath, LPWORD lp
         MultiByteToWideChar(CP_ACP, 0, lpIconPath, -1, lpIconPathW, len);
         hIcon = ExtractAssociatedIconW(hInst, lpIconPathW, lpiIcon);
         WideCharToMultiByte(CP_ACP, 0, lpIconPathW, -1, lpIconPath, MAX_PATH , NULL, NULL);
-        heap_free(lpIconPathW);
+        free(lpIconPathW);
     }
     return hIcon;
 }
@@ -895,13 +894,13 @@ HICON WINAPI ExtractAssociatedIconExA(HINSTANCE hInst, LPSTR lpIconPath, LPWORD
 {
   HICON ret;
   INT len = MultiByteToWideChar( CP_ACP, 0, lpIconPath, -1, NULL, 0 );
-  LPWSTR lpwstrFile = heap_alloc( len * sizeof(WCHAR) );
+  WCHAR *lpwstrFile = malloc( len * sizeof(WCHAR) );
 
   TRACE("%p %s %p %p)\n", hInst, lpIconPath, lpiIconIdx, lpiIconId);
 
   MultiByteToWideChar( CP_ACP, 0, lpIconPath, -1, lpwstrFile, len );
   ret = ExtractAssociatedIconExW(hInst, lpwstrFile, lpiIconIdx, lpiIconId);
-  heap_free(lpwstrFile);
+  free(lpwstrFile);
   return ret;
 }
 
@@ -942,13 +941,13 @@ HRESULT WINAPI SHDefExtractIconA(LPCSTR pszIconFile, int iIndex, UINT uFlags,
 {
   HRESULT ret;
   INT len = MultiByteToWideChar(CP_ACP, 0, pszIconFile, -1, NULL, 0);
-  LPWSTR lpwstrFile = heap_alloc(len * sizeof(WCHAR));
+  WCHAR *lpwstrFile = malloc(len * sizeof(WCHAR));
 
   TRACE("%s %d 0x%08x %p %p %d\n", pszIconFile, iIndex, uFlags, phiconLarge, phiconSmall, nIconSize);
 
   MultiByteToWideChar(CP_ACP, 0, pszIconFile, -1, lpwstrFile, len);
   ret = SHDefExtractIconW(lpwstrFile, iIndex, uFlags, phiconLarge, phiconSmall, nIconSize);
-  heap_free(lpwstrFile);
+  free(lpwstrFile);
   return ret;
 }
 
