@@ -19,8 +19,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
-#include "config.h"
-
 #include <stdlib.h>
 #include <string.h>
 #include <stdarg.h>
@@ -42,7 +40,6 @@
 #include "propsys.h"
 #include "commoncontrols.h"
 
-#include "undocshell.h"
 #include "pidl.h"
 #include "shell32_main.h"
 #include "version.h"
@@ -51,7 +48,6 @@
 #include "shfldr.h"
 
 #include "wine/debug.h"
-#include "wine/unicode.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(shell);
 
@@ -169,7 +165,7 @@ DWORD_PTR WINAPI SHGetFileInfoW(LPCWSTR path,DWORD dwFileAttributes,
     HRESULT hr = S_OK;
     UINT uGilFlags = 0;
 
-    TRACE("%s fattr=0x%x sfi=%p(attr=0x%08x) size=0x%x flags=0x%x\n",
+    TRACE("%s fattr=0x%lx sfi=%p(attr=0x%08lx) size=0x%x flags=0x%x\n",
           (flags & SHGFI_PIDL)? "pidl" : debugstr_w(path), dwFileAttributes,
           psfi, psfi ? psfi->dwAttributes : 0, sizeofpsfi, flags);
 
@@ -276,10 +272,6 @@ DWORD_PTR WINAPI SHGetFileInfoW(LPCWSTR path,DWORD dwFileAttributes,
     /* get the type name */
     if (SUCCEEDED(hr) && (flags & SHGFI_TYPENAME))
     {
-        static const WCHAR szFolder[] = { 'F','o','l','d','e','r',0 };
-        static const WCHAR szFile[] = { 'F','i','l','e',0 };
-        static const WCHAR szSpaceFile[] = { ' ','f','i','l','e',0 };
-
         if (!(flags & SHGFI_USEFILEATTRIBUTES) || (flags & SHGFI_PIDL))
         {
             char ftype[80];
@@ -290,7 +282,7 @@ DWORD_PTR WINAPI SHGetFileInfoW(LPCWSTR path,DWORD dwFileAttributes,
         else
         {
             if (dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
-                strcatW (psfi->szTypeName, szFolder);
+                lstrcatW (psfi->szTypeName, L"Folder");
             else 
             {
                 WCHAR sTemp[64];
@@ -299,7 +291,7 @@ DWORD_PTR WINAPI SHGetFileInfoW(LPCWSTR path,DWORD dwFileAttributes,
                 if (sTemp[0] == 0 || (sTemp[0] == '.' && sTemp[1] == 0))
                 {
                     /* "name" or "name." => "File" */
-                    lstrcpynW (psfi->szTypeName, szFile, 64);
+                    lstrcpynW (psfi->szTypeName, L"File", 64);
                 }
                 else if (!( HCR_MapTypeToValueW(sTemp, sTemp, 64, TRUE) &&
                     HCR_MapTypeToValueW(sTemp, psfi->szTypeName, 80, FALSE )))
@@ -307,11 +299,11 @@ DWORD_PTR WINAPI SHGetFileInfoW(LPCWSTR path,DWORD dwFileAttributes,
                     if (sTemp[0])
                     {
                         lstrcpynW (psfi->szTypeName, sTemp, 64);
-                        strcatW (psfi->szTypeName, szSpaceFile);
+                        lstrcatW (psfi->szTypeName, L" file");
                     }
                     else
                     {
-                        lstrcpynW (psfi->szTypeName, szFile, 64);
+                        lstrcpynW (psfi->szTypeName, L"File", 64);
                     }
                 }
             }
@@ -352,7 +344,6 @@ DWORD_PTR WINAPI SHGetFileInfoW(LPCWSTR path,DWORD dwFileAttributes,
             else
             {
                 WCHAR* szExt;
-                static const WCHAR p1W[] = {'%','1',0};
                 WCHAR sTemp [MAX_PATH];
 
                 szExt = PathFindExtensionW(szFullPath);
@@ -361,12 +352,12 @@ DWORD_PTR WINAPI SHGetFileInfoW(LPCWSTR path,DWORD dwFileAttributes,
                      HCR_MapTypeToValueW(szExt, sTemp, MAX_PATH, TRUE) &&
                      HCR_GetDefaultIconW(sTemp, sTemp, MAX_PATH, &psfi->iIcon))
                 {
-                    if (lstrcmpW(p1W, sTemp))
-                        strcpyW(psfi->szDisplayName, sTemp);
+                    if (wcscmp(L"%1", sTemp))
+                        lstrcpyW(psfi->szDisplayName, sTemp);
                     else
                     {
                         /* the icon is in the file */
-                        strcpyW(psfi->szDisplayName, szFullPath);
+                        lstrcpyW(psfi->szDisplayName, szFullPath);
                     }
                 }
                 else
@@ -407,7 +398,6 @@ DWORD_PTR WINAPI SHGetFileInfoW(LPCWSTR path,DWORD dwFileAttributes,
                 psfi->iIcon = SIC_GetIconIndex(swShell32Name, -IDI_SHELL_FOLDER, 0);
             else
             {
-                static const WCHAR p1W[] = {'%','1',0};
                 WCHAR sTemp[MAX_PATH];
                 WCHAR *szExt;
                 int icon_idx = 0;
@@ -420,8 +410,8 @@ DWORD_PTR WINAPI SHGetFileInfoW(LPCWSTR path,DWORD dwFileAttributes,
                      HCR_MapTypeToValueW(szExt, sTemp, MAX_PATH, TRUE) &&
                      HCR_GetDefaultIconW(sTemp, sTemp, MAX_PATH, &icon_idx))
                 {
-                    if (!lstrcmpW(p1W,sTemp))            /* icon is in the file */
-                        strcpyW(sTemp, szFullPath);
+                    if (!wcscmp(L"%1",sTemp))            /* icon is in the file */
+                        lstrcpyW(sTemp, szFullPath);
 
                     psfi->iIcon = SIC_GetIconIndex(sTemp, icon_idx, 0);
                     if (psfi->iIcon == -1)
@@ -463,8 +453,8 @@ DWORD_PTR WINAPI SHGetFileInfoW(LPCWSTR path,DWORD dwFileAttributes,
 
                     while ((hr = SIC_get_location( psfi->iIcon, file, &size, &icon_idx )) == E_NOT_SUFFICIENT_BUFFER)
                     {
-                        if (file == buf) file = heap_alloc( size );
-                        else file = heap_realloc( file, size );
+                        if (file == buf) file = malloc( size );
+                        else file = realloc( file, size );
                         if (!file) break;
                     }
                     if (SUCCEEDED(hr))
@@ -472,7 +462,7 @@ DWORD_PTR WINAPI SHGetFileInfoW(LPCWSTR path,DWORD dwFileAttributes,
                         ret = PrivateExtractIconsW( file, icon_idx, width, height, &psfi->hIcon, 0, 1, 0);
                         if (ret == 0 || ret == (UINT)-1) hr = E_FAIL;
                     }
-                    if (file != buf) heap_free( file );
+                    if (file != buf) free( file );
                 }
             }
         }
@@ -490,7 +480,7 @@ DWORD_PTR WINAPI SHGetFileInfoW(LPCWSTR path,DWORD dwFileAttributes,
 
     SHFree(pidlLast);
 
-    TRACE ("icon=%p index=0x%08x attr=0x%08x name=%s type=%s ret=0x%08lx\n",
+    TRACE ("icon=%p index=0x%08x attr=0x%08lx name=%s type=%s ret=0x%08Ix\n",
            psfi->hIcon, psfi->iIcon, psfi->dwAttributes,
            debugstr_w(psfi->szDisplayName), debugstr_w(psfi->szTypeName), ret);
 
@@ -523,7 +513,7 @@ DWORD_PTR WINAPI SHGetFileInfoA(LPCSTR path,DWORD dwFileAttributes,
     else
     {
         len = MultiByteToWideChar(CP_ACP, 0, path, -1, NULL, 0);
-        temppath = heap_alloc(len*sizeof(WCHAR));
+        temppath = malloc(len * sizeof(WCHAR));
         MultiByteToWideChar(CP_ACP, 0, path, -1, temppath, len);
         pathW = temppath;
     }
@@ -556,7 +546,7 @@ DWORD_PTR WINAPI SHGetFileInfoA(LPCSTR path,DWORD dwFileAttributes,
         }
     }
 
-    heap_free(temppath);
+    free(temppath);
 
     return ret;
 }
@@ -595,7 +585,7 @@ HICON WINAPI ExtractIconA(HINSTANCE hInstance, const char *file, UINT nIconIndex
 
     fileW = strdupAtoW(file);
     ret = ExtractIconW(hInstance, fileW, nIconIndex);
-    heap_free(fileW);
+    free(fileW);
 
     return ret;
 }
@@ -616,22 +606,19 @@ HICON WINAPI ExtractIconW(HINSTANCE hInstance, LPCWSTR lpszFile, UINT nIconIndex
         ret = PrivateExtractIconsW(lpszFile, 0, cx, cy, NULL, NULL, 0, LR_DEFAULTCOLOR);
         if (ret != (UINT)-1 && ret)
             return (HICON)(UINT_PTR)ret;
-        return NULL;
     }
     else
+    {
         ret = PrivateExtractIconsW(lpszFile, nIconIndex, cx, cy, &hIcon, NULL, 1, LR_DEFAULTCOLOR);
-
-    if (ret == (UINT)-1)
-        return (HICON)1;
-    else if (ret > 0 && hIcon)
-        return hIcon;
-
+        if (ret > 0 && hIcon)
+            return hIcon;
+    }
     return NULL;
 }
 
 HRESULT WINAPI SHCreateFileExtractIconW(LPCWSTR file, DWORD attribs, REFIID riid, void **ppv)
 {
-  FIXME("%s, %x, %s, %p\n", debugstr_w(file), attribs, debugstr_guid(riid), ppv);
+  FIXME("%s, %lx, %s, %p\n", debugstr_w(file), attribs, debugstr_guid(riid), ppv);
   *ppv = NULL;
   return E_NOTIMPL;
 }
@@ -676,7 +663,7 @@ VOID WINAPI Printer_LoadIconsW(LPCWSTR wsPrinterName, HICON * pLargeIcon, HICON
 BOOL WINAPI Printers_RegisterWindowW(LPCWSTR wsPrinter, DWORD dwType,
             HANDLE * phClassPidl, HWND * phwnd)
 {
-    FIXME("(%s, %x, %p (%p), %p (%p)) stub!\n", debugstr_w(wsPrinter), dwType,
+    FIXME("(%s, %lx, %p (%p), %p (%p)) stub!\n", debugstr_w(wsPrinter), dwType,
                 phClassPidl, (phClassPidl != NULL) ? *(phClassPidl) : NULL,
                 phwnd, (phwnd != NULL) ? *(phwnd) : NULL);
 
@@ -691,13 +678,123 @@ VOID WINAPI Printers_UnregisterWindow(HANDLE hClassPidl, HWND hwnd)
     FIXME("(%p, %p) stub!\n", hClassPidl, hwnd);
 } 
 
+struct window_prop_store
+{
+    IPropertyStore IPropertyStore_iface;
+    LONG           ref;
+};
+
+static inline struct window_prop_store *impl_from_IPropertyStore(IPropertyStore *iface)
+{
+    return CONTAINING_RECORD(iface, struct window_prop_store, IPropertyStore_iface);
+}
+
+static ULONG WINAPI window_prop_store_AddRef(IPropertyStore *iface)
+{
+    struct window_prop_store *store = impl_from_IPropertyStore(iface);
+    LONG ref = InterlockedIncrement(&store->ref);
+    TRACE("returning %ld\n", ref);
+    return ref;
+}
+
+static ULONG WINAPI window_prop_store_Release(IPropertyStore *iface)
+{
+    struct window_prop_store *store = impl_from_IPropertyStore(iface);
+    LONG ref = InterlockedDecrement(&store->ref);
+    if (!ref) free(store);
+    TRACE("returning %ld\n", ref);
+    return ref;
+}
+
+static HRESULT WINAPI window_prop_store_QueryInterface(IPropertyStore *iface, REFIID iid, void **obj)
+{
+    struct window_prop_store *store = impl_from_IPropertyStore(iface);
+    TRACE("%p, %s, %p\n", iface, debugstr_guid(iid), obj);
+
+    if (!obj) return E_POINTER;
+    if (IsEqualIID(iid, &IID_IUnknown) || IsEqualIID(iid, &IID_IPropertyStore))
+    {
+        *obj = &store->IPropertyStore_iface;
+    }
+    else
+    {
+        FIXME("no interface for %s\n", debugstr_guid(iid));
+        *obj = NULL;
+        return E_NOINTERFACE;
+    }
+
+    IUnknown_AddRef((IUnknown *)*obj);
+    return S_OK;
+}
+
+static HRESULT WINAPI window_prop_store_GetCount(IPropertyStore *iface, DWORD *count)
+{
+    FIXME("%p, %p\n", iface, count);
+    return E_NOTIMPL;
+}
+
+static HRESULT WINAPI window_prop_store_GetAt(IPropertyStore *iface, DWORD prop, PROPERTYKEY *key)
+{
+    FIXME("%p, %lu,%p\n", iface, prop, key);
+    return E_NOTIMPL;
+}
+
+static HRESULT WINAPI window_prop_store_GetValue(IPropertyStore *iface, const PROPERTYKEY *key, PROPVARIANT *var)
+{
+    FIXME("%p, {%s,%lu}, %p\n", iface, debugstr_guid(&key->fmtid), key->pid, var);
+    return E_NOTIMPL;
+}
+
+static HRESULT WINAPI window_prop_store_SetValue(IPropertyStore *iface, const PROPERTYKEY *key, const PROPVARIANT *var)
+{
+    FIXME("%p, {%s,%lu}, %p\n", iface, debugstr_guid(&key->fmtid), key->pid, var);
+    return S_OK;
+}
+
+static HRESULT WINAPI window_prop_store_Commit(IPropertyStore *iface)
+{
+    FIXME("%p\n", iface);
+    return S_OK;
+}
+
+static const IPropertyStoreVtbl window_prop_store_vtbl =
+{
+    window_prop_store_QueryInterface,
+    window_prop_store_AddRef,
+    window_prop_store_Release,
+    window_prop_store_GetCount,
+    window_prop_store_GetAt,
+    window_prop_store_GetValue,
+    window_prop_store_SetValue,
+    window_prop_store_Commit
+};
+
+static HRESULT create_window_prop_store(IPropertyStore **obj)
+{
+    struct window_prop_store *store;
+
+    if (!(store = malloc(sizeof(*store)))) return E_OUTOFMEMORY;
+    store->IPropertyStore_iface.lpVtbl = &window_prop_store_vtbl;
+    store->ref = 1;
+
+    *obj = &store->IPropertyStore_iface;
+    return S_OK;
+}
+
 /*************************************************************************
  * SHGetPropertyStoreForWindow [SHELL32.@]
  */
 HRESULT WINAPI SHGetPropertyStoreForWindow(HWND hwnd, REFIID riid, void **ppv)
 {
+    IPropertyStore *store;
+    HRESULT hr;
+
     FIXME("(%p %p %p) stub!\n", hwnd, riid, ppv);
-    return E_NOTIMPL;
+
+    if ((hr = create_window_prop_store( &store )) != S_OK) return hr;
+    hr = IPropertyStore_QueryInterface( store, riid, ppv );
+    IPropertyStore_Release( store );
+    return hr;
 }
 
 /*************************************************************************/
@@ -731,7 +828,7 @@ static void paint_dropline( HDC hdc, HWND hWnd )
  */
 DWORD WINAPI SHHelpShortcuts_RunDLLA(DWORD dwArg1, DWORD dwArg2, DWORD dwArg3, DWORD dwArg4)
 {
-    FIXME("(%x, %x, %x, %x) stub!\n", dwArg1, dwArg2, dwArg3, dwArg4);
+    FIXME("(%lx, %lx, %lx, %lx) stub!\n", dwArg1, dwArg2, dwArg3, dwArg4);
     return 0;
 }
 
@@ -741,7 +838,7 @@ DWORD WINAPI SHHelpShortcuts_RunDLLA(DWORD dwArg1, DWORD dwArg2, DWORD dwArg3, D
  */
 DWORD WINAPI SHHelpShortcuts_RunDLLW(DWORD dwArg1, DWORD dwArg2, DWORD dwArg3, DWORD dwArg4)
 {
-    FIXME("(%x, %x, %x, %x) stub!\n", dwArg1, dwArg2, dwArg3, dwArg4);
+    FIXME("(%lx, %lx, %lx, %lx) stub!\n", dwArg1, dwArg2, dwArg3, dwArg4);
     return 0;
 }
 
@@ -768,30 +865,28 @@ HRESULT WINAPI SHLoadInProc (REFCLSID rclsid)
 
 static void add_authors( HWND list )
 {
-    static const WCHAR eol[] = {'\r','\n',0};
-    static const WCHAR authors[] = {'A','U','T','H','O','R','S',0};
     WCHAR *strW, *start, *end;
-    HRSRC rsrc = FindResourceW( shell32_hInstance, authors, (LPCWSTR)RT_RCDATA );
+    HRSRC rsrc = FindResourceW( shell32_hInstance, L"AUTHORS", (LPCWSTR)RT_RCDATA );
     char *strA = LockResource( LoadResource( shell32_hInstance, rsrc ));
     DWORD sizeW, sizeA = SizeofResource( shell32_hInstance, rsrc );
 
     if (!strA) return;
     sizeW = MultiByteToWideChar( CP_UTF8, 0, strA, sizeA, NULL, 0 ) + 1;
-    if (!(strW = heap_alloc( sizeW * sizeof(WCHAR) ))) return;
+    if (!(strW = malloc( sizeW * sizeof(WCHAR) ))) return;
     MultiByteToWideChar( CP_UTF8, 0, strA, sizeA, strW, sizeW );
     strW[sizeW - 1] = 0;
 
-    start = strpbrkW( strW, eol );  /* skip the header line */
+    start = wcspbrk( strW, L"\r\n" );  /* skip the header line */
     while (start)
     {
-        while (*start && strchrW( eol, *start )) start++;
+        while (*start && wcschr( L"\r\n", *start )) start++;
         if (!*start) break;
-        end = strpbrkW( start, eol );
+        end = wcspbrk( start, L"\r\n" );
         if (end) *end++ = 0;
         SendMessageW( list, LB_ADDSTRING, -1, (LPARAM)start );
         start = end;
     }
-    heap_free( strW );
+    free( strW );
 }
 
 /*************************************************************************
@@ -818,7 +913,7 @@ static INT_PTR CALLBACK AboutDlgProc( HWND hWnd, UINT msg, WPARAM wParam,
                                                             "wine_get_build_id");
                 SendDlgItemMessageW(hWnd, stc1, STM_SETICON,(WPARAM)info->hIcon, 0);
                 GetWindowTextW( hWnd, template, ARRAY_SIZE(template) );
-                sprintfW( buffer, template, info->szApp );
+                swprintf( buffer, ARRAY_SIZE(buffer), template, info->szApp );
                 SetWindowTextW( hWnd, buffer );
                 SetWindowTextW( GetDlgItem(hWnd, IDC_ABOUT_STATIC_TEXT1), info->szApp );
                 SetWindowTextW( GetDlgItem(hWnd, IDC_ABOUT_STATIC_TEXT2), info->szOtherStuff );
@@ -827,7 +922,7 @@ static INT_PTR CALLBACK AboutDlgProc( HWND hWnd, UINT msg, WPARAM wParam,
                 if (wine_get_build_id)
                 {
                     MultiByteToWideChar( CP_UTF8, 0, wine_get_build_id(), -1, version, ARRAY_SIZE(version) );
-                    sprintfW( buffer, template, version );
+                    swprintf( buffer, ARRAY_SIZE(buffer), template, version );
                     SetWindowTextW( GetDlgItem(hWnd, IDC_ABOUT_STATIC_TEXT3), buffer );
                 }
                 hWndCtl = GetDlgItem(hWnd, IDC_ABOUT_LISTBOX);
@@ -892,20 +987,20 @@ BOOL WINAPI ShellAboutA( HWND hWnd, LPCSTR szApp, LPCSTR szOtherStuff, HICON hIc
     if (szApp)
     {
         len = MultiByteToWideChar(CP_ACP, 0, szApp, -1, NULL, 0);
-        appW = heap_alloc( len * sizeof(WCHAR));
+        appW = malloc(len * sizeof(WCHAR));
         MultiByteToWideChar(CP_ACP, 0, szApp, -1, appW, len);
     }
     if (szOtherStuff)
     {
         len = MultiByteToWideChar(CP_ACP, 0, szOtherStuff, -1, NULL, 0);
-        otherW = heap_alloc( len * sizeof(WCHAR));
+        otherW = malloc(len * sizeof(WCHAR));
         MultiByteToWideChar(CP_ACP, 0, szOtherStuff, -1, otherW, len);
     }
 
     ret = ShellAboutW(hWnd, appW, otherW, hIcon);
 
-    heap_free(otherW);
-    heap_free(appW);
+    free(otherW);
+    free(appW);
     return ret;
 }
 
@@ -919,8 +1014,6 @@ BOOL WINAPI ShellAboutW( HWND hWnd, LPCWSTR szApp, LPCWSTR szOtherStuff,
     ABOUT_INFO info;
     LOGFONTW logFont;
     BOOL bRet;
-    static const WCHAR wszSHELL_ABOUT_MSGBOX[] =
-        {'S','H','E','L','L','_','A','B','O','U','T','_','M','S','G','B','O','X',0};
 
     TRACE("\n");
 
@@ -932,7 +1025,7 @@ BOOL WINAPI ShellAboutW( HWND hWnd, LPCWSTR szApp, LPCWSTR szOtherStuff,
     SystemParametersInfoW( SPI_GETICONTITLELOGFONT, 0, &logFont, 0 );
     info.hFont = CreateFontIndirectW( &logFont );
 
-    bRet = DialogBoxParamW( shell32_hInstance, wszSHELL_ABOUT_MSGBOX, hWnd, AboutDlgProc, (LPARAM)&info );
+    bRet = DialogBoxParamW( shell32_hInstance, L"SHELL_ABOUT_MSGBOX", hWnd, AboutDlgProc, (LPARAM)&info );
     DeleteObject(info.hFont);
     return bRet;
 }
@@ -942,7 +1035,7 @@ BOOL WINAPI ShellAboutW( HWND hWnd, LPCWSTR szApp, LPCWSTR szOtherStuff,
  */
 void WINAPI FreeIconList( DWORD dw )
 {
-    FIXME("%x: stub\n",dw);
+    FIXME("%lx: stub\n",dw);
 }
 
 /*************************************************************************
@@ -990,7 +1083,7 @@ HRESULT WINAPI DllGetVersion (DLLVERSIONINFO *pdvi)
                                               WINE_FILEVERSION_BUILD,
                                               WINE_FILEVERSION_PLATFORMID);
         }
-        TRACE("%u.%u.%u.%u\n",
+        TRACE("%lu.%lu.%lu.%lu\n",
               pdvi->dwMajorVersion, pdvi->dwMinorVersion,
               pdvi->dwBuildNumber, pdvi->dwPlatformID);
         return S_OK;
@@ -1018,7 +1111,7 @@ HINSTANCE    shell32_hInstance = 0;
  */
 BOOL WINAPI DllMain(HINSTANCE hinstDLL, DWORD fdwReason, LPVOID fImpLoad)
 {
-    TRACE("%p 0x%x %p\n", hinstDLL, fdwReason, fImpLoad);
+    TRACE("%p 0x%lx %p\n", hinstDLL, fdwReason, fImpLoad);
 
     switch (fdwReason)
     {
@@ -1059,20 +1152,12 @@ HRESULT WINAPI DllInstall(BOOL bInstall, LPCWSTR cmdline)
     return S_OK;        /* indicate success */
 }
 
-/***********************************************************************
- *              DllCanUnloadNow (SHELL32.@)
- */
-HRESULT WINAPI DllCanUnloadNow(void)
-{
-    return S_FALSE;
-}
-
 /***********************************************************************
  *		DllRegisterServer (SHELL32.@)
  */
 HRESULT WINAPI DllRegisterServer(void)
 {
-    HRESULT hr = __wine_register_resources( shell32_hInstance );
+    HRESULT hr = __wine_register_resources();
     if (SUCCEEDED(hr)) hr = SHELL_RegisterShellFolders();
     return hr;
 }
@@ -1082,7 +1167,7 @@ HRESULT WINAPI DllRegisterServer(void)
  */
 HRESULT WINAPI DllUnregisterServer(void)
 {
-    return __wine_unregister_resources( shell32_hInstance );
+    return __wine_unregister_resources();
 }
 
 /***********************************************************************
@@ -1090,7 +1175,7 @@ HRESULT WINAPI DllUnregisterServer(void)
  */
 BOOL WINAPI ExtractVersionResource16W(LPWSTR s, DWORD d)
 {
-    FIXME("(%s %x) stub!\n", debugstr_w(s), d);
+    FIXME("(%s %lx) stub!\n", debugstr_w(s), d);
     return FALSE;
 }
 
@@ -1126,7 +1211,7 @@ HRESULT WINAPI SHGetLocalizedName(LPCWSTR path, LPWSTR module, UINT size, INT *r
  */
 HRESULT WINAPI SHSetUnreadMailCountW(LPCWSTR mailaddress, DWORD count, LPCWSTR executecommand)
 {
-    FIXME("%s %x %s: stub\n", debugstr_w(mailaddress), count, debugstr_w(executecommand));
+    FIXME("%s %lx %s: stub\n", debugstr_w(mailaddress), count, debugstr_w(executecommand));
     return E_NOTIMPL;
 }
 
@@ -1135,7 +1220,7 @@ HRESULT WINAPI SHSetUnreadMailCountW(LPCWSTR mailaddress, DWORD count, LPCWSTR e
  */
 HRESULT WINAPI SHEnumerateUnreadMailAccountsW(HKEY user, DWORD idx, LPWSTR mailaddress, INT mailaddresslen)
 {
-    FIXME("%p %d %p %d: stub\n", user, idx, mailaddress, mailaddresslen);
+    FIXME("%p %ld %p %d: stub\n", user, idx, mailaddress, mailaddresslen);
     return E_NOTIMPL;
 }
 
@@ -1148,3 +1233,13 @@ HRESULT WINAPI SHQueryUserNotificationState(QUERY_USER_NOTIFICATION_STATE *state
     *state = QUNS_ACCEPTS_NOTIFICATIONS;
     return S_OK;
 }
+
+/***********************************************************************
+ *              SHCreateDataObject (SHELL32.@)
+ */
+HRESULT WINAPI SHCreateDataObject(PCIDLIST_ABSOLUTE pidl_folder, UINT count, PCUITEMID_CHILD_ARRAY pidl_array,
+                                  IDataObject *object, REFIID riid, void **ppv)
+{
+    FIXME("%p %d %p %p %s %p: stub\n", pidl_folder, count, pidl_array, object, debugstr_guid(riid), ppv);
+    return E_NOTIMPL;
+}
