@@ -112,7 +112,7 @@ static DC_ATTR *alloc_dc_attr(void)
     {
         SIZE_T size = system_info.AllocationGranularity;
         bucket->entries = NULL;
-        if (!NtAllocateVirtualMemory( GetCurrentProcess(), (void **)&bucket->entries, zero_bits(),
+        if (!NtAllocateVirtualMemory( GetCurrentProcess(), (void **)&bucket->entries, zero_bits,
                                       &size, MEM_RESERVE | MEM_COMMIT, PAGE_READWRITE ))
         {
             bucket->next_free = NULL;
@@ -278,7 +278,7 @@ void free_dc_ptr( DC *dc )
     if (dc->hBitmap)
     {
         if (dc->is_display)
-            NtGdiDeleteClientObj( dc->hBitmap );
+            NtGdiDeleteObjectApp( dc->hBitmap );
         else
             GDI_dec_ref_count( dc->hBitmap );
     }
@@ -717,11 +717,10 @@ HDC WINAPI NtGdiOpenDCW( UNICODE_STRING *device, const DEVMODEW *devmode, UNICOD
     /* gdi_lock should not be locked */
     if (is_display)
         funcs = get_display_driver();
-    else if (hspool)
-    {
-        const struct gdi_dc_funcs * (CDECL *wine_get_gdi_driver)( unsigned int ) = hspool;
-        funcs = wine_get_gdi_driver( WINE_GDI_DRIVER_VERSION );
-    }
+    else if (type != WINE_GDI_DRIVER_VERSION)
+        ERR( "version mismatch: %u\n", (unsigned int)type );
+    else
+        funcs = hspool;
     if (!funcs)
     {
         ERR( "no driver found\n" );
@@ -732,7 +731,7 @@ HDC WINAPI NtGdiOpenDCW( UNICODE_STRING *device, const DEVMODEW *devmode, UNICOD
     hdc = dc->hSelf;
 
     if (is_display)
-        dc->hBitmap = NtGdiCreateClientObj( NTGDI_OBJ_SURF );
+        dc->hBitmap = NtGdiCreateCompatibleBitmap( hdc, 1, 1 );
     else
         dc->hBitmap = GDI_inc_ref_count( GetStockObject( DEFAULT_BITMAP ));
 
@@ -1474,7 +1473,7 @@ DWORD WINAPI NtGdiSetLayout( HDC hdc, LONG wox, DWORD layout )
 /**********************************************************************
  *           __wine_get_icm_profile     (win32u.@)
  */
-BOOL CDECL __wine_get_icm_profile( HDC hdc, BOOL allow_default, DWORD *size, WCHAR *filename )
+BOOL WINAPI __wine_get_icm_profile( HDC hdc, BOOL allow_default, DWORD *size, WCHAR *filename )
 {
     PHYSDEV physdev;
     DC *dc;
@@ -1491,7 +1490,7 @@ BOOL CDECL __wine_get_icm_profile( HDC hdc, BOOL allow_default, DWORD *size, WCH
 /***********************************************************************
  *      __wine_get_wgl_driver  (win32u.@)
  */
-struct opengl_funcs *__wine_get_wgl_driver( HDC hdc, UINT version )
+const struct opengl_funcs *__wine_get_wgl_driver( HDC hdc, UINT version )
 {
     BOOL is_display, is_memdc;
     DC *dc;
