@@ -40,6 +40,16 @@ WINE_DECLARE_DEBUG_CHANNEL(win);
 
 #define DESKTOP_ALL_ACCESS 0x01ff
 
+BOOL is_virtual_desktop(void)
+{
+    HANDLE desktop = NtUserGetThreadDesktop( GetCurrentThreadId() );
+    USEROBJECTFLAGS flags = {0};
+    DWORD len;
+
+    if (!NtUserGetObjectInformation( desktop, UOI_FLAGS, &flags, sizeof(flags), &len )) return FALSE;
+    return !!(flags.dwFlags & DF_WINE_VIRTUAL_DESKTOP);
+}
+
 /***********************************************************************
  *           NtUserCreateWindowStation  (win32u.@)
  */
@@ -141,9 +151,10 @@ HDESK WINAPI NtUserCreateDesktopEx( OBJECT_ATTRIBUTES *attr, UNICODE_STRING *dev
                                     DEVMODEW *devmode, DWORD flags, ACCESS_MASK access,
                                     ULONG heap_size )
 {
+    WCHAR buffer[MAX_PATH];
     HANDLE ret;
 
-    if ((device && device->Length) || devmode)
+    if ((device && device->Length) || (devmode && !(flags & DF_WINE_VIRTUAL_DESKTOP)))
     {
         RtlSetLastWin32Error( ERROR_INVALID_PARAMETER );
         return 0;
@@ -163,6 +174,17 @@ HDESK WINAPI NtUserCreateDesktopEx( OBJECT_ATTRIBUTES *attr, UNICODE_STRING *dev
         ret = wine_server_ptr_handle( reply->handle );
     }
     SERVER_END_REQ;
+    if (!devmode) return ret;
+
+    lstrcpynW( buffer, attr->ObjectName->Buffer, attr->ObjectName->Length / sizeof(WCHAR) + 1 );
+    if (!user_driver->pCreateDesktop( buffer, devmode->dmPelsWidth, devmode->dmPelsHeight ))
+    {
+        NtUserCloseDesktop( ret );
+        return 0;
+    }
+
+    /* force update display cache to use virtual desktop display settings */
+    if (flags & DF_WINE_VIRTUAL_DESKTOP) update_display_cache( TRUE );
     return ret;
 }
 
@@ -226,7 +248,7 @@ HDESK WINAPI NtUserGetThreadDesktop( DWORD thread )
  */
 BOOL WINAPI NtUserSetThreadDesktop( HDESK handle )
 {
-    BOOL ret;
+    BOOL ret, was_virtual_desktop = is_virtual_desktop();
 
     SERVER_START_REQ( set_thread_desktop )
     {
@@ -242,6 +264,7 @@ BOOL WINAPI NtUserSetThreadDesktop( HDESK handle )
         thread_info->client_info.top_window = 0;
         thread_info->client_info.msg_window = 0;
         if (key_state_info) key_state_info->time = 0;
+        if (was_virtual_desktop != is_virtual_desktop()) update_display_cache( TRUE );
     }
     return ret;
 }
@@ -397,14 +420,24 @@ static inline TEB64 *NtCurrentTeb64(void) { return (TEB64 *)NtCurrentTeb()->GdiB
 
 HWND get_desktop_window(void)
 {
+    static const WCHAR wine_service_station_name[] =
+        {'_','_','w','i','n','e','s','e','r','v','i','c','e','_','w','i','n','s','t','a','t','i','o','n',0};
     struct ntuser_thread_info *thread_info = NtUserGetThreadInfo();
+    WCHAR name[MAX_PATH];
+    BOOL is_service;
 
     if (thread_info->top_window) return UlongToHandle( thread_info->top_window );
 
+    /* don't create an actual explorer desktop window for services */
+    if (NtUserGetObjectInformation( NtUserGetProcessWindowStation(), UOI_NAME, name, sizeof(name), NULL )
+        && !wcscmp( name, wine_service_station_name ))
+        is_service = TRUE;
+    else
+        is_service = FALSE;
 
     SERVER_START_REQ( get_desktop_window )
     {
-        req->force = 0;
+        req->force = is_service;
         if (!wine_server_call( req ))
         {
             thread_info->top_window = reply->top_window;
@@ -423,7 +456,8 @@ HWND get_desktop_window(void)
         static const WCHAR system_dir[] = {'C',':','\\','w','i','n','d','o','w','s','\\',
             's','y','s','t','e','m','3','2','\\',0};
         RTL_USER_PROCESS_PARAMETERS params = { sizeof(params), sizeof(params) };
-        PS_ATTRIBUTE_LIST ps_attr;
+        ULONG_PTR buffer[offsetof( PS_ATTRIBUTE_LIST, Attributes[2] ) / sizeof(ULONG_PTR)];
+        PS_ATTRIBUTE_LIST *ps_attr = (PS_ATTRIBUTE_LIST *)buffer;
         PS_CREATE_INFO create_info;
         WCHAR desktop[MAX_PATH];
         PEB *peb = NtCurrentTeb()->Peb;
@@ -456,24 +490,30 @@ HWND get_desktop_window(void)
         RtlInitUnicodeString( &params.WindowTitle, appnameW + 4 );
         RtlInitUnicodeString( &params.Desktop, desktop );
 
-        ps_attr.TotalLength = sizeof(ps_attr);
-        ps_attr.Attributes[0].Attribute    = PS_ATTRIBUTE_IMAGE_NAME;
-        ps_attr.Attributes[0].Size         = sizeof(appnameW) - sizeof(WCHAR);
-        ps_attr.Attributes[0].ValuePtr     = (WCHAR *)appnameW;
-        ps_attr.Attributes[0].ReturnLength = NULL;
+        ps_attr->Attributes[0].Attribute    = PS_ATTRIBUTE_IMAGE_NAME;
+        ps_attr->Attributes[0].Size         = sizeof(appnameW) - sizeof(WCHAR);
+        ps_attr->Attributes[0].ValuePtr     = (WCHAR *)appnameW;
+        ps_attr->Attributes[0].ReturnLength = NULL;
+
+        ps_attr->Attributes[1].Attribute    = PS_ATTRIBUTE_TOKEN;
+        ps_attr->Attributes[1].Size         = sizeof(HANDLE);
+        ps_attr->Attributes[1].ValuePtr     = GetCurrentThreadEffectiveToken();
+        ps_attr->Attributes[1].ReturnLength = NULL;
+
+        ps_attr->TotalLength = offsetof( PS_ATTRIBUTE_LIST, Attributes[2] );
 
         if (NtCurrentTeb64() && !NtCurrentTeb64()->TlsSlots[WOW64_TLS_FILESYSREDIR])
         {
             NtCurrentTeb64()->TlsSlots[WOW64_TLS_FILESYSREDIR] = TRUE;
             status = NtCreateUserProcess( &process, &thread, PROCESS_ALL_ACCESS, THREAD_ALL_ACCESS,
                                           NULL, NULL, 0, THREAD_CREATE_FLAGS_CREATE_SUSPENDED, &params,
-                                          &create_info, &ps_attr );
+                                          &create_info, ps_attr );
             NtCurrentTeb64()->TlsSlots[WOW64_TLS_FILESYSREDIR] = FALSE;
         }
         else
             status = NtCreateUserProcess( &process, &thread, PROCESS_ALL_ACCESS, THREAD_ALL_ACCESS,
                                           NULL, NULL, 0, THREAD_CREATE_FLAGS_CREATE_SUSPENDED, &params,
-                                          &create_info, &ps_attr );
+                                          &create_info, ps_attr );
         if (!status)
         {
             NtResumeThread( thread, NULL );
@@ -496,9 +536,8 @@ HWND get_desktop_window(void)
         SERVER_END_REQ;
     }
 
-    if (!thread_info->top_window ||
-        !user_driver->pCreateDesktopWindow( UlongToHandle( thread_info->top_window )))
-        ERR_(win)( "failed to create desktop window\n" );
+    if (!thread_info->top_window) ERR_(win)( "failed to create desktop window\n" );
+    else user_driver->pSetDesktopWindow( UlongToHandle( thread_info->top_window ));
 
     register_builtin_classes();
     return UlongToHandle( thread_info->top_window );
@@ -515,7 +554,8 @@ static HANDLE get_winstations_dir_handle(void)
 
     sprintf( bufferA, "\\Sessions\\%u\\Windows\\WindowStations", (int)NtCurrentTeb()->Peb->SessionId );
     str.Buffer = buffer;
-    str.Length = str.MaximumLength = asciiz_to_unicode( buffer, bufferA ) - sizeof(WCHAR);
+    str.MaximumLength = asciiz_to_unicode( buffer, bufferA );
+    str.Length = str.MaximumLength - sizeof(WCHAR);
     InitializeObjectAttributes( &attr, &str, 0, 0, NULL );
     status = NtOpenDirectoryObject( &dir, DIRECTORY_CREATE_OBJECT | DIRECTORY_TRAVERSE, &attr );
     return status ? 0 : dir;
