@@ -24,13 +24,17 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA 02110-1301, USA
  */
 
+#if 0
+#pragma makedep unix
+#endif
+
 #include "config.h"
-#include "wine/port.h"
 
 #include <assert.h>
 #include <stdarg.h>
 #include <stdlib.h>
 #include <string.h>
+#include <dlfcn.h>
 #ifdef HAVE_EGL_EGL_H
 #include <EGL/egl.h>
 #endif
@@ -42,8 +46,6 @@
 #include "wine/wgl.h"
 #undef GLAPIENTRY
 #include "wine/wgl_driver.h"
-#include "wine/wglext.h"
-#include "wine/library.h"
 #include "wine/debug.h"
 
 WINE_DEFAULT_DEBUG_CHANNEL(android);
@@ -107,14 +109,7 @@ static struct list gl_drawables = LIST_INIT( gl_drawables );
 static void (*pglFinish)(void);
 static void (*pglFlush)(void);
 
-static CRITICAL_SECTION drawable_section;
-static CRITICAL_SECTION_DEBUG critsect_debug =
-{
-    0, 0, &drawable_section,
-    { &critsect_debug.ProcessLocksList, &critsect_debug.ProcessLocksList },
-      0, 0, { (DWORD_PTR)(__FILE__ ": drawable_section") }
-};
-static CRITICAL_SECTION drawable_section = { &critsect_debug, -1, 0, 0, 0, 0 };
+pthread_mutex_t drawable_mutex;
 
 static inline BOOL is_onscreen_pixel_format( int format )
 {
@@ -124,15 +119,15 @@ static inline BOOL is_onscreen_pixel_format( int format )
 static struct gl_drawable *create_gl_drawable( HWND hwnd, HDC hdc, int format )
 {
     static const int attribs[] = { EGL_WIDTH, 1, EGL_HEIGHT, 1, EGL_NONE };
-    struct gl_drawable *gl = HeapAlloc( GetProcessHeap(), 0, sizeof(*gl) );
+    struct gl_drawable *gl = malloc( sizeof(*gl) );
 
     gl->hwnd   = hwnd;
     gl->hdc    = hdc;
     gl->format = format;
-    gl->window = create_ioctl_window( hwnd, TRUE );
+    gl->window = create_ioctl_window( hwnd, TRUE, 1.0f );
     gl->surface = 0;
     gl->pbuffer = p_eglCreatePbufferSurface( display, pixel_formats[gl->format - 1].config, attribs );
-    EnterCriticalSection( &drawable_section );
+    pthread_mutex_lock( &drawable_mutex );
     list_add_head( &gl_drawables, &gl->entry );
     return gl;
 }
@@ -141,26 +136,26 @@ static struct gl_drawable *get_gl_drawable( HWND hwnd, HDC hdc )
 {
     struct gl_drawable *gl;
 
-    EnterCriticalSection( &drawable_section );
+    pthread_mutex_lock( &drawable_mutex );
     LIST_FOR_EACH_ENTRY( gl, &gl_drawables, struct gl_drawable, entry )
     {
         if (hwnd && gl->hwnd == hwnd) return gl;
         if (hdc && gl->hdc == hdc) return gl;
     }
-    LeaveCriticalSection( &drawable_section );
+    pthread_mutex_unlock( &drawable_mutex );
     return NULL;
 }
 
 static void release_gl_drawable( struct gl_drawable *gl )
 {
-    if (gl) LeaveCriticalSection( &drawable_section );
+    if (gl) pthread_mutex_unlock( &drawable_mutex );
 }
 
 void destroy_gl_drawable( HWND hwnd )
 {
     struct gl_drawable *gl;
 
-    EnterCriticalSection( &drawable_section );
+    pthread_mutex_lock( &drawable_mutex );
     LIST_FOR_EACH_ENTRY( gl, &gl_drawables, struct gl_drawable, entry )
     {
         if (gl->hwnd != hwnd) continue;
@@ -168,10 +163,10 @@ void destroy_gl_drawable( HWND hwnd )
         if (gl->surface) p_eglDestroySurface( display, gl->surface );
         if (gl->pbuffer) p_eglDestroySurface( display, gl->pbuffer );
         release_ioctl_window( gl->window );
-        HeapFree( GetProcessHeap(), 0, gl );
+        free( gl );
         break;
     }
-    LeaveCriticalSection( &drawable_section );
+    pthread_mutex_unlock( &drawable_mutex );
 }
 
 static BOOL refresh_context( struct wgl_context *ctx )
@@ -182,7 +177,7 @@ static BOOL refresh_context( struct wgl_context *ctx )
     {
         TRACE( "refreshing hwnd %p context %p surface %p\n", ctx->hwnd, ctx->context, ctx->surface );
         p_eglMakeCurrent( display, ctx->surface, ctx->surface, ctx->context );
-        RedrawWindow( ctx->hwnd, NULL, 0, RDW_INVALIDATE | RDW_ERASE );
+        NtUserRedrawWindow( ctx->hwnd, NULL, 0, RDW_INVALIDATE | RDW_ERASE );
     }
     return ret;
 }
@@ -209,17 +204,16 @@ void update_gl_drawable( HWND hwnd )
             }
         }
         release_gl_drawable( gl );
-        RedrawWindow( hwnd, NULL, 0, RDW_INVALIDATE | RDW_ERASE );
+        NtUserRedrawWindow( hwnd, NULL, 0, RDW_INVALIDATE | RDW_ERASE );
     }
 }
 
-static BOOL set_pixel_format( HDC hdc, int format, BOOL allow_change )
+static BOOL set_pixel_format( HDC hdc, int format, BOOL internal )
 {
     struct gl_drawable *gl;
-    HWND hwnd = WindowFromDC( hdc );
-    int prev = 0;
+    HWND hwnd = NtUserWindowFromDC( hdc );
 
-    if (!hwnd || hwnd == GetDesktopWindow())
+    if (!hwnd || hwnd == NtUserGetDesktopWindow())
     {
         WARN( "not a proper window DC %p/%p\n", hdc, hwnd );
         return FALSE;
@@ -231,10 +225,18 @@ static BOOL set_pixel_format( HDC hdc, int format, BOOL allow_change )
     }
     TRACE( "%p/%p format %d\n", hdc, hwnd, format );
 
+    if (!internal)
+    {
+        /* cannot change it if already set */
+        int prev = win32u_get_window_pixel_format( hwnd );
+
+        if (prev)
+            return prev == format;
+    }
+
     if ((gl = get_gl_drawable( hwnd, 0 )))
     {
-        prev = gl->format;
-        if (allow_change)
+        if (internal)
         {
             EGLint pf;
             p_eglGetConfigAttrib( display, pixel_formats[format - 1].config, EGL_NATIVE_VISUAL_ID, &pf );
@@ -246,8 +248,7 @@ static BOOL set_pixel_format( HDC hdc, int format, BOOL allow_change )
 
     release_gl_drawable( gl );
 
-    if (prev && prev != format && !allow_change) return FALSE;
-    if (__wine_set_pixel_format( hwnd, format )) return TRUE;
+    if (win32u_set_window_pixel_format( hwnd, format, internal )) return TRUE;
     destroy_gl_drawable( hwnd );
     return FALSE;
 }
@@ -257,9 +258,9 @@ static struct wgl_context *create_context( HDC hdc, struct wgl_context *share, c
     struct gl_drawable *gl;
     struct wgl_context *ctx;
 
-    if (!(gl = get_gl_drawable( WindowFromDC( hdc ), hdc ))) return NULL;
+    if (!(gl = get_gl_drawable( NtUserWindowFromDC( hdc ), hdc ))) return NULL;
 
-    ctx = HeapAlloc( GetProcessHeap(), 0, sizeof(*ctx) );
+    ctx = malloc( sizeof(*ctx) );
 
     ctx->config  = pixel_formats[gl->format - 1].config;
     ctx->surface = 0;
@@ -350,10 +351,10 @@ static BOOL android_wglMakeContextCurrentARB( HDC draw_hdc, HDC read_hdc, struct
         return TRUE;
     }
 
-    draw_hwnd = WindowFromDC( draw_hdc );
+    draw_hwnd = NtUserWindowFromDC( draw_hdc );
     if ((draw_gl = get_gl_drawable( draw_hwnd, draw_hdc )))
     {
-        read_gl = get_gl_drawable( WindowFromDC( read_hdc ), read_hdc );
+        read_gl = get_gl_drawable( NtUserWindowFromDC( read_hdc ), read_hdc );
         draw_surface = draw_gl->surface ? draw_gl->surface : draw_gl->pbuffer;
         read_surface = read_gl->surface ? read_gl->surface : read_gl->pbuffer;
         TRACE( "%p/%p context %p surface %p/%p\n",
@@ -368,7 +369,7 @@ static BOOL android_wglMakeContextCurrentARB( HDC draw_hdc, HDC read_hdc, struct
             goto done;
         }
     }
-    SetLastError( ERROR_INVALID_HANDLE );
+    RtlSetLastWin32Error( ERROR_INVALID_HANDLE );
 
 done:
     release_gl_drawable( read_gl );
@@ -387,7 +388,7 @@ static BOOL android_wglSwapIntervalEXT( int interval )
 
     if (interval < 0)
     {
-        SetLastError(ERROR_INVALID_DATA);
+        RtlSetLastWin32Error( ERROR_INVALID_DATA );
         return FALSE;
     }
 
@@ -396,7 +397,7 @@ static BOOL android_wglSwapIntervalEXT( int interval )
     if (ret)
         swap_interval = interval;
     else
-        SetLastError( ERROR_DC_NOT_FOUND );
+        RtlSetLastWin32Error( ERROR_DC_NOT_FOUND );
 
     return ret;
 }
@@ -439,13 +440,14 @@ static struct wgl_context *android_wglCreateContext( HDC hdc )
 /***********************************************************************
  *		android_wglDeleteContext
  */
-static void android_wglDeleteContext( struct wgl_context *ctx )
+static BOOL android_wglDeleteContext( struct wgl_context *ctx )
 {
-    EnterCriticalSection( &drawable_section );
+    pthread_mutex_lock( &drawable_mutex );
     list_remove( &ctx->entry );
-    LeaveCriticalSection( &drawable_section );
+    pthread_mutex_unlock( &drawable_mutex );
     p_eglDestroyContext( display, ctx->context );
-    HeapFree( GetProcessHeap(), 0, ctx );
+    free( ctx );
+    return TRUE;
 }
 
 /***********************************************************************
@@ -464,7 +466,7 @@ static int android_wglDescribePixelFormat( HDC hdc, int fmt, UINT size, PIXELFOR
     memset( pfd, 0, sizeof(*pfd) );
     pfd->nSize = sizeof(*pfd);
     pfd->nVersion = 1;
-    pfd->dwFlags = PFD_SUPPORT_OPENGL | PFD_DRAW_TO_WINDOW | PFD_DOUBLEBUFFER;
+    pfd->dwFlags = PFD_SUPPORT_OPENGL | PFD_DRAW_TO_WINDOW | PFD_DOUBLEBUFFER | PFD_SUPPORT_COMPOSITION;
     pfd->iPixelType = PFD_TYPE_RGBA;
     pfd->iLayerType = PFD_MAIN_PLANE;
 
@@ -501,8 +503,14 @@ static int android_wglGetPixelFormat( HDC hdc )
 {
     struct gl_drawable *gl;
     int ret = 0;
+    HWND hwnd;
 
-    if ((gl = get_gl_drawable( WindowFromDC( hdc ), hdc )))
+    if ((hwnd = NtUserWindowFromDC( hdc )))
+        return win32u_get_window_pixel_format( hwnd );
+
+    /* This code is currently dead, but will be necessary if WGL_ARB_pbuffer
+     * support is introduced. */
+    if ((gl = get_gl_drawable( NULL, hdc )))
     {
         ret = gl->format;
         /* offscreen formats can't be used with traditional WGL calls */
@@ -542,7 +550,7 @@ static BOOL android_wglMakeCurrent( HDC hdc, struct wgl_context *ctx )
         return TRUE;
     }
 
-    hwnd = WindowFromDC( hdc );
+    hwnd = NtUserWindowFromDC( hdc );
     if ((gl = get_gl_drawable( hwnd, hdc )))
     {
         EGLSurface surface = gl->surface ? gl->surface : gl->pbuffer;
@@ -557,7 +565,7 @@ static BOOL android_wglMakeCurrent( HDC hdc, struct wgl_context *ctx )
             goto done;
         }
     }
-    SetLastError( ERROR_INVALID_HANDLE );
+    RtlSetLastWin32Error( ERROR_INVALID_HANDLE );
 
 done:
     release_gl_drawable( gl );
@@ -656,11 +664,11 @@ static void init_extensions(void)
 
     /* load standard functions and extensions exported from the OpenGL library */
 
-#define USE_GL_FUNC(func) if ((ptr = wine_dlsym( opengl_handle, #func, NULL, 0 ))) egl_funcs.gl.p_##func = ptr;
+#define USE_GL_FUNC(func) if ((ptr = dlsym( opengl_handle, #func ))) egl_funcs.gl.p_##func = ptr;
     ALL_WGL_FUNCS
 #undef USE_GL_FUNC
 
-#define LOAD_FUNCPTR(func) egl_funcs.ext.p_##func = wine_dlsym( opengl_handle, #func, NULL, 0 )
+#define LOAD_FUNCPTR(func) egl_funcs.ext.p_##func = dlsym( opengl_handle, #func )
     LOAD_FUNCPTR( glActiveShaderProgram );
     LOAD_FUNCPTR( glActiveTexture );
     LOAD_FUNCPTR( glAttachShader );
@@ -950,24 +958,23 @@ static BOOL egl_init(void)
     static int retval = -1;
     EGLConfig *configs;
     EGLint major, minor, count, i, pass;
-    char buffer[200];
 
     if (retval != -1) return retval;
     retval = 0;
 
-    if (!(egl_handle = wine_dlopen( SONAME_LIBEGL, RTLD_NOW|RTLD_GLOBAL, buffer, sizeof(buffer) )))
+    if (!(egl_handle = dlopen( SONAME_LIBEGL, RTLD_NOW|RTLD_GLOBAL )))
     {
-        ERR( "failed to load %s: %s\n", SONAME_LIBEGL, buffer );
+        ERR( "failed to load %s: %s\n", SONAME_LIBEGL, dlerror() );
         return FALSE;
     }
-    if (!(opengl_handle = wine_dlopen( SONAME_LIBGLESV2, RTLD_NOW|RTLD_GLOBAL, buffer, sizeof(buffer) )))
+    if (!(opengl_handle = dlopen( SONAME_LIBGLESV2, RTLD_NOW|RTLD_GLOBAL )))
     {
-        ERR( "failed to load %s: %s\n", SONAME_LIBGLESV2, buffer );
+        ERR( "failed to load %s: %s\n", SONAME_LIBGLESV2, dlerror() );
         return FALSE;
     }
 
 #define LOAD_FUNCPTR(func) do { \
-        if (!(p_##func = wine_dlsym( egl_handle, #func, NULL, 0 ))) \
+        if (!(p_##func = dlsym( egl_handle, #func ))) \
         { ERR( "can't find symbol %s\n", #func); return FALSE; }    \
     } while(0)
     LOAD_FUNCPTR( eglCreateContext );
@@ -990,13 +997,13 @@ static BOOL egl_init(void)
     TRACE( "display %p version %u.%u\n", display, major, minor );
 
     p_eglGetConfigs( display, NULL, 0, &count );
-    configs = HeapAlloc( GetProcessHeap(), 0, count * sizeof(*configs) );
-    pixel_formats = HeapAlloc( GetProcessHeap(), 0, count * sizeof(*pixel_formats) );
+    configs = malloc( count * sizeof(*configs) );
+    pixel_formats = malloc( count * sizeof(*pixel_formats) );
     p_eglGetConfigs( display, configs, count, &count );
     if (!count || !configs || !pixel_formats)
     {
-        HeapFree( GetProcessHeap(), 0, configs );
-        HeapFree( GetProcessHeap(), 0, pixel_formats );
+        free( configs );
+        free( pixel_formats );
         ERR( "eglGetConfigs returned no configs\n" );
         return 0;
     }
