@@ -19,9 +19,7 @@
 #include "config.h"
 
 #include <sys/types.h>
-#ifdef HAVE_DIRENT_H
-# include <dirent.h>
-#endif
+#include <dirent.h>
 #include <string.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -191,7 +189,7 @@ static void read_afms(FILE *f_c, FILE *f_h)
 	int	i, num_metrics;
 
 	cp = strrchr(de->d_name, '.');	    	    	    /* Does it end in   */
-	if (cp == NULL || _strnicmp(cp, ".afm", -1) != 0)   /*   .afm or .AFM?  */
+	if (cp == NULL || stricmp(cp, ".afm") != 0)   /*   .afm or .AFM?  */
 	    continue;
 
 	f = fopen(de->d_name, "r");
@@ -219,7 +217,7 @@ static void read_afms(FILE *f_c, FILE *f_h)
 	    if (font_family[i] == '-')
 	    	font_family[i] = '_';
 
-	fprintf(f_h, "DECLSPEC_HIDDEN extern const AFM PSDRV_%s;\n", font_family);
+	fprintf(f_h, "extern const AFM PSDRV_%s;\n", font_family);
 	fprintf(f_c, "    &PSDRV_%s,\n", font_family);
 
 	while (1)
