@@ -68,7 +68,7 @@ void WCMD_batch (WCHAR *file, WCHAR *command, BOOL called, WCHAR *startLabel, HA
   prev_context = context;
   context = LocalAlloc (LMEM_FIXED, sizeof (BATCH_CONTEXT));
   context -> h = h;
-  context->batchfileW = heap_strdupW(file);
+  context->batchfileW = xstrdupW(file);
   context -> command = command;
   memset(context -> shift_count, 0x00, sizeof(context -> shift_count));
   context -> prev_context = prev_context;
@@ -110,8 +110,8 @@ void WCMD_batch (WCHAR *file, WCHAR *command, BOOL called, WCHAR *startLabel, HA
  *	to the caller's caller.
  */
 
-  heap_free(context->batchfileW);
-  LocalFree (context);
+  free(context->batchfileW);
+  LocalFree(context);
   if ((prev_context != NULL) && (!called)) {
     WINE_TRACE("Batch completed, but was not 'called' so skipping outer batch too\n");
     prev_context -> skip_rest = TRUE;
@@ -224,8 +224,7 @@ WCHAR *WCMD_parameter_with_delims (WCHAR *s, int n, WCHAR **start,
 WCHAR *WCMD_parameter (WCHAR *s, int n, WCHAR **start, BOOL raw,
                        BOOL wholecmdline)
 {
-  static const WCHAR defaultDelims[] = { ' ', '\t', ',', '=', ';', '\0' };
-  return WCMD_parameter_with_delims (s, n, start, raw, wholecmdline, defaultDelims);
+  return WCMD_parameter_with_delims (s, n, start, raw, wholecmdline, L" \t,=;");
 }
 
 /****************************************************************************
@@ -250,14 +249,14 @@ WCHAR *WCMD_fgets(WCHAR *buf, DWORD noChars, HANDLE h)
   /* We can't use the native f* functions because of the filename syntax differences
      between DOS and Unix. Also need to lose the LF (or CRLF) from the line. */
 
-  if (!WCMD_is_console_handle(h)) {
+  if (!ReadConsoleW(h, buf, noChars, &charsRead, NULL)) {
       LARGE_INTEGER filepos;
       char *bufA;
       UINT cp;
       const char *p;
 
-      cp = GetConsoleCP();
-      bufA = heap_xalloc(noChars);
+      cp = GetOEMCP();
+      bufA = xalloc(noChars);
 
       /* Save current file position */
       filepos.QuadPart = 0;
@@ -265,7 +264,7 @@ WCHAR *WCMD_fgets(WCHAR *buf, DWORD noChars, HANDLE h)
 
       status = ReadFile(h, bufA, noChars, &charsRead, NULL);
       if (!status || charsRead == 0) {
-          heap_free(bufA);
+          free(bufA);
           return NULL;
       }
 
@@ -280,11 +279,10 @@ WCHAR *WCMD_fgets(WCHAR *buf, DWORD noChars, HANDLE h)
       SetFilePointerEx(h, filepos, NULL, FILE_BEGIN);
 
       i = MultiByteToWideChar(cp, 0, bufA, p - bufA, buf, noChars);
-      heap_free(bufA);
+      free(bufA);
   }
   else {
-      status = WCMD_ReadFile(h, buf, noChars, &charsRead);
-      if (!status || charsRead == 0) return NULL;
+      if (!charsRead) return NULL;
 
       /* Find first EOL */
       for (i = 0; i < charsRead; i++) {
@@ -302,60 +300,8 @@ WCHAR *WCMD_fgets(WCHAR *buf, DWORD noChars, HANDLE h)
   return buf;
 }
 
-/* WCMD_splitpath - copied from winefile as no obvious way to use it otherwise */
-void WCMD_splitpath(const WCHAR* path, WCHAR* drv, WCHAR* dir, WCHAR* name, WCHAR* ext)
-{
-        const WCHAR* end; /* end of processed string */
-	const WCHAR* p;	 /* search pointer */
-	const WCHAR* s;	 /* copy pointer */
-
-	/* extract drive name */
-	if (path[0] && path[1]==':') {
-		if (drv) {
-			*drv++ = *path++;
-			*drv++ = *path++;
-			*drv = '\0';
-		}
-	} else if (drv)
-		*drv = '\0';
-
-        end = path + lstrlenW(path);
-
-	/* search for begin of file extension */
-	for(p=end; p>path && *--p!='\\' && *p!='/'; )
-		if (*p == '.') {
-			end = p;
-			break;
-		}
-
-	if (ext)
-		for(s=end; (*ext=*s++); )
-			ext++;
-
-	/* search for end of directory name */
-	for(p=end; p>path; )
-		if (*--p=='\\' || *p=='/') {
-			p++;
-			break;
-		}
-
-	if (name) {
-		for(s=p; s<end; )
-			*name++ = *s++;
-
-		*name = '\0';
-	}
-
-	if (dir) {
-		for(s=path; s<p; )
-			*dir++ = *s++;
-
-		*dir = '\0';
-	}
-}
-
 /****************************************************************************
- * WCMD_HandleTildaModifiers
+ * WCMD_HandleTildeModifiers
  *
  * Handle the ~ modifiers when expanding %0-9 or (%a-z/A-Z in for command)
  *    %~xxxxxV  (V=0-9 or A-Z, a-z)
@@ -386,7 +332,7 @@ void WCMD_splitpath(const WCHAR* path, WCHAR* drv, WCHAR* dir, WCHAR* name, WCHA
  *  Hence search forwards until find an invalid modifier, and then
  *  backwards until find for variable or 0-9
  */
-void WCMD_HandleTildaModifiers(WCHAR **start, BOOL atExecute)
+void WCMD_HandleTildeModifiers(WCHAR **start, BOOL atExecute)
 {
 
 #define NUMMODIFIERS 11
@@ -508,11 +454,11 @@ void WCMD_HandleTildaModifiers(WCHAR **start, BOOL atExecute)
 
     size = GetEnvironmentVariableW(env, NULL, 0);
     if (size > 0) {
-      WCHAR *fullpath = heap_xalloc(size * sizeof(WCHAR));
+      WCHAR *fullpath = malloc(size * sizeof(WCHAR));
       if (!fullpath || (GetEnvironmentVariableW(env, fullpath, size) == 0) ||
           (SearchPathW(fullpath, outputparam, NULL, MAX_PATH, outputparam, NULL) == 0))
           size = 0;
-      heap_free(fullpath);
+      free(fullpath);
     }
 
     if (!size) {
@@ -526,7 +472,7 @@ void WCMD_HandleTildaModifiers(WCHAR **start, BOOL atExecute)
   /* After this, we need full information on the file,
     which is valid not to exist.  */
   if (!skipFileParsing) {
-    if (GetFullPathNameW(outputparam, MAX_PATH, fullfilename, &filepart) == 0) {
+    if (!WCMD_get_fullpath(outputparam, MAX_PATH, fullfilename, &filepart)) {
       exists = FALSE;
       fullfilename[0] = 0x00;
     } else {
@@ -537,11 +483,10 @@ void WCMD_HandleTildaModifiers(WCHAR **start, BOOL atExecute)
     /* 2. Handle 'a' : Output attributes (File doesn't have to exist) */
     if (wmemchr(firstModifier, 'a', modifierLen) != NULL) {
 
-      WCHAR defaults[] = {'-','-','-','-','-','-','-','-','-','\0'};
       doneModifier = TRUE;
 
       if (exists) {
-        lstrcpyW(thisoutput, defaults);
+        lstrcpyW(thisoutput, L"---------");
         if (fileInfo.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
           thisoutput[0]='d';
         if (fileInfo.dwFileAttributes & FILE_ATTRIBUTE_READONLY)
@@ -570,13 +515,13 @@ void WCMD_HandleTildaModifiers(WCHAR **start, BOOL atExecute)
       doneModifier = TRUE;
 
       if (exists) {
-        if (finaloutput[0] != 0x00) lstrcatW(finaloutput, spaceW);
+        if (finaloutput[0] != 0x00) lstrcatW(finaloutput, L" ");
 
         /* Format the time */
         FileTimeToSystemTime(&fileInfo.ftLastWriteTime, &systime);
         GetDateFormatW(LOCALE_USER_DEFAULT, DATE_SHORTDATE, &systime,
                           NULL, thisoutput, MAX_PATH);
-        lstrcatW(thisoutput, spaceW);
+        lstrcatW(thisoutput, L" ");
         datelen = lstrlenW(thisoutput);
         GetTimeFormatW(LOCALE_USER_DEFAULT, TIME_NOSECONDS, &systime,
                           NULL, (thisoutput+datelen), MAX_PATH-datelen);
@@ -589,19 +534,18 @@ void WCMD_HandleTildaModifiers(WCHAR **start, BOOL atExecute)
       /* FIXME: Output full 64 bit size (sprintf does not support I64 here) */
       ULONG/*64*/ fullsize = /*(fileInfo.nFileSizeHigh << 32) +*/
                                   fileInfo.nFileSizeLow;
-      static const WCHAR fmt[] = {'%','u','\0'};
 
       doneModifier = TRUE;
       if (exists) {
-        if (finaloutput[0] != 0x00) lstrcatW(finaloutput, spaceW);
-        wsprintfW(thisoutput, fmt, fullsize);
+        if (finaloutput[0] != 0x00) lstrcatW(finaloutput, L" ");
+        wsprintfW(thisoutput, L"%u", fullsize);
         lstrcatW(finaloutput, thisoutput);
       }
     }
 
     /* 4. Handle 's' : Use short paths (File doesn't have to exist) */
     if (wmemchr(firstModifier, 's', modifierLen) != NULL) {
-      if (finaloutput[0] != 0x00) lstrcatW(finaloutput, spaceW);
+      if (finaloutput[0] != 0x00) lstrcatW(finaloutput, L" ");
 
       /* Convert fullfilename's path to a short path - Save filename away as
          only path is valid, name may not exist which causes GetShortPathName
@@ -618,7 +562,7 @@ void WCMD_HandleTildaModifiers(WCHAR **start, BOOL atExecute)
     /*      Note this overrides d,p,n,x                                 */
     if (wmemchr(firstModifier, 'f', modifierLen) != NULL) {
       doneModifier = TRUE;
-      if (finaloutput[0] != 0x00) lstrcatW(finaloutput, spaceW);
+      if (finaloutput[0] != 0x00) lstrcatW(finaloutput, L" ");
       lstrcatW(finaloutput, fullfilename);
     } else {
 
@@ -630,12 +574,12 @@ void WCMD_HandleTildaModifiers(WCHAR **start, BOOL atExecute)
       BOOL addSpace = (finaloutput[0] != 0x00);
 
       /* Split into components */
-      WCMD_splitpath(fullfilename, drive, dir, fname, ext);
+      _wsplitpath(fullfilename, drive, dir, fname, ext);
 
       /* 5. Handle 'd' : Drive Letter */
       if (wmemchr(firstModifier, 'd', modifierLen) != NULL) {
         if (addSpace) {
-          lstrcatW(finaloutput, spaceW);
+          lstrcatW(finaloutput, L" ");
           addSpace = FALSE;
         }
 
@@ -647,7 +591,7 @@ void WCMD_HandleTildaModifiers(WCHAR **start, BOOL atExecute)
       /* 6. Handle 'p' : Path */
       if (wmemchr(firstModifier, 'p', modifierLen) != NULL) {
         if (addSpace) {
-          lstrcatW(finaloutput, spaceW);
+          lstrcatW(finaloutput, L" ");
           addSpace = FALSE;
         }
 
@@ -659,7 +603,7 @@ void WCMD_HandleTildaModifiers(WCHAR **start, BOOL atExecute)
       /* 7. Handle 'n' : Name */
       if (wmemchr(firstModifier, 'n', modifierLen) != NULL) {
         if (addSpace) {
-          lstrcatW(finaloutput, spaceW);
+          lstrcatW(finaloutput, L" ");
           addSpace = FALSE;
         }
 
@@ -671,7 +615,7 @@ void WCMD_HandleTildaModifiers(WCHAR **start, BOOL atExecute)
       /* 8. Handle 'x' : Ext */
       if (wmemchr(firstModifier, 'x', modifierLen) != NULL) {
         if (addSpace) {
-          lstrcatW(finaloutput, spaceW);
+          lstrcatW(finaloutput, L" ");
           addSpace = FALSE;
         }
 
@@ -684,7 +628,7 @@ void WCMD_HandleTildaModifiers(WCHAR **start, BOOL atExecute)
       if (!doneFileModifier &&
           wmemchr(firstModifier, 's', modifierLen) != NULL) {
         doneModifier = TRUE;
-        if (finaloutput[0] != 0x00) lstrcatW(finaloutput, spaceW);
+        if (finaloutput[0] != 0x00) lstrcatW(finaloutput, L" ");
         lstrcatW(finaloutput, fullfilename);
       }
     }
