@@ -20,9 +20,8 @@
 
 #include <assert.h>
 
-#define NONAMELESSUNION
-#define _WIN32_IE 0x500
 #include <windows.h>
+#include <ntuser.h>
 #include <commctrl.h>
 #include <shellapi.h>
 
@@ -34,6 +33,9 @@
 
 WINE_DEFAULT_DEBUG_CHANNEL(systray);
 
+#define TRAY_MINIMIZE_ALL 419
+#define TRAY_MINIMIZE_ALL_UNDO 416
+
 struct notify_data  /* platform-independent format for NOTIFYICONDATA */
 {
     LONG  hWnd;
@@ -58,7 +60,8 @@ struct notify_data  /* platform-independent format for NOTIFYICONDATA */
     UINT bpp;
 };
 
-static int (CDECL *wine_notify_icon)(DWORD,NOTIFYICONDATAW *);
+#define ICON_DISPLAY_HIDDEN -1
+#define ICON_DISPLAY_DOCKED -2
 
 /* an individual systray icon, unpacked from the NOTIFYICONDATA and always in unicode */
 struct icon
@@ -66,6 +69,8 @@ struct icon
     struct list    entry;
     HICON          image;    /* the image to render */
     HWND           owner;    /* the HWND passed in to the Shell_NotifyIcon call */
+    HWND           window;   /* the adaptor window */
+    BOOL           layered;  /* whether we are using a layered window */
     HWND           tooltip;  /* Icon tooltip */
     UINT           state;    /* state flags */
     UINT           id;       /* the unique id given by the app */
@@ -95,17 +100,16 @@ static struct list taskbar_buttons = LIST_INIT( taskbar_buttons );
 
 static HWND tray_window;
 
-static unsigned int alloc_displayed;
 static unsigned int nb_displayed;
-static struct icon **displayed;  /* array of currently displayed icons */
 
-static BOOL hide_systray, enable_shell;
+static BOOL show_systray = TRUE, enable_shell, enable_taskbar;
 static int icon_cx, icon_cy, tray_width, tray_height;
 static int start_button_width, taskbar_button_width;
 static WCHAR start_label[50];
 
 static struct icon *balloon_icon;
 static HWND balloon_window;
+static POINT balloon_pos;
 
 #define MIN_DISPLAYED 8
 #define ICON_BORDER  2
@@ -119,6 +123,29 @@ static HWND balloon_window;
 
 #define WM_POPUPSYSTEMMENU  0x0313
 
+static LRESULT WINAPI shell_traywnd_proc( HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam );
+static LRESULT WINAPI tray_icon_wndproc( HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam );
+
+static BOOL show_icon( struct icon *icon );
+static BOOL hide_icon( struct icon *icon );
+static BOOL delete_icon( struct icon *icon );
+
+static WNDCLASSEXW shell_traywnd_class =
+{
+    .cbSize = sizeof(WNDCLASSEXW),
+    .style = CS_DBLCLKS | CS_HREDRAW,
+    .lpfnWndProc = shell_traywnd_proc,
+    .hbrBackground = (HBRUSH)COLOR_WINDOW,
+    .lpszClassName = L"Shell_TrayWnd",
+};
+static WNDCLASSEXW tray_icon_class =
+{
+    .cbSize = sizeof(WNDCLASSEXW),
+    .style = CS_HREDRAW | CS_VREDRAW | CS_DBLCLKS,
+    .lpfnWndProc = tray_icon_wndproc,
+    .lpszClassName = L"__wine_tray_icon",
+};
+
 static void do_hide_systray(void);
 static void do_show_systray(void);
 
@@ -134,17 +161,6 @@ static struct icon *get_icon(HWND owner, UINT id)
     return NULL;
 }
 
-static RECT get_icon_rect( struct icon *icon )
-{
-    RECT rect;
-
-    rect.right = tray_width - icon_cx * icon->display;
-    rect.left = rect.right - icon_cx;
-    rect.top = (tray_height - icon_cy) / 2;
-    rect.bottom = rect.top + icon_cy;
-    return rect;
-}
-
 static void init_common_controls(void)
 {
     static BOOL initialized = FALSE;
@@ -167,32 +183,45 @@ static void create_tooltip(struct icon *icon)
     TTTOOLINFOW ti;
 
     init_common_controls();
-    icon->tooltip = CreateWindowExW(WS_EX_TOPMOST, TOOLTIPS_CLASSW, NULL,
-                                   WS_POPUP | TTS_ALWAYSTIP,
-                                   CW_USEDEFAULT, CW_USEDEFAULT,
-                                   CW_USEDEFAULT, CW_USEDEFAULT,
-                                   tray_window, NULL, NULL, NULL);
+    icon->tooltip = CreateWindowExW( WS_EX_TOPMOST, TOOLTIPS_CLASSW, NULL, WS_POPUP | TTS_ALWAYSTIP,
+                                     CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
+                                     icon->window, NULL, NULL, NULL );
 
     ZeroMemory(&ti, sizeof(ti));
     ti.cbSize = sizeof(TTTOOLINFOW);
-    ti.hwnd = tray_window;
+    ti.uFlags = TTF_SUBCLASS | TTF_IDISHWND;
+    ti.hwnd = icon->window;
+    ti.uId = (UINT_PTR)icon->window;
     ti.lpszText = icon->tiptext;
-    if (icon->display != -1) ti.rect = get_icon_rect( icon );
     SendMessageW(icon->tooltip, TTM_ADDTOOLW, 0, (LPARAM)&ti);
 }
 
 static void set_balloon_position( struct icon *icon )
 {
-    RECT rect = get_icon_rect( icon );
+    RECT rect;
     POINT pos;
 
-    MapWindowPoints( tray_window, 0, (POINT *)&rect, 2 );
+    GetWindowRect( icon->window, &rect );
     pos.x = (rect.left + rect.right) / 2;
     pos.y = (rect.top + rect.bottom) / 2;
     SendMessageW( balloon_window, TTM_TRACKPOSITION, 0, MAKELONG( pos.x, pos.y ));
 }
 
-static void balloon_create_timer(void)
+static void update_systray_balloon_position(void)
+{
+    RECT rect;
+    POINT pos;
+
+    if (!balloon_icon) return;
+    GetWindowRect( balloon_icon->window, &rect );
+    pos.x = (rect.left + rect.right) / 2;
+    pos.y = (rect.top + rect.bottom) / 2;
+    if (pos.x == balloon_pos.x && pos.y == balloon_pos.y) return; /* nothing changed */
+    balloon_pos = pos;
+    SendMessageW( balloon_window, TTM_TRACKPOSITION, 0, MAKELONG( pos.x, pos.y ));
+}
+
+static void balloon_create_timer( struct icon *icon )
 {
     TTTOOLINFOW ti;
 
@@ -200,45 +229,47 @@ static void balloon_create_timer(void)
     balloon_window = CreateWindowExW( WS_EX_TOPMOST, TOOLTIPS_CLASSW, NULL,
                                       WS_POPUP | TTS_ALWAYSTIP | TTS_NOPREFIX | TTS_BALLOON | TTS_CLOSE,
                                       CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT, CW_USEDEFAULT,
-                                      tray_window, NULL, NULL, NULL);
+                                      icon->window, NULL, NULL, NULL );
 
     memset( &ti, 0, sizeof(ti) );
     ti.cbSize = sizeof(TTTOOLINFOW);
-    ti.hwnd = tray_window;
-    ti.uFlags = TTF_TRACK;
-    ti.lpszText = balloon_icon->info_text;
+    ti.hwnd = icon->window;
+    ti.uId = (UINT_PTR)icon->window;
+    ti.uFlags = TTF_TRACK | TTF_IDISHWND;
+    ti.lpszText = icon->info_text;
     SendMessageW( balloon_window, TTM_ADDTOOLW, 0, (LPARAM)&ti );
-    if ((balloon_icon->info_flags & NIIF_ICONMASK) == NIIF_USER)
-        SendMessageW( balloon_window, TTM_SETTITLEW, (WPARAM)balloon_icon->info_icon,
-                      (LPARAM)balloon_icon->info_title );
+    if ((icon->info_flags & NIIF_ICONMASK) == NIIF_USER)
+        SendMessageW( balloon_window, TTM_SETTITLEW, (WPARAM)icon->info_icon, (LPARAM)icon->info_title );
     else
-        SendMessageW( balloon_window, TTM_SETTITLEW, balloon_icon->info_flags,
-                      (LPARAM)balloon_icon->info_title );
-    set_balloon_position( balloon_icon );
+        SendMessageW( balloon_window, TTM_SETTITLEW, icon->info_flags, (LPARAM)icon->info_title );
+    balloon_icon = icon;
+    balloon_pos.x = balloon_pos.y = MAXLONG;
+    update_systray_balloon_position();
     SendMessageW( balloon_window, TTM_TRACKACTIVATE, TRUE, (LPARAM)&ti );
-    KillTimer( tray_window, BALLOON_CREATE_TIMER );
-    SetTimer( tray_window, BALLOON_SHOW_TIMER, balloon_icon->info_timeout, NULL );
+    KillTimer( icon->window, BALLOON_CREATE_TIMER );
+    SetTimer( icon->window, BALLOON_SHOW_TIMER, icon->info_timeout, NULL );
 }
 
 static BOOL show_balloon( struct icon *icon )
 {
-    if (icon->display == -1) return FALSE;  /* not displayed */
+    if (!show_systray) return FALSE;  /* systray has been hidden */
+    if (icon->display == ICON_DISPLAY_HIDDEN) return FALSE;  /* not displayed */
     if (!icon->info_text[0]) return FALSE;  /* no balloon */
     balloon_icon = icon;
-    SetTimer( tray_window, BALLOON_CREATE_TIMER, BALLOON_CREATE_TIMEOUT, NULL );
+    SetTimer( icon->window, BALLOON_CREATE_TIMER, BALLOON_CREATE_TIMEOUT, NULL );
     return TRUE;
 }
 
-static void hide_balloon(void)
+static void hide_balloon( struct icon *icon )
 {
     if (!balloon_icon) return;
     if (balloon_window)
     {
-        KillTimer( tray_window, BALLOON_SHOW_TIMER );
+        KillTimer( balloon_icon->window, BALLOON_SHOW_TIMER );
         DestroyWindow( balloon_window );
         balloon_window = 0;
     }
-    else KillTimer( tray_window, BALLOON_CREATE_TIMER );
+    else KillTimer( balloon_icon->window, BALLOON_CREATE_TIMER );
     balloon_icon = NULL;
 }
 
@@ -254,20 +285,19 @@ static void update_balloon( struct icon *icon )
 {
     if (balloon_icon == icon)
     {
-        hide_balloon();
+        hide_balloon( icon );
         show_balloon( icon );
     }
     else if (!balloon_icon)
     {
-        if (!show_balloon( icon )) return;
+        show_balloon( icon );
     }
-    if (!balloon_icon) show_next_balloon();
 }
 
-static void balloon_timer(void)
+static void balloon_timer( struct icon *icon )
 {
-    if (balloon_icon) balloon_icon->info_text[0] = 0;  /* clear text now that balloon has been shown */
-    hide_balloon();
+    icon->info_text[0] = 0;  /* clear text now that balloon has been shown */
+    hide_balloon( icon );
     show_next_balloon();
 }
 
@@ -278,12 +308,49 @@ static void update_tooltip_text(struct icon *icon)
 
     ZeroMemory(&ti, sizeof(ti));
     ti.cbSize = sizeof(TTTOOLINFOW);
-    ti.hwnd = tray_window;
+    ti.uFlags = TTF_SUBCLASS | TTF_IDISHWND;
+    ti.hwnd = icon->window;
+    ti.uId = (UINT_PTR)icon->window;
     ti.lpszText = icon->tiptext;
 
     SendMessageW(icon->tooltip, TTM_UPDATETIPTEXTW, 0, (LPARAM)&ti);
 }
 
+/* get the position of an icon in the stand-alone tray */
+static POINT get_icon_pos( struct icon *icon )
+{
+    POINT pos;
+
+    if (enable_taskbar)
+    {
+        pos.x = tray_width - icon_cx * (icon->display + 1);
+        pos.y = (tray_height - icon_cy) / 2;
+    }
+    else
+    {
+        pos.x = icon_cx * icon->display;
+        pos.y = 0;
+    }
+
+    return pos;
+}
+
+/* get the size of the stand-alone tray window */
+static SIZE get_window_size(void)
+{
+    SIZE size;
+    RECT rect;
+
+    rect.left = 0;
+    rect.top = 0;
+    rect.right = icon_cx * max( nb_displayed, MIN_DISPLAYED );
+    rect.bottom = icon_cy;
+    AdjustWindowRect( &rect, WS_CAPTION, FALSE );
+    size.cx = rect.right - rect.left;
+    size.cy = rect.bottom - rect.top;
+    return size;
+}
+
 /* synchronize tooltip position with tooltip window */
 static void update_tooltip_position( struct icon *icon )
 {
@@ -291,59 +358,268 @@ static void update_tooltip_position( struct icon *icon )
 
     ZeroMemory(&ti, sizeof(ti));
     ti.cbSize = sizeof(TTTOOLINFOW);
-    ti.hwnd = tray_window;
-    if (icon->display != -1) ti.rect = get_icon_rect( icon );
+    ti.uFlags = TTF_SUBCLASS | TTF_IDISHWND;
+    ti.hwnd = icon->window;
+    ti.uId = (UINT_PTR)icon->window;
+    ti.lpszText = icon->tiptext;
     SendMessageW( icon->tooltip, TTM_NEWTOOLRECTW, 0, (LPARAM)&ti );
     if (balloon_icon == icon) set_balloon_position( icon );
 }
 
-/* find the icon located at a certain point in the tray window */
-static struct icon *icon_from_point( int x, int y )
+static void paint_layered_icon( struct icon *icon )
 {
-    if (y < 0 || y >= icon_cy) return NULL;
-    x = tray_width - x;
-    if (x < 0 || x >= icon_cx * nb_displayed) return NULL;
-    return displayed[x / icon_cx];
+    BLENDFUNCTION blend = {AC_SRC_OVER, 0, 255, AC_SRC_ALPHA};
+    int width = GetSystemMetrics( SM_CXSMICON );
+    int height = GetSystemMetrics( SM_CYSMICON );
+    BITMAPINFO *info;
+    HBITMAP dib, mask;
+    HDC hdc;
+    RECT rc;
+    SIZE size;
+    POINT pos;
+    int i, x, y;
+    void *color_bits, *mask_bits;
+    DWORD *ptr;
+    BOOL has_alpha = FALSE;
+
+    GetWindowRect( icon->window, &rc );
+    size.cx = rc.right - rc.left;
+    size.cy = rc.bottom - rc.top;
+    pos.x = (size.cx - width) / 2;
+    pos.y = (size.cy - height) / 2;
+
+    if (!(info = calloc( 1, FIELD_OFFSET( BITMAPINFO, bmiColors[2] ) ))) return;
+    info->bmiHeader.biSize = sizeof(info->bmiHeader);
+    info->bmiHeader.biWidth = size.cx;
+    info->bmiHeader.biHeight = size.cy;
+    info->bmiHeader.biBitCount = 32;
+    info->bmiHeader.biPlanes = 1;
+    info->bmiHeader.biCompression = BI_RGB;
+
+    hdc = CreateCompatibleDC( 0 );
+    if (!(dib = CreateDIBSection( 0, info, DIB_RGB_COLORS, &color_bits, NULL, 0 ))) goto done;
+    SelectObject( hdc, dib );
+    DrawIconEx( hdc, pos.x, pos.y, icon->image, width, height, 0, 0, DI_DEFAULTSIZE | DI_NORMAL );
+
+    /* check if the icon was drawn with an alpha channel */
+    for (i = 0, ptr = color_bits; i < size.cx * size.cy; i++)
+        if ((has_alpha = (ptr[i] & 0xff000000) != 0)) break;
+
+    if (!has_alpha)
+    {
+        unsigned int width_bytes = (size.cx + 31) / 32 * 4;
+
+        info->bmiHeader.biBitCount = 1;
+        info->bmiColors[0].rgbRed = 0;
+        info->bmiColors[0].rgbGreen = 0;
+        info->bmiColors[0].rgbBlue = 0;
+        info->bmiColors[0].rgbReserved = 0;
+        info->bmiColors[1].rgbRed = 0xff;
+        info->bmiColors[1].rgbGreen = 0xff;
+        info->bmiColors[1].rgbBlue = 0xff;
+        info->bmiColors[1].rgbReserved = 0;
+
+        if (!(mask = CreateDIBSection( 0, info, DIB_RGB_COLORS, &mask_bits, NULL, 0 ))) goto done;
+        memset( mask_bits, 0xff, width_bytes * size.cy );
+        SelectObject( hdc, mask );
+        DrawIconEx( hdc, pos.x, pos.y, icon->image, width, height, 0, 0, DI_DEFAULTSIZE | DI_MASK );
+
+        for (y = 0, ptr = color_bits; y < size.cy; y++)
+            for (x = 0; x < size.cx; x++, ptr++)
+                if (!((((BYTE *)mask_bits)[y * width_bytes + x / 8] << (x % 8)) & 0x80))
+                    *ptr |= 0xff000000;
+
+        SelectObject( hdc, dib );
+        DeleteObject( mask );
+    }
+
+    UpdateLayeredWindow( icon->window, 0, NULL, NULL, hdc, NULL, 0, &blend, ULW_ALPHA );
+done:
+    free( info );
+    if (hdc) DeleteDC( hdc );
+    if (dib) DeleteObject( dib );
 }
 
-/* invalidate the portion of the tray window that contains the specified icons */
-static void invalidate_icons( unsigned int start, unsigned int end )
+static BOOL notify_owner( struct icon *icon, UINT msg, LPARAM lparam )
 {
-    RECT rect;
+    WPARAM wp = icon->id;
+    LPARAM lp = msg;
 
-    rect.left = tray_width - (end + 1) * icon_cx;
-    rect.top  = (tray_height - icon_cy) / 2;
-    rect.right = tray_width - start * icon_cx;
-    rect.bottom = rect.top + icon_cy;
-    InvalidateRect( tray_window, &rect, TRUE );
+    if (icon->version >= NOTIFYICON_VERSION_4)
+    {
+        POINT pt = {.x = (short)LOWORD(lparam), .y = (short)HIWORD(lparam)};
+        ClientToScreen( icon->window, &pt );
+        wp = MAKEWPARAM( pt.x, pt.y );
+        lp = MAKELPARAM( msg, icon->id );
+    }
+
+    TRACE( "relaying 0x%x\n", msg );
+    if (!SendNotifyMessageW( icon->owner, icon->callback_message, wp, lp ) &&
+        (GetLastError() == ERROR_INVALID_WINDOW_HANDLE))
+    {
+        WARN( "application window was destroyed, removing icon %u\n", icon->id );
+        delete_icon( icon );
+        return FALSE;
+    }
+    return TRUE;
+}
+
+/* window procedure for the individual tray icon window */
+static LRESULT WINAPI tray_icon_wndproc( HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam )
+{
+    struct icon *icon = (struct icon *)GetWindowLongPtrW( hwnd, GWLP_USERDATA );
+
+    TRACE( "hwnd %p, msg %#x, wparam %#Ix, lparam %#Ix\n", hwnd, msg, wparam, lparam );
+
+    switch (msg)
+    {
+    case WM_NCCREATE:
+    {
+        /* set the icon data for the window from the data passed into CreateWindow */
+        const CREATESTRUCTW *info = (const CREATESTRUCTW *)lparam;
+        icon = info->lpCreateParams;
+        SetWindowLongPtrW( hwnd, GWLP_USERDATA, (LONG_PTR)icon );
+        break;
+    }
+
+    case WM_CLOSE:
+        if (icon->display == ICON_DISPLAY_DOCKED)
+        {
+            TRACE( "icon %u no longer embedded\n", icon->id );
+            hide_icon( icon );
+            show_icon( icon );
+        }
+        return 0;
+
+    case WM_CREATE:
+        icon->window = hwnd;
+        create_tooltip( icon );
+        break;
+
+    case WM_SIZE:
+    case WM_MOVE:
+        if (icon->layered) paint_layered_icon( icon );
+        break;
+
+    case WM_PAINT:
+    {
+        PAINTSTRUCT ps;
+        RECT rc;
+        HDC hdc;
+        int cx, cy;
+
+        if (icon->layered) break;
+
+        cx = GetSystemMetrics( SM_CXSMICON );
+        cy = GetSystemMetrics( SM_CYSMICON );
+        hdc = BeginPaint( hwnd, &ps );
+        GetClientRect( hwnd, &rc );
+        TRACE( "painting rect %s\n", wine_dbgstr_rect( &rc ) );
+        DrawIconEx( hdc, (rc.left + rc.right - cx) / 2, (rc.top + rc.bottom - cy) / 2,
+                    icon->image, cx, cy, 0, 0, DI_DEFAULTSIZE | DI_NORMAL );
+        EndPaint( hwnd, &ps );
+        return 0;
+    }
+
+    case WM_MOUSEMOVE:
+    case WM_LBUTTONDOWN:
+    case WM_LBUTTONUP:
+    case WM_RBUTTONDOWN:
+    case WM_RBUTTONUP:
+    case WM_MBUTTONDOWN:
+    case WM_MBUTTONUP:
+    case WM_LBUTTONDBLCLK:
+    case WM_RBUTTONDBLCLK:
+    case WM_MBUTTONDBLCLK:
+    {
+        MSG message = {.hwnd = hwnd, .message = msg, .wParam = wparam, .lParam = lparam};
+        SendMessageW( icon->tooltip, TTM_RELAYEVENT, 0, (LPARAM)&message );
+        if (!notify_owner( icon, msg, lparam )) break;
+        if (icon->version > 0)
+        {
+            if (msg == WM_LBUTTONUP) notify_owner( icon, NIN_SELECT, lparam );
+            if (msg == WM_RBUTTONUP) notify_owner( icon, WM_CONTEXTMENU, lparam );
+        }
+        break;
+    }
+
+    case WM_WINDOWPOSCHANGED:
+        update_systray_balloon_position();
+        break;
+
+    case WM_TIMER:
+        switch (wparam)
+        {
+        case BALLOON_CREATE_TIMER: balloon_create_timer( icon ); break;
+        case BALLOON_SHOW_TIMER: balloon_timer( icon ); break;
+        }
+        return 0;
+    }
+
+    return DefWindowProcW( hwnd, msg, wparam, lparam );
+}
+
+/* add an icon to the system tray window */
+static void systray_add_icon( struct icon *icon )
+{
+    POINT pos;
+
+    if (icon->display != ICON_DISPLAY_HIDDEN) return;  /* already added */
+
+    icon->display = nb_displayed++;
+    SetWindowLongW( icon->window, GWL_STYLE, GetWindowLongW( icon->window, GWL_STYLE ) | WS_CHILD );
+    SetParent( icon->window, tray_window );
+    pos = get_icon_pos( icon );
+    SetWindowPos( icon->window, 0, pos.x, pos.y, 0, 0, SWP_NOSIZE | SWP_NOACTIVATE | SWP_NOZORDER | SWP_SHOWWINDOW );
+
+    if (nb_displayed == 1 && show_systray) do_show_systray();
+    TRACE( "added %u now %d icons\n", icon->id, nb_displayed );
+}
+
+/* remove an icon from the stand-alone tray */
+static void systray_remove_icon( struct icon *icon )
+{
+    struct icon *ptr;
+    POINT pos;
+
+    if (icon->display == ICON_DISPLAY_HIDDEN) return;  /* already removed */
+
+    assert( nb_displayed );
+    LIST_FOR_EACH_ENTRY( ptr, &icon_list, struct icon, entry )
+    {
+        if (ptr == icon) continue;
+        if (ptr->display < icon->display) continue;
+        ptr->display--;
+        update_tooltip_position( ptr );
+        pos = get_icon_pos( ptr );
+        SetWindowPos( ptr->window, 0, pos.x, pos.y, 0, 0, SWP_NOSIZE | SWP_NOACTIVATE | SWP_NOZORDER );
+    }
+
+    if (!--nb_displayed && !enable_shell) do_hide_systray();
+    TRACE( "removed %u now %d icons\n", icon->id, nb_displayed );
+
+    icon->display = ICON_DISPLAY_HIDDEN;
+    SetParent( icon->window, GetDesktopWindow() );
+    SetWindowLongW( icon->window, GWL_STYLE, GetWindowLongW( icon->window, GWL_STYLE ) & ~WS_CHILD );
 }
 
 /* make an icon visible */
 static BOOL show_icon(struct icon *icon)
 {
-    WINE_TRACE("id=0x%x, hwnd=%p\n", icon->id, icon->owner);
+    TRACE( "id=0x%x, hwnd=%p\n", icon->id, icon->owner );
 
-    if (icon->display != -1) return TRUE;  /* already displayed */
+    if (icon->display != ICON_DISPLAY_HIDDEN) return TRUE;  /* already displayed */
 
-    if (nb_displayed >= alloc_displayed)
+    if (!enable_taskbar && NtUserMessageCall( icon->window, WINE_SYSTRAY_DOCK_INSERT, icon_cx, icon_cy,
+                                              icon, NtUserSystemTrayCall, FALSE ))
     {
-        unsigned int new_count = max( alloc_displayed * 2, 32 );
-        struct icon **ptr;
-        if (displayed) ptr = HeapReAlloc( GetProcessHeap(), 0, displayed, new_count * sizeof(*ptr) );
-        else ptr = HeapAlloc( GetProcessHeap(), 0, new_count * sizeof(*ptr) );
-        if (!ptr) return FALSE;
-        displayed = ptr;
-        alloc_displayed = new_count;
+        icon->display = ICON_DISPLAY_DOCKED;
+        icon->layered = TRUE;
+        SendMessageW( icon->window, WM_SIZE, SIZE_RESTORED, MAKELONG( icon_cx, icon_cy ) );
     }
+    systray_add_icon( icon );
 
-    icon->display = nb_displayed;
-    displayed[nb_displayed++] = icon;
     update_tooltip_position( icon );
-    invalidate_icons( nb_displayed-1, nb_displayed-1 );
-
-    if (nb_displayed == 1 && !hide_systray) do_show_systray();
-
-    create_tooltip(icon);
     update_balloon( icon );
     return TRUE;
 }
@@ -351,24 +627,18 @@ static BOOL show_icon(struct icon *icon)
 /* make an icon invisible */
 static BOOL hide_icon(struct icon *icon)
 {
-    unsigned int i;
+    TRACE( "id=0x%x, hwnd=%p\n", icon->id, icon->owner );
 
-    WINE_TRACE("id=0x%x, hwnd=%p\n", icon->id, icon->owner);
+    if (icon->display == ICON_DISPLAY_HIDDEN) return TRUE;  /* already hidden */
 
-    if (icon->display == -1) return TRUE;  /* already hidden */
-
-    assert( nb_displayed );
-    for (i = icon->display; i < nb_displayed - 1; i++)
+    if (!enable_taskbar && NtUserMessageCall( icon->window, WINE_SYSTRAY_DOCK_REMOVE, 0, 0,
+                                              NULL, NtUserSystemTrayCall, FALSE ))
     {
-        displayed[i] = displayed[i + 1];
-        displayed[i]->display = i;
-        update_tooltip_position( displayed[i] );
+        icon->display = ICON_DISPLAY_HIDDEN;
+        icon->layered = FALSE;
     }
-    nb_displayed--;
-    invalidate_icons( icon->display, nb_displayed );
-    icon->display = -1;
-
-    if (!nb_displayed && !enable_shell) do_hide_systray();
+    ShowWindow( icon->window, SW_HIDE );
+    systray_remove_icon( icon );
 
     update_balloon( icon );
     update_tooltip_position( icon );
@@ -378,12 +648,12 @@ static BOOL hide_icon(struct icon *icon)
 /* Modifies an existing icon record */
 static BOOL modify_icon( struct icon *icon, NOTIFYICONDATAW *nid )
 {
-    WINE_TRACE("id=0x%x, hwnd=%p\n", nid->uID, nid->hWnd);
+    TRACE( "id=0x%x, hwnd=%p\n", nid->uID, nid->hWnd );
 
     /* demarshal the request from the NID */
     if (!icon)
     {
-        WINE_WARN("Invalid icon ID (0x%x) for HWND %p\n", nid->uID, nid->hWnd);
+        WARN( "Invalid icon ID (0x%x) for HWND %p\n", nid->uID, nid->hWnd );
         return FALSE;
     }
 
@@ -396,7 +666,14 @@ static BOOL modify_icon( struct icon *icon, NOTIFYICONDATAW *nid )
     {
         if (icon->image) DestroyIcon(icon->image);
         icon->image = CopyIcon(nid->hIcon);
-        if (icon->display != -1) invalidate_icons( icon->display, icon->display );
+
+        if (icon->display >= 0)
+            InvalidateRect( icon->window, NULL, TRUE );
+        else if (icon->layered)
+            paint_layered_icon( icon );
+        else if (!enable_taskbar)
+            NtUserMessageCall( icon->window, WINE_SYSTRAY_DOCK_CLEAR, 0, 0,
+                               NULL, NtUserSystemTrayCall, FALSE );
     }
 
     if (nid->uFlags & NIF_MESSAGE)
@@ -406,14 +683,14 @@ static BOOL modify_icon( struct icon *icon, NOTIFYICONDATAW *nid )
     if (nid->uFlags & NIF_TIP)
     {
         lstrcpynW( icon->tiptext, nid->szTip, ARRAY_SIZE( icon->tiptext ));
-        if (icon->display != -1) update_tooltip_text(icon);
+        update_tooltip_text( icon );
     }
     if (nid->uFlags & NIF_INFO && nid->cbSize >= NOTIFYICONDATAA_V2_SIZE)
     {
         lstrcpynW( icon->info_text, nid->szInfo, ARRAY_SIZE( icon->info_text ));
         lstrcpynW( icon->info_title, nid->szInfoTitle, ARRAY_SIZE( icon->info_title ));
         icon->info_flags = nid->dwInfoFlags;
-        icon->info_timeout = max(min(nid->u.uTimeout, BALLOON_SHOW_MAX_TIMEOUT), BALLOON_SHOW_MIN_TIMEOUT);
+        icon->info_timeout = max(min(nid->uTimeout, BALLOON_SHOW_MAX_TIMEOUT), BALLOON_SHOW_MIN_TIMEOUT);
         icon->info_icon = nid->hBalloonIcon;
         update_balloon( icon );
     }
@@ -427,24 +704,28 @@ static BOOL add_icon(NOTIFYICONDATAW *nid)
 {
     struct icon  *icon;
 
-    WINE_TRACE("id=0x%x, hwnd=%p\n", nid->uID, nid->hWnd);
+    TRACE( "id=0x%x, hwnd=%p\n", nid->uID, nid->hWnd );
 
     if ((icon = get_icon(nid->hWnd, nid->uID)))
     {
-        WINE_WARN("duplicate tray icon add, buggy app?\n");
+        WARN( "duplicate tray icon add, buggy app?\n" );
         return FALSE;
     }
 
-    if (!(icon = HeapAlloc(GetProcessHeap(), HEAP_ZERO_MEMORY, sizeof(*icon))))
+    if (!(icon = calloc( 1, sizeof(*icon) )))
     {
-        WINE_ERR("out of memory\n");
+        ERR( "out of memory\n" );
         return FALSE;
     }
 
     ZeroMemory(icon, sizeof(struct icon));
     icon->id     = nid->uID;
     icon->owner  = nid->hWnd;
-    icon->display = -1;
+    icon->display = ICON_DISPLAY_HIDDEN;
+
+    CreateWindowExW( WS_EX_LAYERED, tray_icon_class.lpszClassName, NULL, WS_CLIPSIBLINGS | WS_POPUP,
+                     0, 0, icon_cx, icon_cy, 0, NULL, NULL, icon );
+    if (!icon->window) ERR( "Failed to create systray icon window\n" );
 
     list_add_tail(&icon_list, &icon->entry);
 
@@ -452,28 +733,27 @@ static BOOL add_icon(NOTIFYICONDATAW *nid)
 }
 
 /* Deletes tray icon window and icon record */
-static BOOL delete_icon(struct icon *icon)
+static BOOL delete_icon( struct icon *icon )
 {
-    hide_icon(icon);
-    list_remove(&icon->entry);
-    DestroyIcon(icon->image);
-    HeapFree(GetProcessHeap(), 0, icon);
+    hide_icon( icon );
+    list_remove( &icon->entry );
+    DestroyWindow( icon->tooltip );
+    DestroyWindow( icon->window );
+    DestroyIcon( icon->image );
+    free( icon );
     return TRUE;
 }
 
 /* cleanup icons belonging to a window that has been destroyed */
 static void cleanup_systray_window( HWND hwnd )
 {
+    NOTIFYICONDATAW nid = {.cbSize = sizeof(nid), .hWnd = hwnd};
     struct icon *icon, *next;
 
     LIST_FOR_EACH_ENTRY_SAFE( icon, next, &icon_list, struct icon, entry )
         if (icon->owner == hwnd) delete_icon( icon );
 
-    if (wine_notify_icon)
-    {
-        NOTIFYICONDATAW nid = { sizeof(nid), hwnd };
-        wine_notify_icon( 0xdead, &nid );
-    }
+    NtUserMessageCall( hwnd, WINE_SYSTRAY_CLEANUP_ICONS, 0, 0, NULL, NtUserSystemTrayCall, FALSE );
 }
 
 /* update the taskbar buttons when something changed */
@@ -485,6 +765,7 @@ static void sync_taskbar_buttons(void)
     int right = tray_width - nb_displayed * icon_cx;
     HWND foreground = GetAncestor( GetForegroundWindow(), GA_ROOTOWNER );
 
+    if (!enable_taskbar) return;
     if (!IsWindowVisible( tray_window )) return;
 
     LIST_FOR_EACH_ENTRY( win, &taskbar_buttons, struct taskbar_button, entry )
@@ -537,7 +818,7 @@ static BOOL handle_incoming(HWND hwndSource, COPYDATASTRUCT *cds)
     nid.hIcon            = 0;
     nid.dwState          = data->dwState;
     nid.dwStateMask      = data->dwStateMask;
-    nid.u.uTimeout       = data->u.uTimeout;
+    nid.uTimeout         = data->u.uTimeout;
     nid.dwInfoFlags      = data->dwInfoFlags;
     nid.guidItem         = data->guidItem;
     lstrcpyW( nid.szTip, data->szTip );
@@ -558,7 +839,7 @@ static BOOL handle_incoming(HWND hwndSource, COPYDATASTRUCT *cds)
 
         if (cds->cbData < sizeof(*data) + cbMaskBits + cbColourBits)
         {
-            WINE_ERR("buffer underflow\n");
+            ERR( "buffer underflow\n" );
             return FALSE;
         }
         nid.hIcon = CreateIcon(NULL, data->width, data->height, data->planes, data->bpp,
@@ -568,11 +849,9 @@ static BOOL handle_incoming(HWND hwndSource, COPYDATASTRUCT *cds)
     /* try forwarding to the display driver first */
     if (cds->dwData == NIM_ADD || !(icon = get_icon( nid.hWnd, nid.uID )))
     {
-        if (wine_notify_icon && ((ret = wine_notify_icon( cds->dwData, &nid )) != -1))
-        {
-            if (nid.hIcon) DestroyIcon( nid.hIcon );
-            return ret;
-        }
+        if ((ret = NtUserMessageCall( hwndSource, WINE_SYSTRAY_NOTIFY_ICON, cds->dwData, 0,
+                                      &nid, NtUserSystemTrayCall, FALSE )) != -1)
+            goto done;
         ret = FALSE;
     }
 
@@ -590,15 +869,16 @@ static BOOL handle_incoming(HWND hwndSource, COPYDATASTRUCT *cds)
     case NIM_SETVERSION:
         if (icon)
         {
-            icon->version = nid.u.uVersion;
+            icon->version = nid.uVersion;
             ret = TRUE;
         }
         break;
     default:
-        WINE_FIXME("unhandled tray message: %ld\n", cds->dwData);
+        FIXME( "unhandled tray message: %Id\n", cds->dwData );
         break;
     }
 
+done:
     if (nid.hIcon) DestroyIcon( nid.hIcon );
     sync_taskbar_buttons();
     return ret;
@@ -608,7 +888,7 @@ static void add_taskbar_button( HWND hwnd )
 {
     struct taskbar_button *win;
 
-    if (hide_systray) return;
+    if (!enable_taskbar || !show_systray) return;
 
     /* ignore our own windows */
     if (hwnd)
@@ -617,7 +897,7 @@ static void add_taskbar_button( HWND hwnd )
         if (!GetWindowThreadProcessId( hwnd, &process ) || process == GetCurrentProcessId()) return;
     }
 
-    if (!(win = HeapAlloc( GetProcessHeap(), 0, sizeof(*win) ))) return;
+    if (!(win = malloc( sizeof(*win) ))) return;
     win->hwnd = hwnd;
     win->button = CreateWindowW( WC_BUTTONW, NULL, WS_CHILD | BS_OWNERDRAW,
                                  0, 0, 0, 0, tray_window, (HMENU)hwnd, 0, 0 );
@@ -641,7 +921,7 @@ static void remove_taskbar_button( HWND hwnd )
     if (!win) return;
     list_remove( &win->entry );
     DestroyWindow( win->button );
-    HeapFree( GetProcessHeap(), 0, win );
+    free( win );
 }
 
 static void paint_taskbar_button( const DRAWITEMSTRUCT *dis )
@@ -713,33 +993,7 @@ static void show_taskbar_contextmenu( HWND button, LPARAM lparam )
 
 static void do_hide_systray(void)
 {
-    SetWindowPos( tray_window, 0,
-                  GetSystemMetrics(SM_XVIRTUALSCREEN) + GetSystemMetrics(SM_CXVIRTUALSCREEN),
-                  GetSystemMetrics(SM_YVIRTUALSCREEN) + GetSystemMetrics(SM_CYVIRTUALSCREEN),
-                  0, 0, SWP_NOSIZE | SWP_NOZORDER | SWP_NOACTIVATE );
-}
-
-static BOOL notify_owner( struct icon *icon, UINT msg, POINT pt )
-{
-    WPARAM wp = icon->id;
-    LPARAM lp = msg;
-
-    if (icon->version >= NOTIFY_VERSION_4)
-    {
-        ClientToScreen( tray_window, &pt );
-        wp = MAKEWPARAM( pt.x, pt.y );
-        lp = MAKELPARAM( msg, icon->id );
-    }
-
-    TRACE( "relaying 0x%x\n", msg );
-    if (!PostMessageW( icon->owner, icon->callback_message, wp, lp ) &&
-        (GetLastError() == ERROR_INVALID_WINDOW_HANDLE))
-    {
-        WARN( "application window was destroyed, removing icon %u\n", icon->id );
-        delete_icon( icon );
-        return FALSE;
-    }
-    return TRUE;
+    ShowWindow( tray_window, SW_HIDE );
 }
 
 static void do_show_systray(void)
@@ -747,7 +1001,16 @@ static void do_show_systray(void)
     SIZE size;
     NONCLIENTMETRICSW ncm;
     HFONT font;
-    HDC hdc = GetDC( 0 );
+    HDC hdc;
+
+    if (!enable_taskbar)
+    {
+        size = get_window_size();
+        SetWindowPos( tray_window, 0, 0, 0, size.cx, size.cy, SWP_NOMOVE | SWP_NOACTIVATE | SWP_NOZORDER | SWP_SHOWWINDOW );
+        return;
+    }
+
+    hdc = GetDC( 0 );
 
     ncm.cbSize = sizeof(NONCLIENTMETRICSW);
     SystemParametersInfoW( SPI_GETNONCLIENTMETRICS, sizeof(NONCLIENTMETRICSW), &ncm, 0 );
@@ -771,7 +1034,7 @@ static void do_show_systray(void)
     sync_taskbar_buttons();
 }
 
-static LRESULT WINAPI tray_wndproc( HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam)
+static LRESULT WINAPI shell_traywnd_proc( HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam )
 {
     switch (msg)
     {
@@ -779,79 +1042,20 @@ static LRESULT WINAPI tray_wndproc( HWND hwnd, UINT msg, WPARAM wparam, LPARAM l
         return handle_incoming((HWND)wparam, (COPYDATASTRUCT *)lparam);
 
     case WM_DISPLAYCHANGE:
-        if (hide_systray || (!nb_displayed && !enable_shell)) do_hide_systray();
+        if (!show_systray) do_hide_systray();
+        else if (!nb_displayed && !enable_shell) do_hide_systray();
         else do_show_systray();
         break;
 
-    case WM_TIMER:
-        switch (wparam)
-        {
-        case BALLOON_CREATE_TIMER: balloon_create_timer(); break;
-        case BALLOON_SHOW_TIMER:   balloon_timer(); break;
-        }
+    case WM_MOVE:
+        update_systray_balloon_position();
         break;
 
-    case WM_PAINT:
-        {
-            unsigned int i;
-            PAINTSTRUCT ps;
-            HDC hdc;
-
-            hdc = BeginPaint( hwnd, &ps );
-            for (i = 0; i < nb_displayed; i++)
-            {
-                RECT dummy, rect = get_icon_rect( displayed[i] );
-                if (IntersectRect( &dummy, &rect, &ps.rcPaint ))
-                    DrawIconEx( hdc, rect.left + ICON_BORDER, rect.top + ICON_BORDER, displayed[i]->image,
-                                icon_cx - 2*ICON_BORDER, icon_cy - 2*ICON_BORDER,
-                            0, 0, DI_DEFAULTSIZE|DI_NORMAL);
-            }
-            EndPaint( hwnd, &ps );
-            break;
-        }
-
-    case WM_MOUSEMOVE:
-    case WM_LBUTTONDOWN:
-    case WM_LBUTTONUP:
-    case WM_RBUTTONDOWN:
-    case WM_RBUTTONUP:
-    case WM_MBUTTONDOWN:
-    case WM_MBUTTONUP:
-    case WM_LBUTTONDBLCLK:
-    case WM_RBUTTONDBLCLK:
-    case WM_MBUTTONDBLCLK:
-        {
-            MSG message;
-            POINT pt = { (short)LOWORD(lparam), (short)HIWORD(lparam) };
-            struct icon *icon = icon_from_point( pt.x, pt.y );
-            if (!icon) break;
-
-            message.hwnd = hwnd;
-            message.message = msg;
-            message.wParam = wparam;
-            message.lParam = lparam;
-            SendMessageW( icon->tooltip, TTM_RELAYEVENT, 0, (LPARAM)&message );
-
-            if (!notify_owner( icon, msg, pt )) break;
-
-            if (icon->version > 0)
-            {
-                switch (msg)
-                {
-                case WM_RBUTTONUP:
-                    notify_owner( icon, WM_CONTEXTMENU, pt );
-                    break;
-                case WM_LBUTTONUP:
-                    notify_owner( icon, NIN_SELECT, pt );
-                    break;
-                }
-            }
-            break;
-        }
-
     case WM_CLOSE:
         /* don't destroy the tray window, just hide it */
         ShowWindow( hwnd, SW_HIDE );
+        hide_balloon( balloon_icon );
+        show_systray = FALSE;
         return 0;
 
     case WM_DRAWITEM:
@@ -859,7 +1063,15 @@ static LRESULT WINAPI tray_wndproc( HWND hwnd, UINT msg, WPARAM wparam, LPARAM l
         break;
 
     case WM_COMMAND:
-        if (HIWORD(wparam) == BN_CLICKED) click_taskbar_button( (HWND)lparam );
+        if (HIWORD(wparam) == BN_CLICKED)
+        {
+            if (LOWORD(wparam) == TRAY_MINIMIZE_ALL || LOWORD(wparam) == TRAY_MINIMIZE_ALL_UNDO)
+            {
+                FIXME( "Shell command %u is not supported.\n", LOWORD(wparam) );
+                break;
+            }
+            click_taskbar_button( (HWND)lparam );
+        }
         break;
 
     case WM_CONTEXTMENU:
@@ -873,6 +1085,24 @@ static LRESULT WINAPI tray_wndproc( HWND hwnd, UINT msg, WPARAM wparam, LPARAM l
     case WM_MENUCOMMAND:
         return menu_wndproc(hwnd, msg, wparam, lparam);
 
+    case WM_USER + 0:
+        update_systray_balloon_position();
+        return 0;
+
+    case WM_USER + 1:
+    {
+        struct icon *icon;
+
+        LIST_FOR_EACH_ENTRY( icon, &icon_list, struct icon, entry )
+        {
+            if (!icon->window) continue;
+            hide_icon( icon );
+            show_icon( icon );
+        }
+
+        return 0;
+    }
+
     default:
         return DefWindowProcW( hwnd, msg, wparam, lparam );
     }
@@ -896,40 +1126,54 @@ void handle_parent_notify( HWND hwnd, WPARAM wp )
 }
 
 /* this function creates the listener window */
-void initialize_systray( HMODULE graphics_driver, BOOL using_root, BOOL arg_enable_shell )
+void initialize_systray( BOOL using_root, BOOL arg_enable_shell, BOOL arg_show_systray )
 {
-    WNDCLASSEXW class;
-    static const WCHAR classname[] = {'S','h','e','l','l','_','T','r','a','y','W','n','d',0};
+    RECT work_rect, primary_rect, taskbar_rect;
 
-    if (using_root) wine_notify_icon = (void *)GetProcAddress( graphics_driver, "wine_notify_icon" );
+    shell_traywnd_class.hIcon = LoadIconW( 0, (const WCHAR *)IDI_WINLOGO );
+    shell_traywnd_class.hCursor = LoadCursorW( 0, (const WCHAR *)IDC_ARROW );
+    tray_icon_class.hIcon = shell_traywnd_class.hIcon;
+    tray_icon_class.hCursor = shell_traywnd_class.hCursor;
 
     icon_cx = GetSystemMetrics( SM_CXSMICON ) + 2*ICON_BORDER;
     icon_cy = GetSystemMetrics( SM_CYSMICON ) + 2*ICON_BORDER;
-    hide_systray = using_root;
+    show_systray = arg_show_systray;
     enable_shell = arg_enable_shell;
+    enable_taskbar = enable_shell || !using_root;
 
     /* register the systray listener window class */
-    ZeroMemory(&class, sizeof(class));
-    class.cbSize        = sizeof(class);
-    class.style         = CS_DBLCLKS | CS_HREDRAW;
-    class.lpfnWndProc   = tray_wndproc;
-    class.hInstance     = NULL;
-    class.hIcon         = LoadIconW(0, (LPCWSTR)IDI_WINLOGO);
-    class.hCursor       = LoadCursorW(0, (LPCWSTR)IDC_ARROW);
-    class.hbrBackground = (HBRUSH) COLOR_WINDOW;
-    class.lpszClassName = classname;
-
-    if (!RegisterClassExW(&class))
+    if (!RegisterClassExW( &shell_traywnd_class ))
     {
-        WINE_ERR("Could not register SysTray window class\n");
+        ERR( "Could not register SysTray window class\n" );
+        return;
+    }
+    if (!RegisterClassExW( &tray_icon_class ))
+    {
+        ERR( "Could not register Wine SysTray window classes\n" );
         return;
     }
 
-    tray_window = CreateWindowExW( WS_EX_NOACTIVATE, classname, NULL, WS_POPUP,
-                                   0, GetSystemMetrics( SM_CYSCREEN ), 0, 0, 0, 0, 0, 0 );
+    if (enable_taskbar)
+    {
+        SystemParametersInfoW( SPI_GETWORKAREA, 0, &work_rect, 0 );
+        SetRect( &primary_rect, 0, 0, GetSystemMetrics( SM_CXSCREEN ), GetSystemMetrics( SM_CYSCREEN ) );
+        SubtractRect( &taskbar_rect, &primary_rect, &work_rect );
+
+        tray_window = CreateWindowExW( WS_EX_NOACTIVATE, shell_traywnd_class.lpszClassName, NULL, WS_POPUP,
+                                       taskbar_rect.left, taskbar_rect.top, taskbar_rect.right - taskbar_rect.left,
+                                       taskbar_rect.bottom - taskbar_rect.top, 0, 0, 0, 0 );
+    }
+    else
+    {
+        SIZE size = get_window_size();
+        tray_window = CreateWindowExW( 0, shell_traywnd_class.lpszClassName, L"", WS_CAPTION | WS_SYSMENU,
+                                       CW_USEDEFAULT, CW_USEDEFAULT, size.cx, size.cy, 0, 0, 0, 0 );
+        NtUserMessageCall( tray_window, WINE_SYSTRAY_DOCK_INIT, 0, 0, NULL, NtUserSystemTrayCall, FALSE );
+    }
+
     if (!tray_window)
     {
-        WINE_ERR("Could not create tray window\n");
+        ERR( "Could not create tray window\n" );
         return;
     }
 
@@ -937,6 +1181,6 @@ void initialize_systray( HMODULE graphics_driver, BOOL using_root, BOOL arg_enab
 
     add_taskbar_button( 0 );
 
-    if (hide_systray) do_hide_systray();
-    else if (enable_shell) do_show_systray();
+    if (enable_taskbar) do_show_systray();
+    else do_hide_systray();
 }
