@@ -23,6 +23,7 @@
 #include <stdlib.h>
 #include <string.h>
 #include <stdio.h>
+#include <Zydis/Zydis.h>
 
 #include "debugger.h"
 #include "wine/debug.h"
@@ -114,8 +115,8 @@ BOOL memory_write_value(const struct dbg_lvalue* lvalue, DWORD size, void* value
     if (!types_get_info(&lvalue->type, TI_GET_LENGTH, &os)) return FALSE;
     if (size != os)
     {
-        dbg_printf("Size mismatch in memory_write_value, got %u from type while expecting %u\n",
-                   (DWORD)os, size);
+        dbg_printf("Size mismatch in memory_write_value, got %I64u from type while expecting %lu\n",
+                   os, size);
         return FALSE;
     }
 
@@ -211,7 +212,7 @@ void memory_examine(const struct dbg_lvalue *lvalue, int count, char format)
                 memory_report_invalid_addr(linear);
                 break;
             }
-            dbg_printf("{%08x-%04x-%04x-%02x%02x-%02x%02x%02x%02x%02x%02x}\n",
+            dbg_printf("{%08lx-%04x-%04x-%02x%02x-%02x%02x%02x%02x%02x%02x}\n",
                        guid.Data1, guid.Data2, guid.Data3,
                        guid.Data4[0], guid.Data4[1], guid.Data4[2], guid.Data4[3],
                        guid.Data4[4], guid.Data4[5], guid.Data4[6], guid.Data4[7]);
@@ -248,13 +249,13 @@ void memory_examine(const struct dbg_lvalue *lvalue, int count, char format)
     case 'd': DO_DUMP(unsigned int, 4, " %4.4d"); break;
     case 'w': DO_DUMP(unsigned short, 8, " %04x"); break;
     case 'a':
-        if (sizeof(DWORD_PTR) == 4)
+        if (ADDRSIZE == 4)
         {
-            DO_DUMP(DWORD_PTR, 4, " %8.8Ix");
+            DO_DUMP(DWORD, 4, " %8.8lx");
         }
         else
         {
-            DO_DUMP(DWORD_PTR, 2, " %16.16Ix");
+            DO_DUMP(DWORD64, 2, " %16.16I64x");
         }
         break;
     case 'c': DO_DUMP2(char, 32, " %c", (_v < 0x20) ? ' ' : _v); break;
@@ -386,11 +387,11 @@ BOOL memory_get_string(struct dbg_process* pcs, void* addr, BOOL in_debuggee,
         if (!unicode) ret = pcs->process_io->read(pcs->handle, addr, buffer, size, &sz);
         else
         {
-            buffW = HeapAlloc(GetProcessHeap(), 0, size * sizeof(WCHAR));
+            buffW = malloc(size * sizeof(WCHAR));
             ret = pcs->process_io->read(pcs->handle, addr, buffW, size * sizeof(WCHAR), &sz);
             WideCharToMultiByte(CP_ACP, 0, buffW, sz / sizeof(WCHAR), buffer, size,
                                 NULL, NULL);
-            HeapFree(GetProcessHeap(), 0, buffW);
+            free(buffW);
         }
         if (size) buffer[size-1] = 0;
         return ret;
@@ -418,11 +419,11 @@ BOOL memory_get_string_indirect(struct dbg_process* pcs, void* addr, BOOL unicod
             ret = pcs->process_io->read(pcs->handle, ad, buffer, size * sizeof(WCHAR), &sz) && sz != 0;
         else
         {
-            if ((buff = HeapAlloc(GetProcessHeap(), 0, size)))
+            if ((buff = malloc(size)))
             {
                 ret = pcs->process_io->read(pcs->handle, ad, buff, size, &sz) && sz != 0;
                 MultiByteToWideChar(CP_ACP, 0, buff, sz, buffer, size);
-                HeapFree(GetProcessHeap(), 0, buff);
+                free(buff);
             }
             else ret = FALSE;
         }
@@ -452,8 +453,7 @@ char* memory_offset_to_string(char *str, DWORD64 offset, unsigned mode)
     if (mode == 32)
         sprintf(str, "0x%08x", (unsigned int) offset);
     else
-        sprintf(str, "0x%08x%08x", (unsigned int)(offset >> 32),
-                (unsigned int)offset);
+        sprintf(str, "%#016I64x", offset);
 
     return str;
 }
@@ -501,7 +501,6 @@ static void print_typed_basic(const struct dbg_lvalue* lvalue)
         case btInt:
         case btLong:
             if (!memory_fetch_integer(lvalue, size, TRUE, &val_int)) return;
-            if (size == 1) goto print_char;
             dbg_print_hex(size, val_int);
             break;
         case btUInt:
@@ -514,24 +513,25 @@ static void print_typed_basic(const struct dbg_lvalue* lvalue)
             dbg_printf("%f", val_real);
             break;
         case btChar:
-        case btWChar:
-            /* sometimes WCHAR is defined as btChar with size = 2, so discrimate
-             * Ansi/Unicode based on size, not on basetype
-             */
             if (!memory_fetch_integer(lvalue, size, TRUE, &val_int)) return;
-        print_char:
-            if ((size == 1 && isprint((char)val_int)) ||
-                 (size == 2 && val_int < 127 && isprint((char)val_int)))
+            if (size == 1 && isprint((char)val_int))
                 dbg_printf("'%c'", (char)val_int);
             else
-                dbg_printf("%d", (int)val_int);
+                dbg_print_hex(size, val_int);
+            break;
+        case btWChar:
+            if (!memory_fetch_integer(lvalue, size, TRUE, &val_int)) return;
+            if (size == 2 && iswprint((WCHAR)val_int))
+                dbg_printf("L'%lc'", (WCHAR)val_int);
+            else
+                dbg_print_hex(size, val_int);
             break;
         case btBool:
             if (!memory_fetch_integer(lvalue, size, TRUE, &val_int)) return;
             dbg_printf("%s", val_int ? "true" : "false");
             break;
         default:
-            WINE_FIXME("Unsupported basetype %u\n", bt);
+            WINE_FIXME("Unsupported basetype %lu\n", bt);
             break;
         }
         break;
@@ -550,11 +550,11 @@ static void print_typed_basic(const struct dbg_lvalue* lvalue)
             char    buffer[1024];
 
             if (!val_ptr) dbg_printf("0x0");
-            else if (((bt == btChar || bt == btInt) && size64 == 1) || (bt == btUInt && size64 == 2))
+            else if ((bt == btChar && size64 == 1) || (bt == btWChar && size64 == 2))
             {
                 if (memory_get_string(dbg_curr_process, val_ptr, sub_lvalue.in_debuggee,
-                                      size64 == 2, buffer, sizeof(buffer)))
-                    dbg_printf("\"%s\"", buffer);
+                                      bt == btWChar, buffer, sizeof(buffer)))
+                    dbg_printf("%s\"%s\"", bt == btWChar ? "L" : "", buffer);
                 else
                     dbg_printf("*** invalid address %p ***", val_ptr);
                 break;
@@ -619,7 +619,7 @@ static void print_typed_basic(const struct dbg_lvalue* lvalue)
         }
         break;
     default:
-        WINE_FIXME("Unsupported tag %u\n", tag);
+        WINE_FIXME("Unsupported tag %lu\n", tag);
         break;
     }
 }
@@ -641,7 +641,6 @@ void print_basic(const struct dbg_lvalue* lvalue, char format)
     {
         unsigned size;
         dbg_lgint_t res = types_extract_as_lgint(lvalue, &size, NULL);
-        WCHAR wch;
 
         switch (format)
         {
@@ -658,10 +657,7 @@ void print_basic(const struct dbg_lvalue* lvalue, char format)
             return;
 
         case 'u':
-            wch = (WCHAR)(res & 0xFFFF);
-            dbg_printf("%d = '", wch);
-            dbg_outputW(&wch, 1);
-            dbg_printf("'");
+            dbg_printf("%d = '%lc'", (WCHAR)(res & 0xFFFF), (WCHAR)(res & 0xFFFF));
             return;
 
         case 'i':
@@ -701,12 +697,7 @@ void print_bare_address(const ADDRESS64* addr)
     }
 }
 
-/***********************************************************************
- *           print_address
- *
- * Print an 16- or 32-bit address, with the nearest symbol if any.
- */
-void print_address(const ADDRESS64* addr, BOOLEAN with_line)
+static void print_address_symbol(const ADDRESS64* addr, BOOL with_line, const char *sep)
 {
     char                buffer[sizeof(SYMBOL_INFO) + 256];
     SYMBOL_INFO*        si = (SYMBOL_INFO*)buffer;
@@ -715,23 +706,21 @@ void print_address(const ADDRESS64* addr, BOOLEAN with_line)
     DWORD               disp;
     IMAGEHLP_MODULE     im;
 
-    print_bare_address(addr);
-
     si->SizeOfStruct = sizeof(*si);
     si->MaxNameLen   = 256;
     im.SizeOfStruct  = 0;
     if (SymFromAddr(dbg_curr_process->handle, lin, &disp64, si) && disp64 < si->Size)
     {
-        dbg_printf(" %s", si->Name);
+        dbg_printf("%s %s", sep, si->Name);
         if (disp64) dbg_printf("+0x%I64x", disp64);
     }
     else
     {
         im.SizeOfStruct = sizeof(im);
         if (!SymGetModuleInfo(dbg_curr_process->handle, lin, &im)) return;
-        dbg_printf(" %s", im.ModuleName);
+        dbg_printf("%s %s", sep, im.ModuleName);
         if (lin > im.BaseOfImage)
-            dbg_printf("+0x%Ix", lin - im.BaseOfImage);
+            dbg_printf("+0x%Ix", lin - (DWORD_PTR)im.BaseOfImage);
     }
     if (with_line)
     {
@@ -739,7 +728,7 @@ void print_address(const ADDRESS64* addr, BOOLEAN with_line)
 
         il.SizeOfStruct = sizeof(il);
         if (SymGetLineFromAddr64(dbg_curr_process->handle, lin, &disp, &il))
-            dbg_printf(" [%s:%u]", il.FileName, il.LineNumber);
+            dbg_printf(" [%s:%lu]", il.FileName, il.LineNumber);
         if (im.SizeOfStruct == 0) /* don't display again module if address is in module+disp form */
         {
             im.SizeOfStruct = sizeof(im);
@@ -749,6 +738,89 @@ void print_address(const ADDRESS64* addr, BOOLEAN with_line)
     }
 }
 
+/***********************************************************************
+ *           print_address
+ *
+ * Print an 16- or 32-bit address, with the nearest symbol if any.
+ */
+void print_address(const ADDRESS64* addr, BOOLEAN with_line)
+{
+    print_bare_address(addr);
+    print_address_symbol(addr, with_line, "");
+}
+
+void memory_disasm_one_x86_insn(ADDRESS64 *addr, int display)
+{
+    ZydisDisassembledInstruction instr = { .runtime_address = addr->Offset };
+    ZydisDecoder decoder;
+    ZydisDecoderContext ctx;
+    unsigned char buffer[16];
+    SIZE_T len;
+    int i;
+
+    if (!dbg_curr_process->process_io->read( dbg_curr_process->handle, memory_to_linear_addr(addr),
+                                             buffer, sizeof(buffer), &len )) return;
+
+    switch (addr->Mode)
+    {
+    case AddrModeReal:
+    case AddrMode1616:
+        ZydisDecoderInit( &decoder, ZYDIS_MACHINE_MODE_LEGACY_16, ZYDIS_STACK_WIDTH_16 );
+        break;
+    default:
+        if (ADDRSIZE == 4)
+            ZydisDecoderInit( &decoder, ZYDIS_MACHINE_MODE_LEGACY_32, ZYDIS_STACK_WIDTH_32 );
+        else
+            ZydisDecoderInit( &decoder, ZYDIS_MACHINE_MODE_LONG_64, ZYDIS_STACK_WIDTH_64 );
+        break;
+    }
+    ZydisDecoderDecodeInstruction( &decoder, &ctx, buffer, len, &instr.info );
+    ZydisDecoderDecodeOperands( &decoder, &ctx, &instr.info,
+                                instr.operands, instr.info.operand_count );
+
+    if (display)
+    {
+        ZydisFormatter formatter;
+        ZydisFormatterInit( &formatter, ZYDIS_FORMATTER_STYLE_ATT );
+        ZydisFormatterSetProperty( &formatter, ZYDIS_FORMATTER_PROP_HEX_UPPERCASE, 0 );
+        ZydisFormatterFormatInstruction( &formatter, &instr.info, instr.operands,
+                                         instr.info.operand_count_visible, instr.text,
+                                         sizeof(instr.text), instr.runtime_address, NULL );
+        dbg_printf( "%s", instr.text );
+        for (i = 0; i < instr.info.operand_count_visible; i++)
+        {
+            ADDRESS64 a = { .Mode = AddrModeFlat };
+            ZyanU64 addr;
+
+            if (!ZYAN_SUCCESS(ZydisCalcAbsoluteAddress( &instr.info, &instr.operands[i],
+                                                        instr.runtime_address, &addr )))
+                continue;
+
+            if (instr.info.meta.branch_type == ZYDIS_BRANCH_TYPE_NEAR &&
+                instr.operands[i].type == ZYDIS_OPERAND_TYPE_MEMORY &&
+                instr.operands[i].mem.disp.has_displacement &&
+                instr.operands[i].mem.index == ZYDIS_REGISTER_NONE &&
+                (instr.operands[i].mem.base == ZYDIS_REGISTER_NONE ||
+                 instr.operands[i].mem.base == ZYDIS_REGISTER_EIP ||
+                 instr.operands[i].mem.base == ZYDIS_REGISTER_RIP))
+            {
+                unsigned char dest[8];
+                if (dbg_read_memory( (void *)(ULONG_PTR)addr, dest, ADDRSIZE ))
+                {
+                    dbg_printf( " -> " );
+                    a.Offset = ADDRSIZE == 4 ? *(DWORD *)dest : *(DWORD64 *)dest;
+                    print_address( &a, TRUE );
+                    break;
+                }
+            }
+            a.Offset = addr;
+            print_address_symbol( &a, TRUE, instr.info.operand_count_visible > 1 ? " ;" : "" );
+            break;
+        }
+    }
+    addr->Offset += instr.info.length;
+}
+
 BOOL memory_disasm_one_insn(ADDRESS64* addr)
 {
     char        ch;
@@ -788,7 +860,7 @@ void memory_disassemble(const struct dbg_lvalue* xstart,
         memory_disasm_one_insn(&last);
 }
 
-BOOL memory_get_register(DWORD regno, DWORD_PTR** value, char* buffer, int len)
+BOOL memory_get_register(DWORD regno, struct dbg_lvalue* lvalue, char* buffer, int len)
 {
     const struct dbg_internal_var*  div;
 
@@ -818,7 +890,7 @@ BOOL memory_get_register(DWORD regno, DWORD_PTR** value, char* buffer, int len)
     {
         if (div->val == regno)
         {
-            if (!stack_get_register_frame(div, value))
+            if (!stack_get_register_frame(div, lvalue))
             {
                 if (buffer) snprintf(buffer, len, "<register %s not accessible in this frame>", div->name);
                 return FALSE;
@@ -828,6 +900,6 @@ BOOL memory_get_register(DWORD regno, DWORD_PTR** value, char* buffer, int len)
             return TRUE;
         }
     }
-    if (buffer) snprintf(buffer, len, "<unknown register %u>", regno);
+    if (buffer) snprintf(buffer, len, "<unknown register %lu>", regno);
     return FALSE;
 }
