@@ -57,7 +57,6 @@ struct global_cursor
     int                  x;                /* cursor position */
     int                  y;
     rectangle_t          clip;             /* cursor clip rectangle */
-    unsigned int         clip_msg;         /* message to post for cursor clip changes */
     unsigned int         last_change;      /* time of last position change */
     user_handle_t        win;              /* window that contains the cursor */
 };
@@ -110,6 +109,8 @@ extern void queue_cleanup_window( struct thread *thread, user_handle_t win );
 extern int init_thread_queue( struct thread *thread );
 extern int attach_thread_input( struct thread *thread_from, struct thread *thread_to );
 extern void detach_thread_input( struct thread *thread_from );
+extern void set_clip_rectangle( struct desktop *desktop, const rectangle_t *rect,
+                                unsigned int flags, int reset );
 extern void post_message( user_handle_t win, unsigned int message,
                           lparam_t wparam, lparam_t lparam );
 extern void send_notify_message( user_handle_t win, unsigned int message,
@@ -132,9 +133,11 @@ extern rectangle_t *get_region_data( const struct region *region, data_size_t ma
 extern rectangle_t *get_region_data_and_free( struct region *region, data_size_t max_size,
                                               data_size_t *total_size );
 extern int is_region_empty( const struct region *region );
+extern int is_region_equal( const struct region *region1, const struct region *region2 );
 extern void get_region_extents( const struct region *region, rectangle_t *rect );
 extern void offset_region( struct region *region, int x, int y );
 extern void mirror_region( const rectangle_t *client_rect, struct region *region );
+extern void scale_region( struct region *region, unsigned int dpi_from, unsigned int dpi_to );
 extern struct region *copy_region( struct region *dst, const struct region *src );
 extern struct region *intersect_region( struct region *dst, const struct region *src1,
                                         const struct region *src2 );
@@ -153,7 +156,7 @@ extern struct process *get_top_window_owner( struct desktop *desktop );
 extern void get_top_window_rectangle( struct desktop *desktop, rectangle_t *rect );
 extern void post_desktop_message( struct desktop *desktop, unsigned int message,
                                   lparam_t wparam, lparam_t lparam );
-extern void destroy_window( struct window *win );
+extern void free_window_handle( struct window *win );
 extern void destroy_thread_windows( struct thread *thread );
 extern int is_child_window( user_handle_t parent, user_handle_t child );
 extern int is_valid_foreground_window( user_handle_t window );
@@ -174,6 +177,7 @@ extern struct window_class *grab_class( struct process *process, atom_t atom,
 extern void release_class( struct window_class *class );
 extern int is_desktop_class( struct window_class *class );
 extern int is_hwnd_message_class( struct window_class *class );
+extern int get_class_style( struct window_class *class );
 extern atom_t get_class_atom( struct window_class *class );
 extern client_ptr_t get_class_client_ptr( struct window_class *class );
 
@@ -182,11 +186,53 @@ extern client_ptr_t get_class_client_ptr( struct window_class *class );
 extern struct desktop *get_desktop_obj( struct process *process, obj_handle_t handle, unsigned int access );
 extern struct winstation *get_process_winstation( struct process *process, unsigned int access );
 extern struct desktop *get_thread_desktop( struct thread *thread, unsigned int access );
-extern void connect_process_winstation( struct process *process, struct thread *parent );
+extern void connect_process_winstation( struct process *process, struct thread *parent_thread,
+                                        struct process *parent_process );
 extern void set_process_default_desktop( struct process *process, struct desktop *desktop,
                                          obj_handle_t handle );
 extern void close_process_desktop( struct process *process );
-extern void close_thread_desktop( struct thread *thread );
+extern void set_thread_default_desktop( struct thread *thread, struct desktop *desktop, obj_handle_t handle );
+extern void release_thread_desktop( struct thread *thread, int close );
+
+/* checks if two rectangles are identical */
+static inline int is_rect_equal( const rectangle_t *rect1, const rectangle_t *rect2 )
+{
+    return (rect1->left == rect2->left && rect1->right == rect2->right &&
+            rect1->top == rect2->top && rect1->bottom == rect2->bottom);
+}
+
+static inline int is_rect_empty( const rectangle_t *rect )
+{
+    return (rect->left >= rect->right || rect->top >= rect->bottom);
+}
+
+static inline int point_in_rect( const rectangle_t *rect, int x, int y )
+{
+    return (x >= rect->left && x < rect->right && y >= rect->top && y < rect->bottom);
+}
+
+static inline int scale_dpi( int val, unsigned int dpi_from, unsigned int dpi_to )
+{
+    if (val >= 0) return (val * dpi_to + (dpi_from / 2)) / dpi_from;
+    return (val * dpi_to - (dpi_from / 2)) / dpi_from;
+}
+
+static inline void scale_dpi_rect( rectangle_t *rect, unsigned int dpi_from, unsigned int dpi_to )
+{
+    rect->left   = scale_dpi( rect->left, dpi_from, dpi_to );
+    rect->top    = scale_dpi( rect->top, dpi_from, dpi_to );
+    rect->right  = scale_dpi( rect->right, dpi_from, dpi_to );
+    rect->bottom = scale_dpi( rect->bottom, dpi_from, dpi_to );
+}
+
+/* offset the coordinates of a rectangle */
+static inline void offset_rect( rectangle_t *rect, int offset_x, int offset_y )
+{
+    rect->left   += offset_x;
+    rect->top    += offset_y;
+    rect->right  += offset_x;
+    rect->bottom += offset_y;
+}
 
 /* mirror a rectangle respective to the window client area */
 static inline void mirror_rect( const rectangle_t *client_rect, rectangle_t *rect )
@@ -204,7 +250,7 @@ static inline int intersect_rect( rectangle_t *dst, const rectangle_t *src1, con
     dst->top    = max( src1->top, src2->top );
     dst->right  = min( src1->right, src2->right );
     dst->bottom = min( src1->bottom, src2->bottom );
-    return (dst->left < dst->right && dst->top < dst->bottom);
+    return !is_rect_empty( dst );
 }
 
 /* validate a window handle and return the full handle */
