@@ -19,7 +19,6 @@
  */
 
 #include "config.h"
-#include "wine/port.h"
 
 #include <assert.h>
 #include <ctype.h>
@@ -27,91 +26,13 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
-#ifdef HAVE_UNISTD_H
-# include <unistd.h>
-#endif
-#ifdef HAVE_SYS_STAT_H
-# include <sys/stat.h>
-#endif
 
 #include "build.h"
 
-#if defined(_WIN32) && !defined(__CYGWIN__)
-# define PATH_SEPARATOR ';'
-#else
-# define PATH_SEPARATOR ':'
-#endif
-
-static struct strarray tmp_files;
-static struct strarray empty_strarray;
+const char *temp_dir = NULL;
+struct strarray temp_files = { 0 };
 static const char *output_file_source_name;
 
-static const struct
-{
-    const char *name;
-    enum target_cpu cpu;
-} cpu_names[] =
-{
-    { "i386",    CPU_x86 },
-    { "i486",    CPU_x86 },
-    { "i586",    CPU_x86 },
-    { "i686",    CPU_x86 },
-    { "i786",    CPU_x86 },
-    { "amd64",   CPU_x86_64 },
-    { "x86_64",  CPU_x86_64 },
-    { "powerpc", CPU_POWERPC },
-    { "arm",     CPU_ARM },
-    { "armv5",   CPU_ARM },
-    { "armv6",   CPU_ARM },
-    { "armv7",   CPU_ARM },
-    { "armv7a",  CPU_ARM },
-    { "arm64",   CPU_ARM64 },
-    { "aarch64", CPU_ARM64 },
-};
-
-/* atexit handler to clean tmp files */
-void cleanup_tmp_files(void)
-{
-    unsigned int i;
-    for (i = 0; i < tmp_files.count; i++) if (tmp_files.str[i]) unlink( tmp_files.str[i] );
-}
-
-
-void *xmalloc (size_t size)
-{
-    void *res;
-
-    res = malloc (size ? size : 1);
-    if (res == NULL)
-    {
-        fprintf (stderr, "Virtual memory exhausted.\n");
-        exit (1);
-    }
-    return res;
-}
-
-void *xrealloc (void *ptr, size_t size)
-{
-    void *res = realloc (ptr, size);
-    if (size && res == NULL)
-    {
-        fprintf (stderr, "Virtual memory exhausted.\n");
-        exit (1);
-    }
-    return res;
-}
-
-char *xstrdup( const char *str )
-{
-    char *res = strdup( str );
-    if (!res)
-    {
-        fprintf (stderr, "Virtual memory exhausted.\n");
-        exit (1);
-    }
-    return res;
-}
-
 char *strupper(char *s)
 {
     char *p;
@@ -119,88 +40,6 @@ char *strupper(char *s)
     return s;
 }
 
-int strendswith(const char* str, const char* end)
-{
-    int l = strlen(str);
-    int m = strlen(end);
-    return l >= m && strcmp(str + l - m, end) == 0;
-}
-
-char *strmake( const char* fmt, ... )
-{
-    int n;
-    size_t size = 100;
-    va_list ap;
-
-    for (;;)
-    {
-        char *p = xmalloc( size );
-        va_start( ap, fmt );
-	n = vsnprintf( p, size, fmt, ap );
-	va_end( ap );
-        if (n == -1) size *= 2;
-        else if ((size_t)n >= size) size = n + 1;
-        else return p;
-        free( p );
-    }
-}
-
-static struct strarray strarray_copy( struct strarray src )
-{
-    struct strarray array;
-    array.count = src.count;
-    array.max = src.max;
-    array.str = xmalloc( array.max * sizeof(*array.str) );
-    memcpy( array.str, src.str, array.count * sizeof(*array.str) );
-    return array;
-}
-
-static void strarray_add_one( struct strarray *array, const char *str )
-{
-    if (array->count == array->max)
-    {
-        array->max *= 2;
-        if (array->max < 16) array->max = 16;
-        array->str = xrealloc( array->str, array->max * sizeof(*array->str) );
-    }
-    array->str[array->count++] = str;
-}
-
-void strarray_add( struct strarray *array, ... )
-{
-    va_list valist;
-    const char *str;
-
-    va_start( valist, array );
-    while ((str = va_arg( valist, const char *))) strarray_add_one( array, str );
-    va_end( valist );
-}
-
-void strarray_addv( struct strarray *array, char * const *argv )
-{
-    while (*argv) strarray_add_one( array, *argv++ );
-}
-
-void strarray_addall( struct strarray *array, struct strarray args )
-{
-    unsigned int i;
-
-    for (i = 0; i < args.count; i++) strarray_add_one( array, args.str[i] );
-}
-
-struct strarray strarray_fromstring( const char *str, const char *delim )
-{
-    const char *tok;
-    struct strarray array = empty_strarray;
-    char *buf = xstrdup( str );
-
-    for (tok = strtok( buf, delim ); tok; tok = strtok( NULL, delim ))
-	strarray_add_one( &array, strdup( tok ));
-
-    free( buf );
-    return array;
-}
-
 void fatal_error( const char *msg, ... )
 {
     va_list valist;
@@ -288,20 +127,8 @@ static struct strarray get_tools_path(void)
 
     if (!done)
     {
-        dirs = strarray_copy( tools_path );
-
-        /* then append the PATH directories */
-        if (getenv( "PATH" ))
-        {
-            char *p = xstrdup( getenv( "PATH" ));
-            while (*p)
-            {
-                strarray_add_one( &dirs, p );
-                while (*p && *p != PATH_SEPARATOR) p++;
-                if (!*p) break;
-                *p++ = 0;
-            }
-        }
+        strarray_addall( &dirs, tools_path );
+        strarray_addall( &dirs, strarray_frompath( getenv( "PATH" )));
         done = 1;
     }
     return dirs;
@@ -342,17 +169,13 @@ static const char *find_binary( const char *prefix, const char *name )
 
 void spawn( struct strarray args )
 {
-    unsigned int i;
     int status;
     const char *argv0 = find_binary( NULL, args.str[0] );
 
     if (argv0) args.str[0] = argv0;
-    strarray_add_one( &args, NULL );
-    if (verbose)
-        for (i = 0; args.str[i]; i++)
-            fprintf( stderr, "%s%c", args.str[i], args.str[i+1] ? ' ' : '\n' );
+    if (verbose) strarray_trace( args );
 
-    if ((status = _spawnvp( _P_WAIT, args.str[0], args.str )))
+    if ((status = strarray_spawn( args )))
     {
 	if (status > 0) fatal_error( "%s failed with status %u\n", args.str[0], status );
 	else fatal_perror( "winebuild" );
@@ -360,9 +183,50 @@ void spawn( struct strarray args )
     }
 }
 
+static const char *find_clang_tool( struct strarray clang, const char *tool )
+{
+    const char *out = make_temp_file( "print_tool", ".out" );
+    struct strarray args = empty_strarray;
+    int sout = -1;
+    char *path, *p;
+    struct stat st;
+    size_t cnt;
+
+    strarray_addall( &args, clang );
+    strarray_add( &args, strmake( "-print-prog-name=%s", tool ));
+    if (verbose) strarray_add( &args, "-v" );
+
+    sout = dup( fileno(stdout) );
+    freopen( out, "w", stdout );
+    spawn( args );
+    if (sout >= 0)
+    {
+        dup2( sout, fileno(stdout) );
+        close( sout );
+    }
+
+    if (stat(out, &st) || !st.st_size) return NULL;
+
+    path = xmalloc(st.st_size + 1);
+    sout = open(out, O_RDONLY);
+    if (sout == -1) return NULL;
+    cnt = read(sout, path, st.st_size);
+    close(sout);
+    path[cnt] = 0;
+    if ((p = strchr(path, '\n'))) *p = 0;
+    /* clang returns passed command instead of full path if the tool could not be found */
+    if (!strcmp(path, tool))
+    {
+        free( path );
+        return NULL;
+    }
+    return path;
+}
+
 /* find a build tool in the path, trying the various names */
 struct strarray find_tool( const char *name, const char * const *names )
 {
+    struct strarray ret = empty_strarray;
     const char *file;
     const char *alt_names[2];
 
@@ -375,73 +239,113 @@ struct strarray find_tool( const char *name, const char * const *names )
 
     while (*names)
     {
-        if ((file = find_binary( target_alias, *names )))
-        {
-            struct strarray ret = empty_strarray;
-            strarray_add_one( &ret, file );
-            return ret;
-        }
+        if ((file = find_binary( target_alias, *names ))) break;
         names++;
     }
-    fatal_error( "cannot find the '%s' tool\n", name );
+
+    if (!file)
+    {
+        if (cc_command.count) file = find_clang_tool( cc_command, name );
+        if (!file && !(file = find_binary( "llvm", name )))
+        {
+            struct strarray clang = empty_strarray;
+            strarray_add( &clang, "clang" );
+            file = find_clang_tool( clang, strmake( "llvm-%s", name ));
+        }
+    }
+    if (!file) fatal_error( "cannot find the '%s' tool\n", name );
+
+    strarray_add( &ret, file );
+    return ret;
+}
+
+/* find a link tool in the path */
+struct strarray find_link_tool(void)
+{
+    struct strarray ret = empty_strarray;
+    const char *file = NULL;
+
+    if (cc_command.count) file = find_clang_tool( cc_command, "lld-link" );
+    if (!file) file = find_binary( NULL, "lld-link" );
+    if (!file)
+    {
+        struct strarray clang = empty_strarray;
+        strarray_add( &clang, "clang" );
+        file = find_clang_tool( clang, "lld-link" );
+    }
+
+    if (!file) fatal_error( "cannot find the 'lld-link' tool\n" );
+    strarray_add( &ret, file );
+    return ret;
 }
 
 struct strarray get_as_command(void)
 {
-    struct strarray args;
+    struct strarray args = empty_strarray;
+    const char *file;
     unsigned int i;
+    int using_cc = 0;
 
     if (cc_command.count)
     {
-        args = strarray_copy( cc_command );
-        strarray_add( &args, "-xassembler", "-c", NULL );
+        strarray_addall( &args, cc_command );
+        using_cc = 1;
+    }
+    else if (as_command.count)
+    {
+        strarray_addall( &args, as_command );
+    }
+    else if ((file = find_binary( target_alias, "as" )) || (file = find_binary( target_alias, "gas ")))
+    {
+        strarray_add( &args, file );
+    }
+    else if ((file = find_binary( NULL, "clang" )))
+    {
+        strarray_add( &args, file );
+        if (target_alias)
+        {
+            strarray_add( &args, "-target" );
+            strarray_add( &args, target_alias );
+        }
+        using_cc = 1;
+    }
+
+    if (using_cc)
+    {
+        strarray_add( &args, "-xassembler" );
+        strarray_add( &args, "-c" );
         if (force_pointer_size)
-            strarray_add_one( &args, (force_pointer_size == 8) ? "-m64" : "-m32" );
-        if (cpu_option) strarray_add_one( &args, strmake("-mcpu=%s", cpu_option) );
-        if (fpu_option) strarray_add_one( &args, strmake("-mfpu=%s", fpu_option) );
-        if (arch_option) strarray_add_one( &args, strmake("-march=%s", arch_option) );
+            strarray_add( &args, (force_pointer_size == 8) ? "-m64" : "-m32" );
+        if (cpu_option) strarray_add( &args, strmake("-mcpu=%s", cpu_option) );
+        if (fpu_option) strarray_add( &args, strmake("-mfpu=%s", fpu_option) );
+        if (arch_option) strarray_add( &args, strmake("-march=%s", arch_option) );
         for (i = 0; i < tools_path.count; i++)
-            strarray_add_one( &args, strmake("-B%s", tools_path.str[i] ));
+            strarray_add( &args, strmake("-B%s", tools_path.str[i] ));
         return args;
     }
 
-    if (!as_command.count)
-    {
-        static const char * const commands[] = { "gas", "as", NULL };
-        as_command = find_tool( "as", commands );
-    }
-
-    args = strarray_copy( as_command );
-
     if (force_pointer_size)
     {
-        switch (target_platform)
+        switch (target.platform)
         {
         case PLATFORM_APPLE:
-            strarray_add( &args, "-arch", (force_pointer_size == 8) ? "x86_64" : "i386", NULL );
+            strarray_add( &args, "-arch" );
+            strarray_add( &args, (force_pointer_size == 8) ? "x86_64" : "i386" );
             break;
         default:
-            switch(target_cpu)
-            {
-            case CPU_POWERPC:
-                strarray_add_one( &args, (force_pointer_size == 8) ? "-a64" : "-a32" );
-                break;
-            default:
-                strarray_add_one( &args, (force_pointer_size == 8) ? "--64" : "--32" );
-                break;
-            }
+            strarray_add( &args, (force_pointer_size == 8) ? "--64" : "--32" );
             break;
         }
     }
 
-    if (cpu_option) strarray_add_one( &args, strmake("-mcpu=%s", cpu_option) );
-    if (fpu_option) strarray_add_one( &args, strmake("-mfpu=%s", fpu_option) );
+    if (cpu_option) strarray_add( &args, strmake("-mcpu=%s", cpu_option) );
+    if (fpu_option) strarray_add( &args, strmake("-mfpu=%s", fpu_option) );
     return args;
 }
 
 struct strarray get_ld_command(void)
 {
-    struct strarray args;
+    struct strarray args = empty_strarray;
 
     if (!ld_command.count)
     {
@@ -449,37 +353,34 @@ struct strarray get_ld_command(void)
         ld_command = find_tool( "ld", commands );
     }
 
-    args = strarray_copy( ld_command );
+    strarray_addall( &args, ld_command );
 
     if (force_pointer_size)
     {
-        switch (target_platform)
+        switch (target.platform)
         {
         case PLATFORM_APPLE:
-            strarray_add( &args, "-arch", (force_pointer_size == 8) ? "x86_64" : "i386", NULL );
+            strarray_add( &args, "-arch" );
+            strarray_add( &args, (force_pointer_size == 8) ? "x86_64" : "i386" );
             break;
         case PLATFORM_FREEBSD:
-            strarray_add( &args, "-m", (force_pointer_size == 8) ? "elf_x86_64_fbsd" : "elf_i386_fbsd", NULL );
+            strarray_add( &args, "-m" );
+            strarray_add( &args, (force_pointer_size == 8) ? "elf_x86_64_fbsd" : "elf_i386_fbsd" );
             break;
+        case PLATFORM_MINGW:
         case PLATFORM_WINDOWS:
-            strarray_add( &args, "-m", (force_pointer_size == 8) ? "i386pep" : "i386pe", NULL );
+            strarray_add( &args, "-m" );
+            strarray_add( &args, (force_pointer_size == 8) ? "i386pep" : "i386pe" );
             break;
         default:
-            switch(target_cpu)
-            {
-            case CPU_POWERPC:
-                strarray_add( &args, "-m", (force_pointer_size == 8) ? "elf64ppc" : "elf32ppc", NULL );
-                break;
-            default:
-                strarray_add( &args, "-m", (force_pointer_size == 8) ? "elf_x86_64" : "elf_i386", NULL );
-                break;
-            }
+            strarray_add( &args, "-m" );
+            strarray_add( &args, (force_pointer_size == 8) ? "elf_x86_64" : "elf_i386" );
             break;
         }
     }
 
-    if (target_cpu == CPU_ARM && target_platform != PLATFORM_WINDOWS)
-        strarray_add( &args, "--no-wchar-size-warning", NULL );
+    if (target.cpu == CPU_ARM && !is_pe())
+        strarray_add( &args, "--no-wchar-size-warning" );
 
     return args;
 }
@@ -496,37 +397,6 @@ const char *get_nm_command(void)
     return nm_command.str[0];
 }
 
-/* get a name for a temp file, automatically cleaned up on exit */
-char *get_temp_file_name( const char *prefix, const char *suffix )
-{
-    char *name;
-    const char *ext, *basename;
-    int fd;
-
-    if (!prefix || !prefix[0]) prefix = "winebuild";
-    if (!suffix) suffix = "";
-    if ((basename = strrchr( prefix, '/' ))) basename++;
-    else basename = prefix;
-    if (!(ext = strchr( basename, '.' ))) ext = prefix + strlen(prefix);
-    name = xmalloc( sizeof("/tmp/") + (ext - prefix) + sizeof(".XXXXXX") + strlen(suffix) );
-    memcpy( name, prefix, ext - prefix );
-    strcpy( name + (ext - prefix), ".XXXXXX" );
-    strcat( name, suffix );
-
-    if ((fd = mkstemps( name, strlen(suffix) )) == -1)
-    {
-        strcpy( name, "/tmp/" );
-        memcpy( name + 5, basename, ext - basename );
-        strcpy( name + 5 + (ext - basename), ".XXXXXX" );
-        strcat( name, suffix );
-        if ((fd = mkstemps( name, strlen(suffix) )) == -1)
-            fatal_error( "could not generate a temp file\n" );
-    }
-
-    close( fd );
-    strarray_add_one( &tmp_files, name );
-    return name;
-}
 
 /*******************************************************************
  *         buffer management
@@ -543,49 +413,15 @@ unsigned char *output_buffer;
 size_t output_buffer_pos;
 size_t output_buffer_size;
 
-static void check_output_buffer_space( size_t size )
-{
-    if (output_buffer_pos + size >= output_buffer_size)
-    {
-        output_buffer_size = max( output_buffer_size * 2, output_buffer_pos + size );
-        output_buffer = xrealloc( output_buffer, output_buffer_size );
-    }
-}
-
 void init_input_buffer( const char *file )
 {
-    int fd;
-    struct stat st;
-    unsigned char *buffer;
-
-    if ((fd = open( file, O_RDONLY | O_BINARY )) == -1) fatal_perror( "Cannot open %s", file );
-    if ((fstat( fd, &st ) == -1)) fatal_perror( "Cannot stat %s", file );
-    if (!st.st_size) fatal_error( "%s is an empty file\n", file );
-    input_buffer = buffer = xmalloc( st.st_size );
-    if (read( fd, buffer, st.st_size ) != st.st_size) fatal_error( "Cannot read %s\n", file );
-    close( fd );
+    if (!(input_buffer = read_file( file, &input_buffer_size ))) fatal_perror( "Cannot read %s", file );
+    if (!input_buffer_size) fatal_error( "%s is an empty file\n", file );
     input_buffer_filename = xstrdup( file );
-    input_buffer_size = st.st_size;
     input_buffer_pos = 0;
     byte_swapped = 0;
 }
 
-void init_output_buffer(void)
-{
-    output_buffer_size = 1024;
-    output_buffer_pos = 0;
-    output_buffer = xmalloc( output_buffer_size );
-}
-
-void flush_output_buffer(void)
-{
-    open_output_file();
-    if (fwrite( output_buffer, 1, output_buffer_pos, output_file ) != output_buffer_pos)
-        fatal_error( "Error writing to %s\n", output_file_name );
-    close_output_file();
-    free( output_buffer );
-}
-
 unsigned char get_byte(void)
 {
     if (input_buffer_pos >= input_buffer_size)
@@ -618,46 +454,6 @@ unsigned int get_dword(void)
     return ret;
 }
 
-void put_data( const void *data, size_t size )
-{
-    check_output_buffer_space( size );
-    memcpy( output_buffer + output_buffer_pos, data, size );
-    output_buffer_pos += size;
-}
-
-void put_byte( unsigned char val )
-{
-    check_output_buffer_space( 1 );
-    output_buffer[output_buffer_pos++] = val;
-}
-
-void put_word( unsigned short val )
-{
-    if (byte_swapped) val = (val << 8) | (val >> 8);
-    put_data( &val, sizeof(val) );
-}
-
-void put_dword( unsigned int val )
-{
-    if (byte_swapped)
-        val = ((val << 24) | ((val << 8) & 0x00ff0000) | ((val >> 8) & 0x0000ff00) | (val >> 24));
-    put_data( &val, sizeof(val) );
-}
-
-void put_qword( unsigned int val )
-{
-    if (byte_swapped)
-    {
-        put_dword( 0 );
-        put_dword( val );
-    }
-    else
-    {
-        put_dword( val );
-        put_dword( 0 );
-    }
-}
-
 /* pointer-sized word */
 void put_pword( unsigned int val )
 {
@@ -665,16 +461,6 @@ void put_pword( unsigned int val )
     else put_dword( val );
 }
 
-void align_output( unsigned int align )
-{
-    size_t size = align - (output_buffer_pos % align);
-
-    if (size == align) return;
-    check_output_buffer_space( size );
-    memset( output_buffer + output_buffer_pos, 0, size );
-    output_buffer_pos += size;
-}
-
 /* output a standard header for generated files */
 void output_standard_file_header(void)
 {
@@ -683,6 +469,17 @@ void output_standard_file_header(void)
     else
         output( "/* File generated automatically; do not edit! */\n" );
     output( "/* This file can be copied, modified and distributed without restriction. */\n\n" );
+    if (safe_seh)
+    {
+        output( "\t.def    @feat.00\n\t.scl 3\n\t.type 0\n\t.endef\n" );
+        output( "\t.globl  @feat.00\n" );
+        output( ".set @feat.00, 1\n" );
+    }
+    if (thumb_mode)
+    {
+        output( "\t.syntax unified\n" );
+        output( "\t.thumb\n" );
+    }
 }
 
 /* dump a byte stream into the assembly code */
@@ -774,7 +571,7 @@ void close_output_file(void)
  */
 char *open_temp_output_file( const char *suffix )
 {
-    char *tmp_file = get_temp_file_name( output_file_name, suffix );
+    char *tmp_file = make_temp_file( output_file_name, suffix );
     if (!(output_file = fopen( tmp_file, "w" )))
         fatal_error( "Unable to create output file '%s'\n", tmp_file );
     return tmp_file;
@@ -791,7 +588,7 @@ int remove_stdcall_decoration( char *name )
 {
     char *p, *end = strrchr( name, '@' );
     if (!end || !end[1] || end == name) return -1;
-    if (target_cpu != CPU_x86) return -1;
+    if (target.cpu != CPU_i386) return -1;
     /* make sure all the rest is digits */
     for (p = end + 1; *p; p++) if (!isdigit(*p)) return -1;
     *end = 0;
@@ -807,7 +604,9 @@ int remove_stdcall_decoration( char *name )
 void assemble_file( const char *src_file, const char *obj_file )
 {
     struct strarray args = get_as_command();
-    strarray_add( &args, "-o", obj_file, src_file, NULL );
+    strarray_add( &args, "-o" );
+    strarray_add( &args, obj_file );
+    strarray_add( &args, src_file );
     spawn( args );
 }
 
@@ -826,14 +625,10 @@ DLLSPEC *alloc_dll_spec(void)
     spec->type               = SPEC_WIN32;
     spec->base               = MAX_ORDINALS;
     spec->characteristics    = IMAGE_FILE_EXECUTABLE_IMAGE;
-    spec->subsystem          = 0;
+    spec->subsystem          = IMAGE_SUBSYSTEM_WINDOWS_CUI;
     spec->subsystem_major    = 4;
     spec->subsystem_minor    = 0;
-    if (get_ptr_size() > 4)
-        spec->characteristics |= IMAGE_FILE_LARGE_ADDRESS_AWARE;
-    else
-        spec->characteristics |= IMAGE_FILE_32BIT_MACHINE;
-    spec->dll_characteristics = IMAGE_DLLCHARACTERISTICS_NX_COMPAT;
+    spec->dll_characteristics = IMAGE_DLLCHARACTERISTICS_NX_COMPAT | IMAGE_DLLCHARACTERISTICS_DYNAMIC_BASE;
     return spec;
 }
 
@@ -909,43 +704,43 @@ const char *get_stub_name( const ORDDEF *odp, const DLLSPEC *spec )
 }
 
 /* return the stdcall-decorated name for an entry point */
-const char *get_link_name( const ORDDEF *odp )
+const char *get_abi_name( const ORDDEF *odp, const char *name )
 {
     static char *buffer;
     char *ret;
 
-    if (target_cpu != CPU_x86) return odp->link_name;
+    if (target.cpu != CPU_i386) return name;
 
     switch (odp->type)
     {
     case TYPE_STDCALL:
-        if (target_platform == PLATFORM_WINDOWS)
+        if (is_pe())
         {
-            if (odp->flags & FLAG_THISCALL) return odp->link_name;
-            if (odp->flags & FLAG_FASTCALL) ret = strmake( "@%s@%u", odp->link_name, get_args_size( odp ));
-            else if (!kill_at) ret = strmake( "%s@%u", odp->link_name, get_args_size( odp ));
-            else return odp->link_name;
+            if (odp->flags & FLAG_THISCALL) return name;
+            if (odp->flags & FLAG_FASTCALL) ret = strmake( "@%s@%u", name, get_args_size( odp ));
+            else if (!kill_at) ret = strmake( "%s@%u", name, get_args_size( odp ));
+            else return name;
         }
         else
         {
-            if (odp->flags & FLAG_THISCALL) ret = strmake( "__thiscall_%s", odp->link_name );
-            else if (odp->flags & FLAG_FASTCALL) ret = strmake( "__fastcall_%s", odp->link_name );
-            else return odp->link_name;
+            if (odp->flags & FLAG_THISCALL) ret = strmake( "__thiscall_%s", name );
+            else if (odp->flags & FLAG_FASTCALL) ret = strmake( "__fastcall_%s", name );
+            else return name;
         }
         break;
 
     case TYPE_PASCAL:
-        if (target_platform == PLATFORM_WINDOWS && !kill_at)
+        if (is_pe() && !kill_at)
         {
             int args = get_args_size( odp );
             if (odp->flags & FLAG_REGISTER) args += get_ptr_size();  /* context argument */
-            ret = strmake( "%s@%u", odp->link_name, args );
+            ret = strmake( "%s@%u", name, args );
         }
-        else return odp->link_name;
+        else return name;
         break;
 
     default:
-        return odp->link_name;
+        return name;
     }
 
     free( buffer );
@@ -953,98 +748,31 @@ const char *get_link_name( const ORDDEF *odp )
     return ret;
 }
 
-/* parse a cpu name and return the corresponding value */
-int get_cpu_from_name( const char *name )
+const char *get_link_name( const ORDDEF *odp )
 {
-    unsigned int i;
-
-    for (i = 0; i < ARRAY_SIZE(cpu_names); i++)
-        if (!strcmp( cpu_names[i].name, name )) return cpu_names[i].cpu;
-    return -1;
+    return get_abi_name( odp, odp->link_name );
 }
 
-/*****************************************************************
- *  Function:    get_alignment
+/*******************************************************************
+ *         sort_func_list
  *
- *  Description:
- *    According to the info page for gas, the .align directive behaves
- * differently on different systems.  On some architectures, the
- * argument of a .align directive is the number of bytes to pad to, so
- * to align on an 8-byte boundary you'd say
- *     .align 8
- * On other systems, the argument is "the number of low-order zero bits
- * that the location counter must have after advancement."  So to
- * align on an 8-byte boundary you'd say
- *     .align 3
- *
- * The reason gas is written this way is that it's trying to mimic
- * native assemblers for the various architectures it runs on.  gas
- * provides other directives that work consistently across
- * architectures, but of course we want to work on all arches with or
- * without gas.  Hence this function.
- *
- *
- *  Parameters:
- *    align  --  the number of bytes to align to. Must be a power of 2.
+ * Sort a list of functions, removing duplicates.
  */
-unsigned int get_alignment(unsigned int align)
+int sort_func_list( ORDDEF **list, int count, int (*compare)(const void *, const void *) )
 {
-    unsigned int n;
+    int i, j;
 
-    assert( !(align & (align - 1)) );
-
-    switch(target_cpu)
-    {
-    case CPU_x86:
-    case CPU_x86_64:
-        if (target_platform != PLATFORM_APPLE) return align;
-        /* fall through */
-    case CPU_POWERPC:
-    case CPU_ARM:
-    case CPU_ARM64:
-        n = 0;
-        while ((1u << n) != align) n++;
-        return n;
-    }
-    /* unreached */
-    assert(0);
-    return 0;
+    if (!count) return 0;
+    qsort( list, count, sizeof(*list), compare );
+    for (i = j = 0; i < count; i++) if (compare( &list[j], &list[i] )) list[++j] = list[i];
+    return j + 1;
 }
 
+
 /* return the page size for the target CPU */
 unsigned int get_page_size(void)
 {
-    switch(target_cpu)
-    {
-    case CPU_x86:
-    case CPU_x86_64:
-    case CPU_POWERPC:
-    case CPU_ARM:
-        return 0x1000;
-    case CPU_ARM64:
-        return 0x10000;
-    }
-    /* unreached */
-    assert(0);
-    return 0;
-}
-
-/* return the size of a pointer on the target CPU */
-unsigned int get_ptr_size(void)
-{
-    switch(target_cpu)
-    {
-    case CPU_x86:
-    case CPU_POWERPC:
-    case CPU_ARM:
-        return 4;
-    case CPU_x86_64:
-    case CPU_ARM64:
-        return 8;
-    }
-    /* unreached */
-    assert(0);
-    return 0;
+    return 0x1000;  /* same on all platforms */
 }
 
 /* return the total size in bytes of the arguments on the stack */
@@ -1058,11 +786,12 @@ unsigned int get_args_size( const ORDDEF *odp )
         {
         case ARG_INT64:
         case ARG_DOUBLE:
+            if (target.cpu == CPU_ARM) size = (size + 7) & ~7;
             size += 8;
             break;
         case ARG_INT128:
             /* int128 is passed as pointer on x86_64 */
-            if (target_cpu != CPU_x86_64)
+            if (target.cpu != CPU_x86_64)
             {
                 size += 16;
                 break;
@@ -1081,10 +810,11 @@ const char *asm_name( const char *sym )
 {
     static char *buffer;
 
-    switch (target_platform)
+    switch (target.platform)
     {
+    case PLATFORM_MINGW:
     case PLATFORM_WINDOWS:
-        if (target_cpu != CPU_x86) return sym;
+        if (target.cpu != CPU_i386) return sym;
         if (sym[0] == '@') return sym;  /* fastcall */
         /* fall through */
     case PLATFORM_APPLE:
@@ -1097,42 +827,67 @@ const char *asm_name( const char *sym )
     }
 }
 
-/* return an assembly function declaration for a C function name */
-const char *func_declaration( const char *func )
+/* return the assembly name for an ARM64/ARM64EC function */
+const char *arm64_name( const char *sym )
 {
-    static char *buffer;
+    switch (target.platform)
+    {
+    case PLATFORM_MINGW:
+    case PLATFORM_WINDOWS:
+        if (target.cpu == CPU_ARM64EC) return strmake( "\"#%s\"", sym );
+        /* fall through */
+    default:
+        return asm_name( sym );
+    }
+}
 
-    switch (target_platform)
+/* return an assembly function declaration for a C function name */
+void output_function_header( const char *func, int global )
+{
+    const char *name = arm64_name( func );
+
+    output( "\t.text\n" );
+
+    switch (target.platform)
     {
     case PLATFORM_APPLE:
-        return "";
+        if (global) output( "\t.globl %s\n\t.private_extern %s\n", name, name );
+        break;
+    case PLATFORM_MINGW:
     case PLATFORM_WINDOWS:
-        free( buffer );
-        buffer = strmake( ".def %s; .scl 2; .type 32; .endef", asm_name(func) );
+        if (target.cpu == CPU_ARM64EC) output( ".section .text,\"xr\",discard,%s\n\t", name );
+        output( "\t.def %s\n\t.scl 2\n\t.type 32\n\t.endef\n", name );
+        if (global) output( "\t.globl %s\n", name );
+        if (thumb_mode) output( "\t.thumb_func\n" );
         break;
     default:
-        free( buffer );
-        switch(target_cpu)
+        switch (target.cpu)
         {
         case CPU_ARM:
+            output( "\t.type %s,%%function\n", name );
+            if (thumb_mode) output( "\t.thumb_func\n" );
+            break;
         case CPU_ARM64:
-            buffer = strmake( ".type %s,%%function", func );
+            output( "\t.type %s,%%function\n", name );
             break;
         default:
-            buffer = strmake( ".type %s,@function", func );
+            output( "\t.type %s,@function\n", name );
             break;
         }
+        if (global) output( "\t.globl %s\n\t.hidden %s\n", name, name );
         break;
     }
-    return buffer;
+    output( "\t.balign 4\n" );
+    output( "%s:\n", name );
 }
 
 /* output a size declaration for an assembly function */
 void output_function_size( const char *name )
 {
-    switch (target_platform)
+    switch (target.platform)
     {
     case PLATFORM_APPLE:
+    case PLATFORM_MINGW:
     case PLATFORM_WINDOWS:
         break;
     default:
@@ -1154,14 +909,28 @@ void output_cfi( const char *format, ... )
     va_end( valist );
 }
 
+/* output a .seh directive */
+void output_seh( const char *format, ... )
+{
+    va_list valist;
+
+    if (!is_pe()) return;
+    va_start( valist, format );
+    fputc( '\t', output_file );
+    vfprintf( output_file, format, valist );
+    fputc( '\n', output_file );
+    va_end( valist );
+}
+
 /* output an RVA pointer */
 void output_rva( const char *format, ... )
 {
     va_list valist;
 
     va_start( valist, format );
-    switch (target_platform)
+    switch (target.platform)
     {
+    case PLATFORM_MINGW:
     case PLATFORM_WINDOWS:
         output( "\t.rva " );
         vfprintf( output_file, format, valist );
@@ -1176,16 +945,49 @@ void output_rva( const char *format, ... )
     va_end( valist );
 }
 
+/* output an RVA pointer or ordinal for a function thunk */
+void output_thunk_rva( int ordinal, const char *format, ... )
+{
+    if (ordinal == -1)
+    {
+        va_list valist;
+
+        va_start( valist, format );
+        switch (target.platform)
+        {
+        case PLATFORM_MINGW:
+        case PLATFORM_WINDOWS:
+            output( "\t.rva " );
+            vfprintf( output_file, format, valist );
+            fputc( '\n', output_file );
+            if (get_ptr_size() == 8) output( "\t.long 0\n" );
+            break;
+        default:
+            output( "\t%s ", get_asm_ptr_keyword() );
+            vfprintf( output_file, format, valist );
+            output( " - .L__wine_spec_rva_base\n" );
+            break;
+        }
+        va_end( valist );
+    }
+    else
+    {
+        if (get_ptr_size() == 4) output( "\t.long 0x8000%04x\n", ordinal );
+        else output( "\t.quad 0x800000000000%04x\n", ordinal );
+    }
+}
+
 /* output the GNU note for non-exec stack */
 void output_gnu_stack_note(void)
 {
-    switch (target_platform)
+    switch (target.platform)
     {
+    case PLATFORM_MINGW:
     case PLATFORM_WINDOWS:
     case PLATFORM_APPLE:
         break;
     default:
-        switch(target_cpu)
+        switch (target.cpu)
         {
         case CPU_ARM:
         case CPU_ARM64:
@@ -1205,15 +1007,18 @@ const char *asm_globl( const char *func )
     static char *buffer;
 
     free( buffer );
-    switch (target_platform)
+    switch (target.platform)
     {
     case PLATFORM_APPLE:
         buffer = strmake( "\t.globl _%s\n\t.private_extern _%s\n_%s:", func, func, func );
         break;
+    case PLATFORM_MINGW:
     case PLATFORM_WINDOWS:
-        buffer = strmake( "\t.globl %s%s\n%s%s:", target_cpu == CPU_x86 ? "_" : "", func,
-                          target_cpu == CPU_x86 ? "_" : "", func );
+    {
+        const char *name = asm_name( func );
+        buffer = strmake( "\t.globl %s\n%s:", name, name );
         break;
+    }
     default:
         buffer = strmake( "\t.globl %s\n\t.hidden %s\n%s:", func, func, func );
         break;
@@ -1234,7 +1039,7 @@ const char *get_asm_ptr_keyword(void)
 
 const char *get_asm_string_keyword(void)
 {
-    switch (target_platform)
+    switch (target.platform)
     {
     case PLATFORM_APPLE:
         return ".asciz";
@@ -1245,9 +1050,10 @@ const char *get_asm_string_keyword(void)
 
 const char *get_asm_export_section(void)
 {
-    switch (target_platform)
+    switch (target.platform)
     {
     case PLATFORM_APPLE:   return ".data";
+    case PLATFORM_MINGW:
     case PLATFORM_WINDOWS: return ".section .edata";
     default:               return ".section .data";
     }
@@ -1255,7 +1061,7 @@ const char *get_asm_export_section(void)
 
 const char *get_asm_rodata_section(void)
 {
-    switch (target_platform)
+    switch (target.platform)
     {
     case PLATFORM_APPLE: return ".const";
     default:             return ".section .rodata";
@@ -1264,9 +1070,10 @@ const char *get_asm_rodata_section(void)
 
 const char *get_asm_rsrc_section(void)
 {
-    switch (target_platform)
+    switch (target.platform)
     {
     case PLATFORM_APPLE:   return ".data";
+    case PLATFORM_MINGW:
     case PLATFORM_WINDOWS: return ".section .rsrc";
     default:               return ".section .data";
     }
@@ -1274,9 +1081,41 @@ const char *get_asm_rsrc_section(void)
 
 const char *get_asm_string_section(void)
 {
-    switch (target_platform)
+    switch (target.platform)
     {
     case PLATFORM_APPLE: return ".cstring";
     default:             return ".section .rodata";
     }
 }
+
+const char *arm64_page( const char *sym )
+{
+    static char *buffer;
+
+    switch (target.platform)
+    {
+    case PLATFORM_APPLE:
+        free( buffer );
+        buffer = strmake( "%s@PAGE", sym );
+        return buffer;
+    default:
+        return sym;
+    }
+}
+
+const char *arm64_pageoff( const char *sym )
+{
+    static char *buffer;
+
+    free( buffer );
+    switch (target.platform)
+    {
+    case PLATFORM_APPLE:
+        buffer = strmake( "%s@PAGEOFF", sym );
+        break;
+    default:
+        buffer = strmake( ":lo12:%s", sym );
+        break;
+    }
+    return buffer;
+}
